c     
c*******************************************************************************
c     
c     
c     A E R O S O L   A N D   T O X I C S   D E P O S I T I O N   M O D E L     
c     
c     W I T H                                    
c     
c     V A R I A B L E - N E S T E D   G R I D                   
c
c     Transport and chemistry modeling component of 
c
c     Regional Modeling System for Aerosols and Deposition
c     (REMSAD)
c     
c     
c     VERSION: 7.06                                  
c     June 19, 2003                               
c
c     Copyright 1996, 2001, 2002, 2003  Systems Applications International
c       The REMSAD modeling system software is the property of SAI 
c       and is protected by registered copyrights. Distribution of 
c       the software outside of your organization is not permitted. 
c       Preparation and distribution of any derivative software using 
c       any portions of the REMSAD code, without the express permission 
c       of SAI, is also not permitted. By accepting the REMSAD code 
c       provided in this release, you are also accepting the 
c       limitations, terms, and provisions of use as noted above.
c
c     
c     
c*******************************************************************************
c     
c     
c     Developed By 
c     Systems Applications International
c     101 Lucas Valley Road, Suite 230
c     San Rafael, California  94903
c     (415) 507-7108
c     
c     
c     Technical contact:  Tom Myers, SAI
c
c     SAI Contributors:
c       Version 7
c         Tom Myers
c         Gary Whitten
c         Ana Alvarez
c       Version 6
c         Tom Myers
c         Gary Whitten
c         Mohan Gupta
c       Version 5
c         Dongfen Gao
c         Gerard Mansell
c       Version 4
c         Gerard Mansell     
c         Gary Whitten
c       Prior versions
c         Chris Emery
c         Alex Kuklin
c         Mary Ligocki
c         Tom Myers
c     
c     Program code based on UAM-V, version 1.23, Revision 1.5, 95/02/14,
c     (c) Copr. 1992  Systems Applications International
c     
c*******************************************************************************
c
c-----V7.03, 8/23:
c     -Revised convergence criteria for fast species in step4
c     -Changed dimension of lmap in comchem to mxgas
c-----V7.02, 7/16:
c     -Updated mercury sulfite reaction rates. 
c     -Disabled elemental mercury dry dep. 
c-----V7.01, 5/10:
c     -Revised molecular weights and added temperature dependencies for
c      SOA species partitioning.  Added deposition of SOA precursors.
c-----V7.00, 5/02:
c     -Added direct production of SOA species from gas phase chemistry
c-----V6.40, 5/02:
c     -Changed minimum cloud liquid water value in sulfate
c     -Revised conversion of cloud liquid water to units of atm-l/mole
c     -Changed scavrat to use rain and cloud liquid water data as
c      read from file rather than calculate these from rainfall.
c-----V6.30, 3/02:
c     -Updated certain data assignments in aqbdat
c     -Updated some aqueous Hg chemical rates
c     -Moved time check for oh/o3 file in update.f to avoid
c      synchronization errors.
c-----95/08/24:
c     -Updated CPU time calculations
c-----95/09/05:
c     -modified all calls to FGRTRP to include the Ts flag and Ts array
c     (or dummy) in argument list
c     -removed calls to FGRTP for Ts, as this is now done in UPDATE
c-----95/09/18:
c     -restructured so that atmospheric mass field (P*, etc) is updated for
c     the current timestep after horizontal advection is solved
c     -alternate x/y advection each coarse grid timestep
c-----95/09/19:
c     -moved call to HGTNEW and HTFNEW to STEPXY and STEPXYF, respectively
c-----95/10/26:
c     -fixed check for mass summary diagnostics to print at each output time
c     rather than at the exact top of each hour
c-----95/11/08:
c     -added dpositf and wpositf to CRSFIL and NSTFIL calls
c     -added ifine index to SRLYRF, FVRTWD, STEPZF and FVRTDIF calls, and 
c     reduced their argument lists because of the inclusion of DEFFIN.CMD
c     
c-----96/09/05:
c     -Model frozen ... versn ATDM v1.00 09/05/96
c     
c-----97/02/25:
c     -Updated CPU time calculations
c     
c-----99/04/22: dfg
c     --Updated chemistry solver and added fast solver.
c     --Added option if OH will be print out.
c     --Initialization of oh fine grid conc array (OHCONF)
c     --Passign nfemit(ifine) to EFTOT routine instead of passing nospec
c     because the # of model species may be lesa the the # of emitted
c     species.
c     
c     
      include 'cpumar.cmd'
      include 'parfin.cmd'
      include 'parchm.cmd'
      include 'parm.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'aerosl.cmd'
      include 'ctlflg.cmd'
      include 'com3.cmd'
      include 'deffin.cmd'
      include 'shrfin.cmd'
      include 'com1f.cmd'
      include 'efcon.cmd'
      include 'balanc.cmd'
      include 'iemcon.cmd'
      include 'stack.cmd'
      include 'dtksub.cmd'
      include 'testdata.cmd'
      include 'kweight.cmd'
c     
      dimension tarray(2)
      logical endfl,datafl,prntfl, lrstrt
      integer aem
      character*80 fn
      character*24 versn
      dimension elapg(maxfin)
      logical lfdone
      logical lonhr
      logical updtafl
c     
      data nmxpas /1000/
      data maxpas /15/
      data versn /'ATDM v7.06, 06/19/03     '/
      data endfl,datafl,prntfl/3*.false./, mavg/2/
      data updtafl /.false./
c     
      write(*,3003) versn
 3003 format(' start model simulation ',a)
c     
c-----Open input files
c     control file read on unit 7, all other files opened in setup. 
c     
      itest = 0
      write(*,3010)
 3010 format(' open input files')
      call getarg(1,fn)
      open(unit=7,file=fn,form='formatted',status='old')
      write(*,'(a,a)') 'opening control file ',fn
      if (iargc() .gt. 1) then
         call getarg(2,fn)
         read(fn,'(i10)') idebug
      else
         idebug = 0
      endif
      idiag = and(idebug,1)
      iflxdb = and(idebug,16384)
      tmpinc=0.
c     
c-----Initialize the model
c     
      write(*,3012) 
 3012 format(' initialize model')
c     
c     Initialize coarse and fine grid deposition velocities to zero.
c     
      do 45 l=1,mxspec
         do 35 j=1,mxy
            do 25 i=1,mxx
               depvel(i,j,l) = 0.
 25         continue
 35      continue
 45   continue
c     
      do 57 ifine=1,maxfin
         do 55 l=1,mxspec
            do 53 ij=1,mxfxby
               depvelf(ij,l,ifine) = 0.
 53         continue
 55      continue
 57   continue
c     
      call setup(time,ndate,tend,idaynf,totime,tdatnx,lrstrt,idebug)
      updtafl=.true.
      call flush(66)
      call flush(6)
      call flush(8)
c     
c-----Set fine grid concentrations to coarse grid concs initially
c     and interpolate lat/lon and avterr of each cell to fine grids.
c     
      write (*,*) 'numfin =', numfin
c     
      if (numfin .eq. 0) nmxpas = 0
      do 350 ifine=1,numfin
c     
c     initilizing the radical for fine grids.
c     
         do jjrad=1,mxfx*mxfy*mxfz
            do iirad=1,mxrads
               radconf(iirad,ifine,jjrad)=1.0e-12
               if (iirad .eq. 10) then
                  radconf(iirad,ifine,jjrad)=1.0e-16
               end if
            end do
         end do
c     
c---- Set fine OH concentrations if necessary. dfg, 4/14/99
c     
         if (lmoh) then
            do i=1,mxfx
               do j=1,mxfy
                  do k=1,mxfz
                     ohconf(i,j,k,ifine)=1.0E-12
                     ho2conf(i,j,k,ifine)=1.0E-12
                     no3conf(i,j,k,ifine)=1.0E-12
                     aso2conf(i,j,k,ifine)=1.0E-12
                     pecconf(i,j,k,ifine)=1.0E-12
                  end do
               end do
            end do
         endif
c     
         if(.not.lrstrt) then
            write(*,*)'lrstrt= ',lrstrt,' ... calling finfil'
            call finfil(cncf(1,1,1,ifine),pstarf(1,ifine),nfx(ifine),
     &           nfy(ifine),nospec,nfz(ifine),ifine)
            call fmscal(nfx(ifine),nfy(ifine),nfz(ifine),delfx(1,ifine),
     &           delfy(ifine),sigmaf(1,ifine),pstarf(1,ifine),
     &           cncf(1,1,1,ifine),ifine)
            write(*,'(a,(1x,1pe10.3,5e10.3))') ' tot mass by spec:',
     &           (ftmas(l,ifine),l=1,nospec)
         endif
         inttyp = 20
         call fgrtrp(mxx,mxy,nox,noy,1,ctlon,
     &        nfx(ifine),nfy(ifine),1,
     &        ctlonf(1,ifine),sigmaf(1,ifine),
     &        ifine,inttyp,0,dummy,.false.,.false.,idebug)
         call fgrtrp(mxx,mxy,nox,noy,1,ctlat,
     &        nfx(ifine),nfy(ifine),1,
     &        ctlatf(1,ifine),sigmaf(1,ifine),
     &        ifine,inttyp,0,dummy,.false.,.false.,idebug)
         call fgrtrp(mxx,mxy,nox,noy,1,avterr,
     &        nfx(ifine),nfy(ifine),1,
     &        avterrf(1,ifine),sigmaf(1,ifine),
     &        ifine,inttyp,0,dummy,.false.,.false.,idebug)
 350  continue
c     
c-----Write initial concentrations to inst files
c     
      call prtint(time,ndate)
      write (*,*) 'after prtint'
c     
c-----Determine deposition velocities for all species
c     
      if (ldepn) then 
         call surlyr(time,ndate)
         if (idiag .ne. 0) then
            write(*,'(a)') ' finished surlyr'
         endif
c     
c     
c     Interpolate to get fine grid deposition velocities
c     
         do 120 ifine=1,numfin
            if (ifsrf(ifine) .le. 0) then
               inttyp = 20
               call fgrtrp(mxx,mxy,nox,noy,nospec,depvel,
     &              nfx(ifine),nfy(ifine),nospec,
     &              depvelf(1,1,ifine),sigmaf(1,ifine),
     &              ifine,inttyp,0,dummy,.false.,.false.,idebug)
c--------mxd 8/25 diag
c               write(*,'(a,10e11.4)')
c     &         'depvelf of HNO3 after fgrtrp in the 120 loop:',
c     &         (depvelf(i,10,1),i=31,40)
c
               if (idiag .ne. 0) then
                  write(*,'(a,i3)')' finished fgrtrp depvelf for fg#',ifine
               endif
            else
c     
c     Otherwise calculate deposition velocities for fine grid
c     
c     
               call srlyrf(time,ndate,nfx(ifine),nfy(ifine),nfz(ifine),
     &              nospec,ctlonf(1,ifine),ctlatf(1,ifine),
     &              cloudf(1,1,ifine),temprf(1,1,ifine),
     &              tfsurf(1,ifine),uf(1,1,ifine),vf(1,1,ifine),
     &              presf(1,1,ifine),htf(1,1,ifine),
     &              flandf(1,1,ifine),lstrsf,lsnowf,ch2of(1,1,ifine),
     &              rainf(1,ifine),psfcf(1,ifine),depvelf(1,1,ifine),
     &              ifine)
               if (idiag .ne. 0) then
                  write(*,'(a,i3)') ' finished srlyrf for fg#',ifine
               endif
c     
            endif
 120     continue
      endif
c     
c--------mxd 8/25 diag
c               write(*,'(a,10e11.4)')
c     &         'depvelf of HNO3 after the 120 loop in Atdm5:',
c     &         (depvelf(i,10,1),i=31,40)
c
c-----Determine length of coarse grid time step
c     
      call dtsize(time,ndate,deltat)
      if (idiag .ne. 0) then
         write(*,'(a)') ' finished dtsize'
      endif
c     
      dummy=etime(tarray)
      cpu=tarray(1)
      cpu1=cpu
      write(*,3013) cpu
 3013 format(' model initialized cpusecs=',f10.1)
      call flush(66)
      call flush(6)
      call flush(8)
c     
c-----Initialize average concentrations
c     
      call averag(1,deltat,time,ndate)
c     
c     Now for fine grids
c     
      do 5 ifine=1,numfin
         call avergf(1,nfx(ifine),nfy(ifine),nfz(ifine),nospec,
     &        presf,temprf,
     &        cncf(1,1,1,ifine),coldf(1,1,ifine),
     &        avconf(1,1,ifine),avgtmf(ifine),
     &        deltf(ifine),time,ndate)
    5 continue
c     
c-----Advance the model one time step in this loop. Adjust deltat so that
c     the model hits print-times, data-update-times and end-time exactly.
c     
      tprtnx=totime+dtprnt
      nts = 0
 10   continue
      nts = nts + 1
      if (idiag .ne. 0) then
         write(*,'(a)') ' at top of time step loop'
         call flush(6)
      endif
c     
C     idex = 88 
C     jdex = 11
      idex = 31 
      jdex = 67
      write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
      tstptm=0.0
      verttm=0.0
      bdrtm=0.0
      stpxtm=0.0
      stpztm=0.0
      vrttm=0.0
      rchtm=0.0
      avgtm=0.0
      updtm=0.0
      rdemtm=0.0
      xfintm=0.0
c     
      cpuold=cpu1
      dummy=etime(tarray)
      totnx=xtime(totime,deltat,1)
c     
c     Check the accumulated 24 hr time; should we print at the end of this
c     step?
c     
      if(totnx.lt.tprtnx-1.e-9) go to 20
      dtold=deltat
c     
c     Adjust time step so we hit print-time exactly instead of over-
c     shooting it.
c     
      deltat=xtime(tprtnx,totime,2)
      totnx=tprtnx
      prntfl=.true.
      mavg=3
c     
c     If we want to read new data as well as print, set the dataflag
c     
      if(abs(tdatnx-tprtnx).le.1.e-9) go to 30
c     
c     Check the accumulated 24 hr time; should we update the data at the
c     end of this step?  This section is skipped when tdatnx=tprtnx.
c     
 20   if(totnx.lt.tdatnx-1.e-9) go to 40
c     
c     Adjust the time step so we hit data-update-time exactly without
c     over shooting it.
c     
      deltat=xtime(tdatnx,totime,2)
      totnx=tdatnx
      prntfl=.false.
 30   datafl=.true.
 40   tnext=xtime(time,deltat,1)
      if(tnext.lt.2400.) go to 50
      tnext=tnext-2400.
      if (ndate .lt. 100000) then
         ndate = ndate + 1
      else
         ndate=ndayad(ndate)
      endif
 50   continue
c     
c     Check the time and date; should we complete the simulation at the
c     end of this time step?
c     
      if (ndate .lt. 100000) then
         jd1 = ndate
      else
         call caltoj(ndate,jd1)
      endif
      if (idaynf .lt. 100000) then
         jd2 = idaynf
      else
         call caltoj(idaynf,jd2)
      endif
      tchk1 = 2400.*(jd1-jd2) + tnext
      if (tchk1 .lt. tend-1.e-2) go to 60
      endfl=.true.
      datafl=.false.
      prntfl=.true.
      mavg=3
c     
c     Adjust time step so we hit the end of the simulation exactly.
c     
      deltat=deltat-xtime(tchk1,tend,2)
      tnext=tend
      ndate=idaynf
      totnx=xtime(totime,deltat,1)
 60   continue
c     
c-----Do mass budget calculations on output hours
c     
      if (mod(int(totime+.01),int(dtprnt)) .eq. 0) then
         lonhr = .true.
      else
         lonhr = .false.
      endif
      if (lonhr) then
         call mascal
         do 65 ifine=1,numfin
            call fmscal(nfx(ifine),nfy(ifine),nfz(ifine),delfx(1,ifine),
     &           delfy(ifine),sigmaf(1,ifine),pstarf(1,ifine),
     &           cncf(1,1,1,ifine),ifine)
            if (iflxdb .ne. 0) then
               call xmscal(mxx,mxy,mxz,mxspec,nox,noy,nlayer,
     &              deltax,deltay,conc,ifine)
               call xmsprt(time,ndate,ifine)
            endif
            call fmsprt(time,ndate,ifine)
 65      continue
      endif
      call masflo(time,ndate,deltat,lonhr)

      if (idiag .ne. 0) then
         call mascal
         write(*,'(a,(1x,1pe10.3,5e10.3))') ' tot mass by spec:',
     &        (tmas(l),l=1,nospec)
         call flush(6)
c     write(*,'(a,(1x,6e10.3))') ' Initial tot mass by spec:',
c     &      (tmasi(l),l=1,nospec)
      endif
c     
      timlft=deltat
      jtest = 0
      do 5001 it1=1,maxpas
         write(*,*) '  it1 ... timlft :',it1,timlft   
         write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
c     rewind (51)
c     rewind (ifinst)
c     do iturn = 1, 4
c     read (51)
c     end do
c     do iturn = 1, 3
c     read (ifinst)
c     end do
c     do iturn = 1, numfin
c     read (ifinst)
c     end do
c     call printr(time,ndate)
c     write (*,*) 'stop after printr'
c     stop
         if ( timlft .le. 0.001 ) goto 5011
         if ( dtsub(1) .le. timlft ) then
            dtk1=dtsub(1)
         else
            dtk1=timlft
         endif
         dummy=dtime(tarray)
c     
c-----Call hgtnew to update P* 
c     
         call hgtnew(dtk1)
         if (idiag .ne. 0) then
            write(*,'(a)') ' finished hgtnew'
            call flush(6)
         endif
c     
c-----Calculate coarse grid vertical velocity component (sigmadot) 
c     
         call vertwd(time,ndate,idebug)
         if (idiag .ne. 0) then
            write(*,'(a)') ' finished vertwnd'
         endif
c     
c-----Inject point and area emissions into the coarse grid
c     
         call emcors(time,ndate,dtk1,idebug)
         if (idiag .ne. 0) then
            write(*,'(a)') ' finished emcors'
            call mascal
            write(*,'(a,(1x,1pe10.3,5e10.3))') ' tot mass by spec:',
     &           (tmas(l),l=1,nospec)
         endif
         write(*,'(a)') ' finished emcors'
         write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
         dummy=dtime(tarray)
         verttm=verttm+tarray(1)
c     
c-----Fill boundary concentrations for fine grids from 
c     coarse grid concentration array and do mass budget calculations
c     
c         write (*,*) 'before 70'
         do 70 ifine=1,numfin
            call bdrfil(ifine,nfx(ifine),nfy(ifine),nfz(ifine),
     &           pstarf(1,ifine))
            if (idiag .ne. 0) then
               write(*,'(a,i4)') ' finished bdrfil for grid ',ifine
            endif
            call xmsflo(mxx,mxy,mxz,mxspec,nox,noy,nlayer,deltax,deltay,
     &           conc,u,v,dtk1,ifine)
 70      continue
c         write (*,*) 'after 70'
         dummy=dtime(tarray)
         bdrtm=bdrtm+tarray(1)
c     
c     
c     Take a half step in time for vrtdif,stepz,stepxy, then a full
c     time step for rchem then another half time step on stepxy,stepz,
c     vrtdif, in that order
c     
         timlft1=dtk1/2.
         do 5002 it2=1,maxpas
            write(*,*) '  it2 ... timlft1 :',it2,timlft1
            write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
            if ( timlft1 .le. 001 ) goto 5012
            if ( dtsub(2) .le. timlft1 ) then
               dtk2=dtsub(2)
            else
               dtk2=timlft1
            endif
c     
c-----Perform vertical diffusion
c     
            write (*,*) 'before vrtdif-1'
            call vrtdif(dtk2,idebug)
            write (*,*) 'after vrtdif-1'
            write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
            if (idiag .ne. 0) then
               write(*,'(a)') ' finished vrtdif'
               call mascal
               write(*,'(a,(1x,1pe10.3,5e10.3))') ' tot mass by spec:',
     &              (tmas(l),l=1,nospec)
            endif
            dummy=dtime(tarray)
            vrttm=vrttm+tarray(1)
c     
c-----Perform vertical advection
c     
            write (*,*) 'before stepz'
            call stepz(dtk2,idebug)
            write (*,*) 'after stepz'
            write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
            if (idiag .ne. 0) then
               write(*,'(a)') ' finished stepz'
               call mascal
               write(*,'(a,(1x,1pe10.3,5e10.3))') ' tot mass by spec:',
     &              (tmas(l),l=1,nospec)
            endif
            dummy=dtime(tarray)
            stpztm=stpztm+tarray(1)
c     
            timlft2=dtk2
            write (*,*) 'before 5003'
            do 5003 it3=1,maxpas
c     write(*,*) ' it3 ... timlft2 :',it3,timlft2
               if ( timlft2 .le. 0.001 ) goto 5013
               if ( dtsub(3) .le. timlft2 ) then
                  dtk3=dtsub(3)
               else
                  dtk3=timlft2
               endif
c     
c-----Perform horizontal advection
c     
               call stepx_04(dtk3,time,ndate)
               call stepy_04(dtk3,time,ndate)
               write(*,'(a)') ' finished stepxy'
            write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
               if (idiag .ne. 0) then
                  write(*,'(a)') ' finished stepxy'
                  call mascal
                  write(*,'(a,(1x,1pe10.3,5e10.3))') ' tot mass by spec:',
     &                 (tmas(l),l=1,nospec)
               endif
c     
c     do 400 k = 1,nlayer
c     do 401 l = 1,nospec
c     do 402 j = 1,noy
c     if (istr(j).ge.nox) goto 402
c     i1 = istr(j)
c     i2 = iend(j)
c     do 403 i = i1,i2
c     conc(i,j,l,k) = amax1(bdnl(l),conc(i,j,l,k))
c     403        continue
c     402      continue
c     401    continue
c     400  continue
c     
               write (*,*) 'before continue 5003'
               timlft2=timlft2-dtk3
 5003       continue
 5013       continue
            dummy=dtime(tarray)
            stpxtm=stpxtm+tarray(1)
            timlft1=timlft1-dtk2
 5002    continue
 5012    continue
         write (*,*) 'after 5012'
         write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
c     
         call flush(6)
c     
c***************************************************************
c     
c     FINE GRID SECTION  (will be bypassed if numfin is 0)
c     
c     write (*,*) 'species order'
c     write (*,*) 
c     *  kNO,        kNO2,       kO3,        kCARB,      kVOC,       kISOP,
c     *  kPAN,       kCO,        kHNO2,      kH2O2,      kHNO3,      kNTR,
c     *  kSO2,       kSULF,      kNH4S,      kPNO3,      kNH3,       kGSO4,
c     *  kASO4,      kNH4N
c     
         dummy=dtime(tarray)
         dummy=etime(tarray)
         cpuf1=tarray(1)
         if ( numfin .gt. 0 ) call xfine1(time,ndate,dtk1,
     &                                    mavg,updtafl,idebug)
         write (*,*) 'after fine grids'
         dummy=dtime(tarray)
         dummy=etime(tarray)
         xfintm=xfintm+tarray(1)-cpuf1
c     
c-----Perform chemistry on coarse grid 
c     
         call flush(66)
         call flush(8)
c     write (*,*) 'species order'
c     write (*,*) 
c     *  kNO,        kNO2,       kO3,        kCARB,      kVOC,       kISOP,
c     *  kPAN,       kCO,        kHNO2,      kH2O2,      kHNO3,      kNTR,
c     *  kSO2,       kSULF,      kNH4S,      kPNO3,      kNH3,       kGSO4,
c     *  kASO4,      kNH4N
c     
         if (lpmchm .or. ltxchm) then
            if (lmech) then
               itest = itest + 1
               call rchem2(dtk1,time,ndate,idebug)
            else
               call rchem(dtk1,time,ndate,idebug)
            endif
            write (*,*) 'cpumar =', cpumar
            write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
            if (idiag .ne. 0) then
               write(*,'(a)') ' finished rchem'
               call mascal
               write(*,'(a,(1x,1pe10.3,5e10.3))') ' tot mass by spec:',
     &              (tmas(l),l=1,nospec)
            endif
         endif
c     write (*,*) 'stopped in Atdm after rchem'
c     stop
         dummy=dtime(tarray)
         rchtm=rchtm+tarray(1)
c     
         timlft1=dtk1/2.
c     
         do 6002 it22=1,maxpas
c     write(*,*) ' it22 ... timlft1 :',it22,timlft1
            if ( timlft1 .le. 0.001 ) goto 6012
            if ( dtsub(2) .le. timlft1 ) then
               dtk2=dtsub(2)
            else
               dtk2=timlft1
            endif
c     
            timlft2=dtk2
            dummy=dtime(tarray)
            do 6003 it32=1,maxpas
c     write(*,*) ' it32 ... timlft2 :',it32,timlft2
               if ( timlft2 .le. 0.001 ) goto 6013
               if ( dtsub(3) .le. timlft2 ) then
                  dtk3=dtsub(3)
               else
                  dtk3=timlft2
               endif
c     
c-----Perform horizontal advection
c     
c-----Multiply conc by P* for advection routines
c     
c     do 450 k = 1,nlayer
c     do 451 l = 1,nospec
c     do 452 j = 1,noy
c     if (istr(j).ge.nox) goto 452
c     i1 = istr(j)
c     i2 = iend(j)
c     do 453 i = i1,i2
c     conc(i,j,l,k) = conc(i,j,l,k)*pstar(i,j)
c     453        continue
c     452      continue
c     451    continue
c     450  continue
c     
               write (*,*) 'before stepx'
               call stepx_04(dtk3,time,ndate)
               call stepy_04(dtk3,time,ndate)
            write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
               if (idiag .ne. 0) then
                  write(*,'(a)') ' finished stepxy'
                  call mascal
                  write(*,'(a,(1x,1pe10.3,5e10.3))') ' tot mass by spec:',
     &                 (tmas(l),l=1,nospec)
               endif
c     
c     do 460 k = 1,nlayer
c     do 461 l = 1,nospec
c     do 462 j = 1,noy
c     if (istr(j).ge.nox) goto 462
c     i1 = istr(j)
c     i2 = iend(j)
c     do 463 i = i1,i2
c     conc(i,j,l,k) = amax1(bdnl(l),conc(i,j,l,k))
c     463        continue
c     462      continue
c     461    continue
c     460  continue
c     
               write (*,*) 'before continue 6003', it32, maxpas
               timlft2=timlft2-dtk3
c     
 6003       continue
            write (*,*) 'after 6003'
 6013       continue
            write (*,*) 'after 6013'
            dummy=dtime(tarray)
            stpxtm=stpxtm+tarray(1)
c     
c-----Perform vertical advection
c     
            write (*,*) 'before stepz'
            call stepz(dtk2,idebug)
            write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
            if (idiag .ne. 0) then
               write(*,'(a)') ' finished stepz'
               call mascal
               write(*,'(a,(1x,1pe10.3,5e10.3))') ' tot mass by spec:',
     &              (tmas(l),l=1,nospec)
            endif
            dummy=dtime(tarray)
            stpztm=stpztm+tarray(1)
c     
c-----Perform vertical diffusion
c     
            write (*,*) 'before vrtdif'
            call vrtdif(dtk2,idebug)
            write (*,*) 'after vrtdif'
            write(*,*) 'conc: ', (conc(idex,jdex,k,1),k=1,nospec)
            if (idiag .ne. 0) then
               write(*,'(a)') ' finished vrtdif'
               call mascal
               write(*,'(a,(1x,1pe10.3,5e10.3))') ' tot mass by spec:',
     &              (tmas(l),l=1,nospec)
            endif
            dummy=dtime(tarray)
            vrttm=vrttm+tarray(1)
c     
            timlft1=timlft1-dtk2
c     
 6002    continue
 6012    continue
c     
         timlft=timlft-dtk1
c     
 5001 continue
 5011 continue
c     
 1002 format (5(2x,i10),10(2x,f10.4))
 1003 format (100(1x,e10.4))
c     write (*,*) 'start write in atdm'
c     write (*,1002) testidate,testtime,avetime
c     write (*,1003) (conc(41,40,isp,1),isp=1,77)
c     write (*,*) 'end write in atdm'
c     
c-----Compute time average concentrations
c     
      time=tnext
      totime=totnx
      write(66,80) time,totime,deltat
 80   format(/10x,'time',f8.2,10x,'totime',f8.2,10x,'deltat',f8.2)
      dummy=dtime(tarray)
      call averag(mavg,deltat,time,ndate)
      dummy=dtime(tarray)
      avgtm=avgtm+tarray(1)
c     
c-----Update input data if necessary
c     
      if (datafl) then
         call update(time,ndate,tdatnx,totime,idebug)
c     
c-----Determine length of coarse grid time step
c     
         call dtsize(time,ndate,deltat)
         updtafl=.true.
c     
c-----Determine deposition velocities for all species
c     
         if (ldepn) then
            call surlyr(time,ndate)
            if (idiag .ne. 0) then
               write(*,'(a)') ' finished surlyr'
            endif
c     
c     Interpolate to get fine grid deposition velocities
c     
            do 180 ifine=1,numfin
               if (ifsrf(ifine) .le. 0) then
                  inttyp = 20
                  call fgrtrp(mxx,mxy,nox,noy,nospec,depvel,
     &                 nfx(ifine),nfy(ifine),nospec,
     &                 depvelf(1,1,ifine),sigmaf(1,ifine),
     &                 ifine,inttyp,0,dummy,.false.,.false.,idebug)
c--------mxd 8/25 diag
c               write(*,'(a,10e11.4)')
c     &         'depvelf of HNO3 after fgrtrp in the 180 loop:',
c     &         (depvelf(i,10,1),i=31,40)
c
                  if (idiag .ne. 0) then
                     write(*,'(a,i3)')
     &                    ' Finished fgrtrp depvelf for fg#',ifine
                  endif
               else
c     
c     Otherwise calculate deposition velocities for fine grid
c     
c     
                  call srlyrf(time,ndate,nfx(ifine),nfy(ifine),nfz(ifine),
     &                 nospec,ctlonf(1,ifine),ctlatf(1,ifine),
     &                 cloudf(1,1,ifine),temprf(1,1,ifine),
     &                 tfsurf(1,ifine),uf(1,1,ifine),vf(1,1,ifine),
     &                 presf(1,1,ifine),htf(1,1,ifine),
     &                 flandf(1,1,ifine),lstrsf, lsnowf,
     &                 ch2of(1,1,ifine),rainf(1,ifine),
     &                 psfcf(1,ifine),depvelf(1,1,ifine),ifine)
c     
                  if (idiag .ne. 0) then
                     write(*,'(a,i3)') ' Finished srlyrf for fg#',ifine
                  endif
c     
               endif
 180        continue
         endif
c     
c--------mxd 8/25 diag
c               write(*,'(a,10e11.4)')
c     &         'depvelf of HNO3 after the 180 loop in Atdm5:',
c     &         (depvelf(i,10,1),i=31,40)
c
         datafl=.false.
c     
      else
         if(prntfl) deltat =dtold
      endif
c     
      dummy = dtime(tarray)
      updtm=updtm+tarray(1)
c     
c-----If end of simulation, skip read of emissions and write to output files
c     
      if (endfl) go to 91
c     
c-----Prepare to read emissions files
c     
      if (larea) then
         kntaem = 0
         call gunit(aem,time,ndate)
         do 1901 ifine=1,numfin
            if (aem .eq. aem1) then 
               ifaem(ifine) = ifaem1(ifine)
            else if (aem .eq. aem2) then
               ifaem(ifine) = ifaem2(ifine)
            else if (aem .eq. aem3) then
               ifaem(ifine) = ifaem3(ifine)
            endif
 1901    continue
c     
c-----Read area emissions
c     
 190     continue
         call getqar(mxx,mxy,nox,noy,naemit,aem,time,ndate,
     &        emtim1,iemdy1,emtim2,iemdy2,kaemit,
     &        katol,kaxyl,qarea,iqeof)
c     write (*,*) 'iqeof -1 =', iqeof
         if (iqeof .ne. 0) go to 195
         do 1902 ifine=1,numfin
            if (ifaem(ifine) .gt. 0) then
               call getqar(nfx(ifine),nfy(ifine),
     &              nfx(ifine),nfy(ifine),nfemit(ifine),
     &              ifaem(ifine),time,ndate,
     &              emtm1f(ifine),iemd1f(ifine),emtm2f(ifine),
     &              iemd2f(ifine),kfemit(1,ifine), kaftol(ifine),
     &              kafxyl(ifine),qfarea(1,1,ifine),iqeof)
c     write (*,*) 'iqeof - 2 =', iqeof
               if (iqeof .ne. 0) go to 195
            endif
 1902    continue
         go to 199
c     
 195     continue
         kntaem = kntaem + 1
         if (kntaem .gt. 2) then
            write(*,'(a)') ' cannot synch emissions file'
            stop
         endif
         rewind aem
         call hedqar(nospec,nox,noy,aem,naemit,kaemit,katol,kaxyl)
         do 1951 ifine=1,numfin
            if (ifaem(ifine) .gt. 0) then
               rewind ifaem(ifine)
               call hedqar(nospec,nfx(ifine),nfy(ifine),ifaem(ifine),
     &              nfemit(ifine), kfemit(1,ifine),
     &              kaftol(ifine), kafxyl(ifine))
            endif
 1951    continue
         go to 190
      endif
 199  continue
c     
c-----Read point source emissions
c     
      kntpnt=0
 290  continue
      if (lpts) then
         call getpt(time,ndate,edf,nemit,iqeof)
         if (iqeof .gt. 0) then
            kntpnt = kntpnt + 1
            if (kntpnt .gt. 2) then
               write(*,'(a)') ' cannot synch ptsource file'
               stop
            endif
            rewind edf
            call hedpts(nospec, nox, noy, edf, nemit, kemit)
            go to 290
         endif
      endif
c     
      write (*,*) 'before 93'
c-----Total point and area sources
c     
      if(.not.prntfl) go to 100
      do 93 ll=1,naemit
         l=kaemit(ll)
         if (l.eq.0) goto 93
         tarea(l,1)=0.0
         do 95 j=1,noy
            do 94 i=1,nox
               tarea(l,1)=tarea(l,1)+qarea(i,j,ll)
 94         continue
 95      continue
 93   continue
      do 96 ll=1,nemit
         l=kemit(ll)
         if (l.eq.0) goto 96
         tarea(l,2)=0.0
         do 92 i=1,nopts
            tarea(l,2)=tarea(l,2)+ptemis(ll,i)
 92      continue
 96   continue
      write(8,3900) (tarea(i,1),i=1,nospec)
      write(8,3901) (tarea(i,2),i=1,nospec)
 3900 format('  tarea= ',/,10(1pe12.4))
 3901 format(' tpoint= ',/,10(1pe12.4))
 3300 format(' read area source data ',i10,f10.1,i10,i10)
c     
c-----Total area source emissions on fine grids
c     
      do 98 ifine=1,numfin
c     call eftot(nfx(ifine),nfy(ifine),nospec,
         call eftot(nfx(ifine),nfy(ifine),nfemit(ifine),
     &        qfarea(1,1,ifine),ifine,idebug)
 98   continue
c     
      write (*,*) 'end of 98'
c-----Write to output files
c     
 91   prntfl=.false.
      mavg=2
      rewind (51)
      do iturn = 1, 4
         read (51)
      end do
      if (ifinst .gt. 0) then
        rewind (ifinst)
        do iturn = 1, 3
           read (ifinst)
        end do
        do iturn = 1, numfin
           read (ifinst)
        end do
      endif
      write (*,*) 'before printr'
      call printr(time,ndate)
      write (*,*) 'after printr'
      tprtnx=totime+dtprnt
      if(mod(tprtnx,100.0) .gt. 59.9)then
         tprtnx=tprtnx-60.0+100.0
      endif
c     
      write (*,*) 'before average'
c     
c-----Reinitialize the averaging process for the next print interval
c     
      call averag(1,deltat,time,ndate)
c     
c-----Reinitialize fine grid average concs
c     
      write (*,*) 'before 105'
c     
      do 105 ifine=1,numfin
         call avergf(1,nfx(ifine),nfy(ifine),nfz(ifine),nospec,
     &        presf(1,1,ifine),temprf(1,1,ifine),
     &        cncf(1,1,1,ifine),coldf(1,1,ifine),
     &        avconf(1,1,ifine),avgtmf(ifine),
     &        deltf(ifine),time,ndate)
 105  continue
c     
c-----End of simulation? 
c     
 100  continue
      dummy=dtime(tarray)
      rdemtm=rdemtm+tarray(1)
      dummy=etime(tarray)
      cpu10=tarray(1)
      elapt=cpu10-cpuold
      write(*,3301) time,ndate,deltat,cpu10,elapt,verttm,bdrtm,stpxtm,
     &     stpztm,vrttm,rchtm,avgtm,updtm,rdemtm
      write(66,3301) time,ndate,deltat,cpu10,elapt,verttm,bdrtm,stpxtm,
     &     stpztm,vrttm,rchtm,avgtm,updtm,rdemtm
      if (time .gt. 500.0) then
         if (ndate .eq. 980101) then
            if (deltat .eq. 30.0) then
c     stop
            end if
         end if
      end if
 3301 format(' End dt:',f7.1,i7,f7.2,/,
     &     ' Total elapsed time=',f15.1,' This time step=',f7.2,/,
     &     ' vertwd/emcors/hgtnew ',f7.2,/,
     &     ' bdrfil               ',f7.2,/,
     &     ' stepxy               ',f7.2,/,
     &     ' stepz                ',f7.2,/,
     &     ' vrtdif               ',f7.2,/,
     &     ' rchem                ',f7.2,/,
     &     ' averag               ',f7.2,/,
     &     ' update/surlyr        ',f7.2,/,
     &     ' rd ems/printr/averg  ',f7.2)
      if ( numfin .gt. 0 ) write(*,'(a,f7.2)') 'xfine   ',xfintm
      cpu1=cpu10
      call flush(66)
      call flush(8)
      if(.not.endfl) go to 10
c     
c-----If it is the end of the simulation, print last hour's and total 
c     cumulative mass fluxes
c     
      lonhr = .true.
      call mascal
      do 73 ifine=1,numfin
         call fmscal(nfx(ifine),nfy(ifine),nfz(ifine),delfx(1,ifine),
     &        delfy(ifine),sigmaf(1,ifine),pstarf(1,ifine),
     &        cncf(1,1,1,ifine),ifine)
         if (iflxdb .ne. 0) then
            call xmscal(mxx,mxy,mxz,mxspec,nox,noy,nlayer,
     &           deltax,deltay,conc,ifine)
            call xmsprt(time,ndate,ifine)
         endif
         call fmsprt(time,ndate,ifine)
 73   continue
      call masflo(time,ndate,deltat,lonhr)
c     
      do 75 ifine = 1, numfin
         if (iflxdb .ne. 0) then
            call xmscal(mxx,mxy,mxz,mxspec,nox,noy,nlayer,
     &           deltax,deltay,conc,ifine)
            call xcmprt(time,ndate,ifine)   
         endif
         call fcmprt(time,ndate,ifine)   
 75   continue
      call cumprt(time,ndate)   
c     
      stop
      end
