c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine averag(m,deltat,time,ndate)
c
c-----ATDM v1.00
c
c     Computes the average concentrations using
c     the rectangle rule for the time integrations.
c
c-----95/02/22:
c     -from UAM-V Revision 1.3  94/07/27  09:10:46  tcm
c     -removed code that keeps track of running hourly O3 averages
c-----95/04/05:
c     -altered running average calculation for ug/m3, rather than ppm
c
c     when  m=1 we begin the time average
c           m=2 we continue to generate the average
c           m=3 we conclude the averaging
c
c     Input arguments:
c               m:              averaging flag
c               deltat:         time step (min)
c               time:           model time
c               ndate:          model date
c     Files written:
c               6:              standard output file unit (*)     
c     Called by:
c               atdm 
c
      save
c
      include 'parm.cmd'
      include 'parfin.cmd'
      include 'parchm.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
      real mfac
c
      idex = 31
      jdex = 67
      write(*,4000) time,ndate,deltat,m
      if (m.eq.1) then
        avgtme=0.0
        do 10 k=1,nospc1
          do 9 j=1,noy
            do 8 i=1,nox
              avcong(i,j,k) = 0.0
    8       continue
    9     continue
   10   continue
      endif
c
c     add in cold arrays to avcon arrays
c
      if (m .gt. 1) then
        avgtme=avgtme+deltat
        do 115 k=1,nospc1
          do 110 j=1,noy
            if (istr(j).gt.nox) go to 110
            i1=istr(j) + 1
            i2=iend(j) - 1
            do 105 i=i1,i2
              if (j.le.jstr(i)) go to 105
              if (j.ge.jend(i)) go to 105
              avcong(i,j,k) = avcong(i,j,k) + 0.5*deltat*coldg(i,j,k)
              if (i .eq. idex .and. j .eq. jdex) then
                write(*,*) 'Inside 105 loop, avcong, deltat, coldg: '
                write(*,*) i,j,k,avcong(i,j,k), deltat, coldg(i,j,k)
              endif
  105       continue
  110     continue
  115   continue
c
c     add in current instantenous concentrations
c
        do 295 k=1,nospc1
          ksp = jind(k)
c         write(*,*) 'In averag, k, ksp, molwt(ksp): '
c         write(*,*) k, ksp, molwt(ksp)
          do 290 j=1,noy
            if (istr(j).gt.nox) go to 290
            i1=istr(j) + 1
            i2=iend(j) - 1
            do 285 i=i1,i2
              if (j.le.jstr(i)) go to 285
              if (j.ge.jend(i)) go to 285
              mfac = cpfac*(1013.25/press(i,j,1))*(temper(i,j,1)/273.)
              avcong(i,j,k) = avcong(i,j,k) + 0.5*deltat*molwt(ksp)*
     &                        conc(i,j,k,1)/mfac
              if (i .eq. idex .and. j .eq. jdex) then
                write(*,*) 'Inside 285 loop, avcong, deltat, molwt: '
                write(*,*) i,j,k,avcong(i,j,k), deltat, molwt(ksp)
                write(*,*) 'conc, mfac: '
                write(*,*) conc(i,j,k,1), mfac
              endif
  285       continue
  290     continue
  295   continue
      endif
c
c     close out averaging
c
      if (m.eq.3) then
        do 315 k=1,nospc1
          do 310 j=1,noy
            if (istr(j).gt.nox) go to 310
            i1=istr(j) + 1
            i2=iend(j) - 1
            do 305 i=i1,i2
              if (j.le.jstr(i)) go to 305
              if (j.ge.jend(i)) go to 305
              avcong(i,j,k) = avcong(i,j,k)/avgtme
              if (i .eq. idex .and. j .eq. jdex) then
                write(*,*) 'Inside 305 loop, avcong, avgtme: '
                write(*,*) i,j,k,avcong(i,j,k), avgtme
              endif
  305       continue
  310     continue
  315   continue
      endif
c
c     load conc into cold arrays for next time step
c
      do 350 k=1,nospc1
        ksp = jind(k)
c         write(*,*) 'In averag (2), k, ksp, molwt(ksp): '
c         write(*,*) k, ksp, molwt(ksp)
        do 340 j=1,noy
          if (istr(j).gt.nox) go to 340
          i1=istr(j) + 1
          i2=iend(j) - 1
          do 330 i=i1,i2
            if (j.le.jstr(i)) go to 330
            if (j.ge.jend(i)) go to 330
            mfac = cpfac*(1013.25/press(i,j,1))*(temper(i,j,1)/273.)
            coldg(i,j,k) = molwt(ksp)*conc(i,j,k,1)/mfac
            if (i .eq. idex .and. j .eq. jdex) then
            write(*,*) 'Inside 330 loop, coldg,molwt,conc,mfac: '
            write(*,*) i,j,k,coldg(i,j,k),molwt(ksp),conc(i,j,k,1),mfac
            endif
  330     continue
  340   continue
  350 continue
c
c     write(*,4100)
 4000 format(' ENTER AVERAG ',f10.1,i10,f10.2,'  m = ',i3)
 4100 format(' LEAVE AVERAG')
      return
      end
