c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine emfine(nxf,nyf,nzf,nfspec,nfspec1,delx,dely,sigfin,
     &                  qfarea,htf,pstrf,cncf,ifine,idebug)
c     subroutine emfine(nxf,nyf,nzf,nfspec,delx,dely,sigfin,
c    &                  qfarea,htf,pstrf,cncf,ifine,idebug)
c
c-----ATDM v1.00
c
c     Injects emissions into a fine grid 
c
c-----95/02/22:
c     -from UAM-V Revision 1.2  94/02/10  20:55:07  tcm
c     -modified code for new PTSRCX and PTSRCY variables (see hedpts)
c     -removed call to CONVRT
c-----95/06/15:
c     -emissions converted from mol/hr to ppm rather than to umol/m3
c     -redefined checks of source grid location with respect to
c      boundary cells (e.g., cheking i.le.istr instead of i.lt.istr)
c
c     Input arguments:
c               nxf:            number of fine grid cells in x-direction
c               nyf:            number of fine grid cells in y-direction
c               nzf:            number of fine grid layers
c               nfspec:         number of modeled species
c               delx:           cell size in x-direction (km)
c               dely:           cell size in y-direction (km)
c               sigfin:         fine grid sigma structure
c               qfarea:         array of fine grid area emissions (mol/min)
c               htf:            array of fine grid layer intrfce hghts (m)
c               pstrf:          array of fine grid P* (mb)
c               ifine:          fine grid index
c               idebug:         diagnostic debug flag
c               cncf:           fine grid concentration array (ppm)
c     Output arguments:
c               cncf:           fine grid concentration array (ppm)
c     Files written:
c               6:              standard output file unit (*) 
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
      include 'deffin.cmd'
      include 'efcon.cmd'
      include 'balanc.cmd'
      include 'stack.cmd'
      include 'latlon.cmd'
c
      dimension htf(nxf,nyf,nzf), cncf(nxf,nyf,nfspec1,nzf),
     &          qfarea(nxf,nyf,nfspec), pstrf(nxf,nyf),
     &          sigfin(nzf+1)
      dimension delx(nyf)
      real mfac
      data grav /9.8/, amolwt /28.8/
c
c  bit seven of idebug on (i.e., the sixty-fours location) will turn
c  on debug writes in emfine.
c
      idiag = and(idebug,64)
      if (idiag .ne. 0) then
        write(*,*) 'entering emfine dt = ',deltf(ifine)
        write(*,*) 'ifine = ',ifine
      endif
c
c     release area sources and point sources
c
      if (.not.lpts) goto 11
      do 10 l = 1,nopts
        i = int(ptsrcx(l)) + 1
        j = int(ptsrcy(l)) + 1
        if (i.gt.nox) i = nox
        if (i.lt.1) i = 1
        if (j.gt.noy) j = noy
        if (j.lt.1) j = 1
        if (i.le.istr(j)) goto 10
        if (i.ge.iend(j)) goto 10
        if (j.le.jstr(i)) goto 10
        if (j.ge.jend(i)) goto 10
        ixf = int((ptsrcx(l) - ixfb(ifine) + 1)*nhf(ifine) + 2)
        jyf = int((ptsrcy(l) - jyfb(ifine) + 1)*nhf(ifine) + 2)
        if (idiag .ne. 0) then
          write(*,*) 'nopts, l, i, j, ixf, jyf: ', nopts,
     &              l,i,j,ixf,jyf
          write(*,*) 'ptsrcx, ptsrcy:', ptsrcx(l), ptsrcy(l)
          write(*,*) 'ixfb, jyfb, nhf:',
     &               ixfb(ifine), jyfb(ifine), nhf(ifine)
        endif
        if (ixf .le. 1 .or. ixf .ge. nxf) go to 10
        if (jyf .le. 1 .or. jyf .ge. nyf) go to 10
c
        do 5 k = 1,nzf
          if (heff(l).gt.htf(ixf,jyf,k)) go to 5
c
c       put point source emissions in layer k
c
          dsigma = sigfin(k) - sigfin(k+1)
          vfac = deltf(ifine)/(1.e6*dely*delx(jyf)*dsigma)
          mfac = 10.*grav*amolwt/pstrf(ixf,jyf)
          do 4 lisp = 1,nemit
            isp = kemit(lisp)
            if (isp .le. 0) go to 4
            if (isp .gt. nospec) stop118
            if (idiag .ne. 0) then
              write(*,*) 'isp, k:  ', isp, k
              write(*,*) 'deltf:', deltf(ifine)
              write(*,*) ' ptemis, pstarf, dely, delx, dsigma:',
     &          ptemis(lisp,l), pstrf(ixf,jyf),dely,delx(jyf),
     &          dsigma
              write(*,*) 'cncf: ',cncf(ixf,jyf,isp,k)
            endif
c
            cncf(ixf,jyf,isp,k) = cncf(ixf,jyf,isp,k)+ptemis(lisp,l)*
     &                            vfac*mfac
            fptflo(isp,ifine) = fptflo(isp,ifine) +
     &                          ptemis(lisp,l)*deltf(ifine)
            if (isp .eq. kno2) then
              if (kno2s .gt. 0) then
                cncf(ixf,jyf,kno2s,k) = cncf(ixf,jyf,kno2s,k)
     &                                    +ptemis(lisp,l)*vfac*mfac
                fptflo(kno2s,ifine) = fptflo(kno2s,ifine) +
     &                                   ptemis(lisp,l)*deltf(ifine)
              endif
            endif
c
            if (idiag .ne. 0) then
              write(*,*) 'after, cncf: ',cncf(ixf,jyf,isp,k)
            endif
    4     continue
          go to 10
    5   continue
   10 continue
c
c     add in area emissions
c
   11 if (idiag .ne. 0) then
        write(*,*) ' about to do area sources, larea = ',larea
      endif
c
      if (larea) then
        if (idiag .ne. 0) then
          write(*,*) ' looping over grid, nxf, nyf = ',nxf, nyf
        endif
        do 30 j = 2,nyf-1
          do 25 i = 2,nxf-1
            dsigma = 1. - sigfin(2)
            vfac = deltf(ifine)/(1.e6*dely*delx(j)*dsigma)
            mfac = 10.*grav*amolwt/pstrf(i,j)
c
            do 20 lisp = 1,nfemit(ifine)
              isp = kfemit(lisp,ifine)
              if (isp .le. 0) go to 20
              if (tfarea(isp,ifine).le.0.0) go to 20
              if (idiag .ne. 0) then
                if (i .gt. 9 .and. i .lt. 13) then
                  if (j .gt. 9 .and. j .lt. 13) then
                    write(*,*) 'area source :  isp, lisp:  ', isp, lisp
                    write(*,*) 'cell:  ',i,j
                    write(*,*) 'deltf:', deltf(ifine)
                    write(*,*) ' qfarea, pstarf, dely, delx, htf:',
     &                  qfarea(i,j,lisp), pstrf(i,j),dely,delx(j),
     &                  htf(i,j,1)
                    write(*,*) 'cncf: ',cncf(i,j,isp,1)
                  endif
                endif
              endif
c
              cncf(i,j,isp,1) = cncf(i,j,isp,1)+qfarea(i,j,lisp)*
     &                          vfac*mfac
              feflo(isp,ifine) = feflo(isp,ifine) +
     &                           qfarea(i,j,lisp)*deltf(ifine)
              if (isp .eq. kno2) then
                if (kno2s .gt. 0) then
                  cncf(i,j,kno2s,1) = cncf(i,j,kno2s,1)
     &                                  +qfarea(i,j,lisp)*vfac*mfac
                  feflo(kno2s,ifine) = feflo(kno2s,ifine) +
     &                           qfarea(i,j,lisp)*deltf(ifine)
                endif
              endif
c
              if (idiag .ne. 0) then
                if (i .gt. 9 .and. i .lt. 13) then
                  if (j .gt. 9 .and. j .lt. 13) then
                    write(*,*) 'after, cncf: ',cncf(i,j,isp,1)
                  endif
                endif
              endif
   20       continue
   25     continue
   30   continue
      endif
c
      return
      end
