c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine fbdfil(ifget,nxfg,nyfg,nzfg,nspecf,pstrfg,cf1)
c
c-----ATDM v1.00
c
c     Fills in boundary concentrations for fine grids from another fine grid
c
c-----95/03/31:
c     -from UAM-V Revision 1.2  94/02/10  20:55:28  tcm
c     -modifications made following UAM-V to correct vertical nesting code
c-----95/06/28 (GEM):
c     -Replaced htg in calling arguments with pstrfg (=pstarf for ifget
c      grid) ... pass as pstarf(1,ifget)
c     -Replaced references to heights with sigma's  (in common)
c     -modified calculation of bcf for ppm and sigma
c     -Simplified some vertical nesting weighting factors since sigma
c      is constant in space & time
c     -Replaced call to arrget with call to getpstrfs to get pstarf
c      for the ifset grid for conversion to ppm
c     -added data statements for grav and amolwt
c-----95/06/29 (GEM):
c     -combined North & South boundaries and East & West boundaries
c      together in a single loop over iedg  (possible since call to
c      arrget has been removed ... gtpstrfs uses iedg argument to
c      determine which row/column to return)
c-----95/12/13:
c     -fixed goto statement at line 289 changed goto 35 to goto 235
c-----96/08/12:
c     -fixed pointer problem when recalculating values in ppm
c-----96/11/18:
c     -fixed calculation of BC's.  Conc's are adjusted by ratio of
c      P* on the two grids ... i.e., vfac=dsigma
c
c     Variables used in setting bcs:
c
c     igloc - fine grid number from which bcs are to be obtained,
c             has a value for each edge, and each cell on that 
c             edge, and each fine grid.
c
c     fhloc - a floating point value specifying the location in
c             the grid identified by igloc of the left (or bottom,
c             if this is east of west boundary) edge of the cell
c             in the current grid for which the bc is to be set.
c             Dimensioned the same as igloc.  This value is in 
c             grid units in the grid from which concs are being
c             obtained.  If a north or south boundary is being
c             set, this is an x location.  If an east or west
c             boundary is being set, this is a y location.
c
c     jhloc - The cell number in the grid from which concentrations
c             are being obtained.  If a north or south boundary
c             is being set, this is a row number.  If an east or
c             west boundary is being set, this is a column number.
c             dimensioned the same as igloc.
c
c     wkg   - Weighting factor for each layer to account for 
c             vertical nesting.
c 
c     igloc, fhloc, and jhloc are set in rvmmap.  wkg is calculated
c     within this routine. See detailed explanation of the use of these 
c     variables below for north boundary.  Use is similar for other 
c     boundaries.
c
c     Input arguments:
c               ifget:          fine grid index from which to get BCs    
c               nxfg:           number of cells in x-direction for grid ifget 
c               nyfg;           number of cells in y-direction for grid ifget 
c               nzfg:           number of layers for grid ifget
c               nspecf:         number of modeled species
c               pstrfg:         array of pstarf for ifget
c               cf1:            array of concentrations (ppm) for ifget
c     Routines called:
c               gtpstrfs
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'shrfin.cmd'
      include 'deffin.cmd'
      include 'com1f.cmd'
      include 'ngmap.cmd'
c
      dimension cf1(nxfg,nyfg,nspecf,nzfg)
      dimension pstrfg(nxfg,nyfg),pstrfs(mxfxy)
      dimension wkg(mxfz)
c
      real mfac,vfac
c
      data grav /9.8/, amolwt /28.8/
c
c  loop over all fine grids, skip the one from which we are getting
c  the concentrations to set boundaries on others
c
      do 400 ifset=1,numfin
        if (ifset .eq. ifget) go to 400
c
c  fill in north & south boundaries
c
c  we are currently setting bcs for ifset from the concentrations
c  in ifget.  Only those cells that need bcs from ifget will be
c  set on this call.
c
	do 200 iedg=1,2
c
        do 50 i=2,nfx(ifset)-1
c
c      ---if this boundary cell does not need concs from
c         ifget, skip setting bc for this cell.
c
          if (igloc(i,iedg,ifset) .ne. ifget) go to 50
c
c      ---hrat is the size of an ifset cell in units of ifget cells
c
          hrat = float(nhf(ifset))/float(nhf(ifget))
c
c      ---weight is used to do weighted average of cells in ifget
c         to get bc for ifset.  If weight is 1., then bc is set 
c         from one cell only.
c
          weight = amin1(1.,(1.-amod(fhloc(i,iedg,ifset),1.))*hrat)
c
c      ---ic, jc give cell location in ifget
c
          ic = int(fhloc(i,iedg,ifset)+.0005)+1
          jc = jhloc(i,iedg,ifset)
c
          do 40 n=1,nspecf
            kget = 1
c
c        ---if there are more layers in the grid being set
c           than in the grid from which we get concs, then
c           we just use same conc in each layer contained 
c           within the larger layer.
c
            if (nfz(ifset) .ge. nzfg) then
              do 30 k=1,nfz(ifset)
c
		dsigma=sigmaf(kget,ifget)-sigmaf(kget+1,ifget)
		vfac=dsigma
		mfac=pstrfg(ic,jc)/(10.0*grav*amolwt)
c
                bcf(i,iedg,n,k,ifset) = cf1(ic,jc,n,kget)*weight*
     &                                                    mfac*vfac
                carry = 1. - weight
                icp = ic
c
c            ---if the cell being set overlaps another cell of
c               the grid from which we get concs, move on to 
c               get the contribution of the next cell.
c
   25           icp = icp + 1
		mfac=pstrfg(icp,jc)/(10.0*grav*amolwt)
                if (carry .gt. 0.) then
                  w2 = amin1(hrat,carry)
                  bcf(i,iedg,n,k,ifset) = bcf(i,iedg,n,k,ifset) 
     &                          + cf1(icp,jc,n,kget)*w2*mfac*vfac
                  carry = carry - w2
                  go to 25
                endif
c
c            ---if sigma for grid to be set has reached the
c               sigma of the layer we are setting from , move
c               on to next layer
c
                if (abs(sigmaf(k+1,ifset)-sigmaf(kget+1,ifget)) .le.
     &                                        0.001 ) kget = kget + 1
c
   30         continue
c
            else
c
c          ---in this case, the grid we are setting bcs from has
c             more layers than the grid being set, so weighting
c             factors are calculated for each layer.  These compound
c             with the horizontal weighting.
c
              nkget = 0
              kg0 = 1
              k = 1
              bcf(i,iedg,n,k,ifset) = 0.
   35         nkget = nkget + 1
	      wkg(nkget) = (sigmaf(nkget,ifget)-sigmaf(nkget+1,ifget))/
     &                     (sigmaf(k,ifset)-sigmaf(k+1,ifset))
	      if ( abs(sigmaf(nkget+1,ifget)-sigmaf(k+1,ifset)) .gt.
     &                                     0.001 ) goto 35
c
              mfac=pstrfg(ic,jc)/(10.*grav*amolwt)
              do 36 kget=kg0,nkget
		dsigma=sigmaf(kget,ifget)-sigmaf(kget+1,ifget)
		vfac=dsigma
                bcf(i,iedg,n,k,ifset) = bcf(i,iedg,n,k,ifset) 
     &              + cf1(ic,jc,n,kget)*weight*wkg(kget)*mfac*vfac
   36         continue
              carry = 1. - weight
              icp = ic
   37         icp = icp + 1
c
              mfac=pstrfg(icp,jc)/(10.*grav*amolwt)
              if (carry .gt. 0.) then
                w2 = amin1(hrat,carry)
                do 39 kget=kg0,nkget
		  vfac=(sigmaf(kget,ifget)-sigmaf(kget+1,ifget))
                  bcf(i,iedg,n,k,ifset) = bcf(i,iedg,n,k,ifset) 
     &                + cf1(icp,jc,n,kget)*w2*wkg(kget)*mfac*vfac
   39           continue
                carry = carry - w2
                go to 37
              endif
c
c          ---bump things up for the next layer and go back
c             to get bc for that next layer.
c
              k = k + 1
              if (k .le. nfz(ifset)) then
                kg0 = nkget + 1
                bcf(i,iedg,n,k,ifset) = 0.
                go to 35
              endif
c
            endif
   40     continue
   50   continue
c
c-----Recalculate bcf as concentration (ppm) instead of mass
c
c  ---get vector of pstarf for this edge cell of the
c     grid to be set
c
        call gtpstrfs(mxfxy,nfx(ifset),nfy(ifset),iedg,
     &                pstarf(1,ifset),pstrfs)
c
        do 53 i=2,nfx(ifset)-1
c
          if (igloc(i,iedg,ifset) .ne. ifget) go to 53
c
	  do 52 k=1,nfz(ifset)
	    dsigma=sigmaf(k,ifset)-sigmaf(k+1,ifset)
c	    if ( iedg .eq. 1 ) then
c	      vfac=1.e6*delfx(nfy(ifset),ifset)*delfy(ifset)*dsigma
c	    else
c	      vfac=1.e6*delfx(1,ifset)*delfy(ifset)*dsigma
c	    endif
            vfac=dsigma
	    mfac=pstrfs(i)/(10.*grav*amolwt)
	    do 51 n=1,nspecf
	      bcf(i,iedg,n,k,ifset)=bcf(i,iedg,n,k,ifset)/(vfac*mfac)
   51       continue
   52     continue
   53   continue
c
  200   continue
c
c  fill in east & west boundaries
c
        do 300 iedg=3,4
c
        do 250 j=2,nfy(ifset)-1
          if (igloc(j,iedg,ifset) .ne. ifget) go to 250
          hrat = float(nhf(ifset))/float(nhf(ifget))
          weight = amin1(1.,(1.-amod(fhloc(j,iedg,ifset),1.))*hrat)
          ic = jhloc(j,iedg,ifset)
          jc = int(fhloc(j,iedg,ifset)+.0005)+1
          do 240 n=1,nspecf
            kget = 1
            if (nfz(ifset) .ge. nzfg) then
              do 230 k=1,nfz(ifset)
		dsigma=sigmaf(kget,ifget)-sigmaf(kget+1,ifget)
		vfac=dsigma
		mfac=pstrfg(ic,jc)/(10.0*grav*amolwt)
                bcf(j,iedg,n,k,ifset) = cf1(ic,jc,n,kget)*weight*
     &                                                    mfac*vfac
                carry = 1. - weight
                jcp = jc
  225           jcp = jcp + 1
		mfac=pstrfg(ic,jcp)/(10.0*grav*amolwt)
                if (carry .gt. 0.) then
                  w2 = amin1(hrat,carry)
                  bcf(j,iedg,n,k,ifset) = bcf(j,iedg,n,k,ifset) 
     &                          + cf1(ic,jcp,n,kget)*w2*mfac*vfac
                  carry = carry - w2
                  go to 225
                endif
c
                if (abs(sigmaf(k+1,ifset)-sigmaf(kget+1,ifget)) .le.
     &                                        0.001 ) kget = kget + 1
  230         continue
c
            else
              nkget = 0
              kg0 = 1
              k = 1
              bcf(j,iedg,n,k,ifset) = 0.
  235         nkget = nkget + 1
	      wkg(nkget) = (sigmaf(nkget,ifget)-sigmaf(nkget+1,ifget))/
     &                     (sigmaf(k,ifset)-sigmaf(k+1,ifset))
	      if ( abs(sigmaf(nkget+1,ifget)-sigmaf(k+1,ifset)) .gt.
     &                                     0.001 ) goto 235
c
              mfac=pstrfg(ic,jc)/(10.*grav*amolwt)
              do 236 kget=kg0,nkget
		dsigma=sigmaf(kget,ifget)-sigmaf(kget+1,ifget)
		vfac=dsigma
                bcf(j,iedg,n,k,ifset) = bcf(j,iedg,n,k,ifset) 
     &              + cf1(ic,jc,n,kget)*weight*wkg(kget)*mfac*vfac
  236         continue
              carry = 1. - weight
              jcp = jc
  237         jcp = jcp + 1
c
              mfac=pstrfg(ic,jcp)/(10.*grav*amolwt)
              if (carry .gt. 0.) then
                w2 = amin1(hrat,carry)
                do 239 kget=kg0,nkget
		  vfac=(sigmaf(kget,ifget)-sigmaf(kget+1,ifget))
                  bcf(j,iedg,n,k,ifset) = bcf(j,iedg,n,k,ifset) 
     &                + cf1(ic,jcp,n,kget)*w2*wkg(kget)*mfac*vfac
  239           continue
                carry = carry - w2
                go to 237
              endif
              k = k + 1
              if (k .le. nfz(ifset)) then
                kg0 = nkget + 1
                bcf(j,iedg,n,k,ifset) = 0.
                go to 235
              endif
c
            endif
  240     continue
  250   continue
c
c-----Recalculate bcf as concentration (ppm) instead of mass
c
c  ---get vector of pstarf for this edge cell of the
c     grid to be set
c
        call gtpstrfs(mxfxy,nfx(ifset),nfy(ifset),iedg,
     &                pstarf(1,ifset),pstrfs)
c
        do 253 j=2,nfy(ifset)-1
c
          if (igloc(j,iedg,ifset) .ne. ifget) go to 253
c
	  do 252 k=1,nfz(ifset)
	    dsigma=sigmaf(k,ifset)-sigmaf(k+1,ifset)
	    vfac=dsigma
	    mfac=pstrfs(j)/(10.*grav*amolwt)
	    do 251 n=1,nspecf
	      bcf(j,iedg,n,k,ifset)=bcf(j,iedg,n,k,ifset)/(vfac*mfac)
  251       continue
  252     continue
  253   continue
c
c
  300   continue
c
c    ---end of loop over fine grids
  400 continue
c
      return
      end
