c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine fstep(deltat,idebug)
c
c-----ATDM v1.00
c
c     Calculates maximum time step and number of time steps
c     to be used on fine grid in completing one of the
c     coarse grid size steps
c
c-----95/05/30:
c     -from UAM-V Revision 1.2  94/02/10  20:56:20  tcm
c     -renamed ifmdf to ifwind
c  
c     Variables calculated are:
c
c          ndftot - total number of time steps on fine grid
c                   to complete one coarse grid step
c          ndfmax - maximum number of steps to take in a
c                   pass (this will be updated in ATDM)
c          ndfcom - number of steps completed (initialized to
c                   zero here)
c          deltf  - time step size on fine grid
c
c     Input arguments:
c               deltat:         coarse grid time step (min)
c               idebug:         diagnostic debug flag
c     Files written:
c               6:              standard output file unit (*)
c     Routines called:
c               fgmax
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'com1f.cmd'
      include 'comuam.cmd'
      include 'ngmap.cmd'
      include 'dtksubf.cmd'
c
      real umaxi(mxfy,mxfz),vmaxj(mxfx,mxfz)
      real umaxk(mxfz),vmaxk(mxfz)
      real dtik(mxfy,mxfz),dtjk(mxfx,mxfz),dt1k(mxfz)
      real diff(mxfz),dt2k(mxfz)
c
c  Third bit of idebug (i.e., 4's place) will turn on diagnostic
c  writes in fstep
c
      idiag = and(idebug,4)
c
c   loop over all fine grids
c
      do 300 ifine=1,numfin
c
        niggle=1+(nhf(ifine)+1)/2
	deltf(ifine)=1.e-20
	dtmax=1.e-20
	dtmin=1.e20
c
        do k=1,nfz(ifine)
	  diff(k)=0.0
	  deltkf(k,ifine)=1.e20
          do i=2,nfx(ifine)-1
            vmaxj(i,k)=-999.
            dtjk(i,k)=1.0e20
          enddo
          do j=2,nfy(ifine)-1
            umaxi(j,k)=-999.
            dtik(j,k)=1.0e20
          enddo
        enddo
c
c  calculate maximum absolute x and y component of wind speed within
c  this fine grid, from coarse grid values if fine grid winds will
c  be interpolated or from fine grid if fine grid winds have already
c  been read in.
c
        call fgmax_04(nfx(ifine),nfy(ifine),nfz(ifine),
     &             uf(1,1,ifine),vf(1,1,ifine),umaxi,vmaxj)
c
c 	  do i=2,nfx(ifine)-1
c 	     write(*,*) 'i,vmaxj :',i,(vmaxj(i,k),k=1,nfz(ifine))
c 	  enddo
c 	  do j=2,nfy(ifine)-1
c 	     write(*,*) 'j,umaxi :',j,(umaxi(j,k),k=1,nfz(ifine))
c 	  enddo
c
        delxmn = 1.e20
        do 101 j=1,nfy(ifine)
          delxmn = amin1(delxmn,delfx(j,ifine))
  101   continue
c
c   Find the minimum horizontal grid resolution to calculate
c   time step later
c
        if(delxmn .gt. delfy(ifine))then
          horzds=delfy(ifine)
        else
          horzds=delxmn
        endif
c
c   calculate maximum time step based on wind speed and grid size
c
        do k=1,nfz(ifine)
  	  do j=2,nfy(ifine)-1
	    dtik(j,k)=amin1(dtik(j,k),0.5*delfy(ifine)/umaxi(j,k))
	    do i=2,nfx(ifine)-1
	      dtjk(i,k)=amin1(dtjk(i,k),0.5*delfx(j,ifine)/vmaxj(i,k))
	    enddo
	  enddo
	enddo
c
c        do i=2,nfx(ifine)-1
c	  write(*,'(a,i4,9f7.2)') 'i,dtjk :',
c     &           	   i,(dtjk(i,k),k=1,nfz(ifine))
c	enddo
ccc
c        do j=2,nfy(ifine)-1
c	  write(*,'(a,i4,9f7.2)') 'j,dtik :',
c     &              	   j,(dtik(j,k),k=1,nfz(ifine))
c	enddo
ccc
c   calculate limitation due to horizontal diffusivity
c  
        horz2i = 1./horzds**2
        do 200 k=1,nlayer
	  kf=kvmap(k,ifine)
	  do kk=k,kf
          do 190 j=jyfb(ifine),jyfe(ifine)
            do 180 i=ixfb(ifine),ixfe(ifine)
              diff(kk)=amax1(diff(kf),kh(i,j,k)*horz2i)
  180       continue
  190     continue
	  enddo
  200   continue
c
c     determine maximum deltat for this time step
c
        do k=1,nfz(ifine)
          if (diff(k) .gt. 0.) then
            dt2k(k)=0.15/diff(k)
          else
            dt2k(k)=1.e20
          endif
c
	  deltkf(k,ifine)=amin1(dt2k(k),deltat)
	  dtmax=amax1(dtmax,deltkf(k,ifine))
	  dtmin=amin1(dtmin,deltkf(k,ifine))
	  deltf(ifine)=amax1(deltf(ifine),deltkf(k,ifine),deltat)
c	  write(*,*) ' k,deltkf,deltf :',k,deltkf(k,ifine),deltf(ifine)
	enddo
c       write(*,*) ' dt2,deltat :',(dt2k(k),k=1,nfz(ifine)),deltat
c	write(*,*) 'dtmax,dtmin :',dtmax,dtmin
c
        dtsubf(1,ifine)=deltf(ifine)
	dtsubf(2,ifine)=deltf(ifine)
	rnchk=amod(dtsubf(1,ifine),dtmax)
	if ( rnchk .ge. 0.001 ) then
	  nstp=int(dtsubf(1,ifine)/dtmax)+1
	  dtmax=dtsubf(1,ifine)/float(nstp)
	endif
c
        if ( idiag .ne. 0 ) then
          write(*,*)' dtmax,deltkf :',
     &                   dtmax,(deltkf(k,ifine),k=1,nfz(ifine))
          call flush(6)
        endif
c
        do k=1,nfz(ifine)
	  if ( deltkf(k,ifine) .ge. dtmax ) deltkf(k,ifine)=dtmax
	  nstp=int(dtmax/deltkf(k,ifine))
	  nchk=int(amod(deltkf(k,ifine),dtmax))
	  if ( nchk .ne. 0 ) then
	    nstp=nstp+1
	    deltkf(k,ifine)=dtmax/float(nstp)
	  endif
c	  write(*,*) ' k,deltkf,nstp :',k,deltkf(k,ifine),nstp
	enddo
	dtsubf(3,ifine)=dtmax

        call adjdt1f(dtmax,nfx(ifine),nfy(ifine),nfz(ifine),
     &                                      dtik,dtjk,ifine,idiag)
c
c	do j=2,nfy(ifine)-1
c	  write(*,'(a,i4,9f7.2)') 'j,dtfsi :',
c     &                             j,(dtfsi(j,k,ifine),k=1,nfz(ifine))
c        enddo
c	do i=2,nfx(ifine)-1
c	  write(*,'(a,i4,9f7.2)') 'i,dtfsj :',
c     &                             i,(dtfsj(i,k,ifine),k=1,nfz(ifine))
c        enddo
cc
       if ( idiag .ne. 0 ) then
          write(*,*)' dtmax,deltkf -- adjusted :',
     &                   dtmax,(deltkf(k,ifine),k=1,nfz(ifine))
          call flush(6)
       endif
c
c
c     make sure that deltf goes into next delta t evenly
c
        if (ifglvl(ifine) .eq. 1) then
          dtchk = deltat
        else
          dtchk = deltf(ifgptr(ifine))
        endif
        ndftot(ifine) = int(dtchk/dtmax)
	nchk=int(amod(dtchk,dtmax))
	if ( nchk .ne. 0 ) then
	  ndftot(ifine)=ndftot(ifine)+1
	endif
        ndfmax(ifine) = ndftot(ifine)
        ndfcom(ifine) = 0
        deltf(ifine) = dtchk/float(ndftot(ifine))
        if (ifglvl(ifine) .gt. 1) then
          ndftot(ifine) = ndftot(ifine)*ndftot(ifgptr(ifine))
          ndfmax(ifgptr(ifine)) = 1
        endif
c
c   check all edges of this grid.  if any edge touches another
c   fine grid edge, then we must limit to one step per pass
c
        do 240 iedg=1,2
          do 230 i=2,nfx(ifine)-1
            if (igloc(i,iedg,ifine) .ne. ifgptr(ifine)) then
              ndfmax(ifine) = 1
            endif
  230     continue
  240   continue
        do 260 iedg=3,4
          do 250 i=2,nfy(ifine)-1
            if (igloc(i,iedg,ifine) .ne. ifgptr(ifine)) then
              ndfmax(ifine) = 1
            endif
  250     continue
  260   continue
c
        ndftots(ifine)=ndftot(ifine)
	ndfmaxs(ifine)=ndfmax(ifine)
	ndfcoms(ifine)=ndfcom(ifine)
c
c        if (idiag .ne. 0) then
c          write(*,'(a,/,a,/,7e10.4,i4)')
c     &      ' fstep calculates: umax, vmax, diff, dt2, dtmax, deltat,',
c     &      ' deltf, ndtf', umax, vmax, diff, dt2, dtmax, deltat,
c     &      deltf(ifine), ndftot(ifine)
c        endif
c
  300 continue
c
      return
      end
