c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine fvrtdif(nxf,nyf,nzf,nspecf,dtf,ifine,
     &                   pstarf,presf,temprf,ch2of,fkv,htf,
     &                   depvelf,cncf,dpositf,idebug)
c
c-----ATDM v1.00
c
c     Performs explicit vertical diffusion calculation on a fine grid
c     in sigma coordinates
c
c-----95/06/15:
c     C Emery SAI
c-----95/06/23:
c     -revised mfac and vfac for use with sigma rather than height
c     -added data statements for amolwt
c-----95/08/08:
c     -added nitrate equilibrium calculation and toxics partitioning
c-----95/10/04:
c     -added special case for dtdep > dtdif in calculation of diffusion
c      timestep
c-----95/10/05:
c     -revised calculation of PM10 for toxic partitioning to be in units
c      of ug/m3 rather than ppm
c-----95/11/08:
c     -added code to skip cells covered by a fine grid
c-----97/02/21:
c     -passed k to subroutine nitrat now.(nitrat is now replaced by remeql3)
c-----97/06/25:DFG
c     -passed pressure (i.e. pecll) to nitrat subroutine.(nitrat is now replaced by remeql3)
c
c     Based on routines from SAQM/RADM and UAM-V v1.23 Revision 1.2 94/02/10
c
c     Input arguments:
c               nxf:            number of fine grid cell in x-direction
c               nyf:            number of fine grid cell in y-direction
c               nzf:            number of fine grid layers
c               nspecf:         number of modeled species
c               dtf:            fine grid time step (min)
c               ifine:          fine grid index
c               pstarf:         array of P* (mb)
c               presf:          array of pressure (mb)
c               temprf:         array of temperature (K)
c               ch2of:          array of water vapor concentration (kg/kg)
c               fkv:            array of vertical diffusivities (m2/s)
c               htf:            array of layer interface heights (m)
c               depvelf:        array of deposition velocities (m/s)
c               cncf:           array of fine grid concentrations (ppm)
c               dpositf:        array of fine grid deposited mass (g/km2)
c               idebug:         diagnostic debug flag
c     Output arguments:
c               cncf:           array of fine grid concentrations (ppm)
c               dpositf:        array of fine grid deposited mass (g/km2)
c     Routines called:
c               remeql3    toxpart
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'balanc.cmd'
      include 'comuam.cmd'
      include 'com3.cmd'
      include 'comchem.cmd'
      include 'ctlflg.cmd'
      include 'deffin.cmd'
c
      dimension ixff0(maxfin), ixff1(maxfin),
     &          jyff0(maxfin), jyff1(maxfin)
      dimension pstarf(nxf,nyf),presf(nxf,nyf,nzf),temprf(nxf,nyf,nzf),
     &          ch2of(nxf,nyf,nzf),fkv(nxf,nyf,nzf),htf(nxf,nyf,nzf),
     &          depvelf(nxf,nyf,nspecf),cncf(nxf,nyf,nspecf,nzf),
     &          dpositf(nxf,nyf,nspecf)
      real sigmah(mxfz),dsigh(mxfz),dsigi(mxfz),wt(mxfz)
      real kfact(mxfz), mfac
      real nh3,nh4s,nh4n
      logical lprnt
      data eps /0.622/, rdry /287./, grav /9.8/, amolwt /28.8/
      data e0 /6.11/, lv /2.5e6/, rv /461./
c
c-----Bit 8 (128's location) turns on debug writes in vrtdif
c
      write (*,*) idebug
      write (*,*) and(idebug,128)
      idiag = and(idebug,128)
      if (idiag.ne.0) write(*,*)'Entering FVRTDIF, deltf = ',dtf
c
c----Set up some parameters relating the grids
c
      do 5 if1 = 1,numfin
        if (ifgptr(if1) .ne. ifine) go to 5
        ixff0(if1) = (ixfb(if1) - ixfb(ifine))*nhf(ifine) + 2
        ixff1(if1) = (ixfe(if1) - ixfb(ifine) + 1)*nhf(ifine) + 1
        jyff0(if1) = (jyfb(if1) - jyfb(ifine))*nhf(ifine) + 2
        jyff1(if1) = (jyfe(if1) - jyfb(ifine) + 1)*nhf(ifine) + 1
        if (ixff0(if1).le.2 .or. ixff1(if1).ge.nxf .or.
     &      jyff0(if1).le.2 .or. jyff1(if1).ge.nyf) then
          write(*,*) ' erroneous grid parameters in fvrtdif'
          write(*,*) ' ixff0, ixff1, jyff0, jyff1: ',ixff0,ixff1,
     &                 jyff0, jyff1
          write(*,*) 'if1, ifine: ',if1,ifine
          stop
        endif
   5  continue
c
c-----Calculate depth parameters
c
      kmax = nzf
      do 10 k = 1,kmax
        sigmah(k) = (sigmaf(k,ifine) + sigmaf(k+1,ifine))/2.
        dsigh(k) = sigmaf(k+1,ifine) - sigmaf(k,ifine)
   10 continue
      do 11 k = 1,kmax-1
        dsigi(k) = 1./(sigmah(k+1) - sigmah(k))
   11 continue
      if (idiag.ne.0) then
        write(*,*)'sigmah,dsigh,dsigi:'
        write(*,*) (sigmah(k),k=1,kmax)
        write(*,*) (dsigh(k),k=1,kmax)
        write(*,*) (dsigi(k),k=1,kmax-1)
      endif
c
c-----Loop over rows and columns
c
      do 100 j = 2,nyf-1
        do 110 i = 2,nxf-1
c
c-----Skip cells covered by another finer grid
c
          do 115 if1 = 1,numfin
            if (ifgptr(if1).eq.ifine) then
              if (i.ge.ixff0(if1) .and. i.le.ixff1(if1)
     &            .and. j.ge.jyff0(if1) .and. j.le.jyff1(if1))
     &            go to 110
            endif
  115     continue
c
          lprnt = .false.
c          if (i.eq.17 .and. j.eq.11) lprnt = .true.
          if (i.eq.nxf/2 .and. j.eq.nyf/2) lprnt = .true.
          do 120 k = 1,kmax-1
c
c-----Determine nitrate equilibrium if this is a PM chemistry run, or if
c     its a Toxic chemistry run with nitrates
c
            pcell = presf(i,j,k)
            tcell = temprf(i,j,k)
c           if (lpmchm .or. (ltxchm .and. khno3.ne.0)) then
c             ev = ch2of(i,j,k)*presf(i,j,k)/(ch2of(i,j,k) + eps)
c             es = e0*exp((lv/rv)*(1./273. - 1./tcell))
c             rhcell = 100.*ev/es
c
c               aso4 and gso4 should not be modified by remeql3
c               so calculation of fractions and redistribution
c               of sulfate has been removed.
c               since prior versions of REMSAD have assumed
c               that all pariculate nitrate is nh4no3,
c               we will assume that here also.  This means
c               that we do not need to calculate the fraction
c               of nh4 as nitrate (frnh4n) before remeql3.
c               The assumption that this fraction would be
c               the same after remeql3 is not correct anyway.
c               (tcm, 10/18/00)
c                 tso4  = cncf(i,j,kaso4,k) + cncf(i,j,kgso4,k)
c                 thno3 = cncf(i,j,khno3,k)
c                 tnh3o = cncf(i,j,knh3,k)
c                 pno3  = cncf(i,j,kpno3,k)
c                 panh4 = cncf(i,j,knh4n,k) + cncf(i,j,knh4s,k)
Ctcm              frnh4n= cncf(i,j,knh4n,k)/panh4
Ctcm              paso4 = cncf(i,j,kaso4,k) + cncf(i,j,kgso4,k)
Ctcm              fraso4= cncf(i,j,kaso4,k)/paso4
c
c                 call remeql3(tso4,thno3,tnh3o,rhcell,tcell,
c    &                 pcell,pno3,panh4,paso4,i,j,k)
c
c     check against lower bound for these species
c
c                 cncf(i,j,khno3,k) = amax1(thno3,bdnl(khno3))
c                 cncf(i,j,knh3,k)  = amax1(tnh3o,bdnl(knh3))
Ctcm              cncf(i,j,kaso4,k) = amax1(paso4*fraso4,bdnl(kaso4))
Ctcm              cncf(i,j,kgso4,k) = amax1(paso4*(1-fraso4),bdnl(kgso4))
c                 cncf(i,j,kpno3,k) = amax1(pno3,bdnl(kpno3))
Ctcm              cncf(i,j,knh4n,k) = amax1(panh4*frnh4n,bdnl(knh4n))
Ctcm              cncf(i,j,knh4s,k) = amax1(panh4*(1-frnh4n),bdnl(knh4s))
c                 cncf(i,j,knh4n,k) = amax1(pno3,bdnl(knh4n))
c                 cncf(i,j,knh4s,k) = amax1(panh4-pno3,bdnl(knh4s))
c              end if
c
c-----Partition certain toxic species between vapor and particle phase
c
            if (ltxchm) then
              pm10 = 4.
              mfac = cpfac*(1013.25/presf(i,j,k))*(tcell/273.)
              do 35 l = ngas+1,nspecf
                if (l.eq.kpom1p .or. l.eq.kpom2p .or. l.eq.kdioxp .or.
     &            l.eq.katrazp) goto 35
                lsp = jind(l)
                pm10 = pm10 + molwt(lsp)*cncf(i,j,l,k)/mfac
  35          continue
              if (kpom1.ne.0)
     &          call toxpart(pm10,tcell,cncf(i,j,kpom1,k),
     &                       cncf(i,j,kpom1p,k),1)
              if (kpom2.ne.0)
     &          call toxpart(pm10,tcell,cncf(i,j,kpom2,k),
     &                       cncf(i,j,kpom2p,k),2)
              if (kdiox.ne.0)
     &          call toxpart(pm10,tcell,cncf(i,j,kdiox,k),
     &                       cncf(i,j,kdioxp,k),3)
              if (katraz.ne.0)
     &          call toxpart(pm10,tcell,cncf(i,j,katraz,k),
     &                       cncf(i,j,katrazp,k),4)
            endif
c
c-----Calculate density and Kv-factor profile
c
            pbar = 100.*(presf(i,j,k)+presf(i,j,k+1))/2.
            tvk = temprf(i,j,k)*(eps + ch2of(i,j,k))/
     &            (eps*(1. + ch2of(i,j,k)))
            tvkp1 = temprf(i,j,k+1)*(eps + ch2of(i,j,k+1))/
     &              (eps*(1. + ch2of(i,j,k+1)))
            tvbar = (tvk + tvkp1)/2. 
            densf = pbar/(rdry*tvbar)
            kfact(k) = fkv(i,j,k)*densf**2*60.*dtf*dsigi(k)*
     &                 (grav/(100.*pstarf(i,j)))**2
c           if (lprnt .and. idiag.ne.0) then
c             write(*,*)'k,fkv,pbar,tvbar,densf,pstarf,kfact'
c             write(*,*) k,fkv(i,j,k),pbar,tvbar,densf,pstarf(i,j),
c    &                   kfact(k)
c           endif
 120      continue
          vfac = -1.e6*delfx(j,ifine)*delfy(ifine)*dsigh(1)
          mfac = pstarf(i,j)/(10.*grav*amolwt)
c
c-----Determine maximum time step and number of sub-time steps
c
          dtmin = 60.*dtf
          do 130 k = 1,kmax-1
            dt = 60.*dtf*dsigi(k)*dsigh(k)**2/(2.*kfact(k))
            dtmin = amin1(dtmin,dt)
c           if (lprnt .and. idiag.ne.0) write(*,*)'k,dtf,dt,dtmin',
c    &                                             k,dtf,dt,dtmin
 130      continue
          if (dtmin.ge.60.*dtf) then
            ndt = 1
          else
            ndt = int(60.*dtf/dtmin) + 1
          endif
          dtdif = 60.*dtf/float(ndt)
c         if (lprnt .and. idiag.ne.0) write(*,*)'ndt,dtdif',ndt,dtdif
c
c-----Loop over all species
c
          do 140 l = 1,nspecf
            lsp = jind(l)
c
c-----Load deposition rate
c     calculate deposition velocity of NH4S as a weighted average of
c     dep velocity for GSO4 and ASO4
c
c     calculate step size for deposition (make it one quarter
c     of max possible)
c
            if (l.eq.knh4s) then
               tconc = cncf(i,j,kgso4,1) + cncf(i,j,kaso4,1)
               dep = -(depvelf(i,j,kgso4)*cncf(i,j,kgso4,1) +
     &                 depvelf(i,j,kaso4)*cncf(i,j,kaso4,1))/tconc
            else
               dep = -depvelf(i,j,l)
            endif
            dtdep = htf(i,j,1)/(4.*amax1(dep,1.e-6))
            dt = amin1(dtdif,dtdep)
            if (dt.eq.dtdif) goto 145
            if (dt.ge.60.*dtf) then
              ndt = 1
            else
              ndt = int(60.*dtf/dt) + 1
            endif
 145        dt = 60.*dtf/float(ndt)
c           if (lprnt .and. idiag.ne.0) write(*,*)'l,dtdep,dt,ndt',
c    &                                             l,dtdep,dt,ndt
c
c-----Start of vertical diffusion; loop over sub time-steps
c
            do 150 n = 1,ndt
c
c   dpflow and delc1 in moles; h in m; dposit in g/km2
c
            delc1 = (dt*dep/htf(i,j,1))*cncf(i,j,l,1)*vfac*mfac
            fdpflo(l,ifine) = fdpflo(l,ifine) - delc1
            dpositf(i,j,l) = dpositf(i,j,l) + molwt(lsp)*delc1/
     &                      (delfy(ifine)*delfx(j,ifine))
c           if (lprnt .and. idiag.ne.0) then
c             write(*,*)'FVRTDIF: l,n',l,n
c             write(*,*)'dt,dep,htf,cncf,vfac,mfac'
c             write(*,*) dt,dep,htf(i,j,1),cncf(i,j,l,1),vfac,mfac 
c             write(*,*)'delc1,molwt,delfy,delfx'
c             write(*,*) delc1,molwt(lsp),delfy(ifine),delfx(j,ifine)
c             write(*,*)'dpositf',dpositf(i,j,l)
c           endif
c
            wt(1) = cncf(i,j,l,1)
     &            + kfact(1)*(cncf(i,j,l,2) - cncf(i,j,l,1))
     &              /(dsigh(1)*float(ndt))
     &            - cncf(i,j,l,1)*dt*dep/htf(i,j,1)
            wt(1) = amax1(wt(1),0.)
c
            do 160 k = 2,kmax-1
              wt(k) = cncf(i,j,l,k)
     &              + (kfact(k)*(cncf(i,j,l,k+1) - cncf(i,j,l,k))
     &              -  kfact(k-1)*(cncf(i,j,l,k) - cncf(i,j,l,k-1)))
     &                /(dsigh(k)*float(ndt))
              wt(k) = amax1(wt(k),0.)
 160        continue
c
            k = kmax
            wt(k) = cncf(i,j,l,k)
     &            - kfact(k-1)*(cncf(i,j,l,k) - cncf(i,j,l,k-1))
     &              /(dsigh(k)*float(ndt))
            wt(k) = amax1(wt(k),0.)
c
c-----Load concentration array
c
            do 170 k=1,kmax
c             if (lprnt .and. idiag.ne.0) then
c               write(*,*)'k,cncf,wt',k,cncf(i,j,l,k),wt(k)
c             endif
              cncf(i,j,l,k) = amax1(wt(k),bdnl(l))
c             if (lprnt .and. idiag.ne.0) 
c    &          write(*,*)'cncf',cncf(i,j,l,k)
 170        continue
c
c-----End sub-time loop
c
 150        continue
c
 140      continue
 110    continue
 100  continue
c
c      stop
      return
      end
