c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine fvrtwd(nxf,nyf,nzf,psnewf,psoldf,pstarf,uf,vf,
     &                  sigdotf,ifine)
c
c-----ATDM v1.00
c
c     Calculates vertical sigma velocity on the fine grid
c     (d(sigma)/dt = sigdot)
c
c-----95/06/15:
c     C Emery SAI
c-----95/10/04:
c     -fixed erroneous calculation of div by adding missing dsigma factor
c-----95/11/08:
c     -added code to skip cells covered by a fine grid
c
c     Input arguments:
c               nxf:            number of fine grid cells in x-direction    
c               nyf:            number of fine grid cells in y-direction    
c               nzf:            number of fine grid layers
c               psnewf:         new value of surface pressure (mb)
c               psoldf:         old value of surface pressure (mb)
c               pstarf:         array of current P* values (mb)
c               uf:             array of x-component windspeeds (km/min)
c               vf:             array of y-component windspeeds (km/min)
c               ifine:          fine grid index
c     Output arguments:
c               sigdotf:        vertical sigma velocity (1/s)
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parfin.cmd'
      include 'parchm.cmd'
      include 'com3.cmd'
      include 'advec.cmd'
      include 'deffin.cmd'
c
      dimension ixff0(maxfin), ixff1(maxfin),
     &          jyff0(maxfin), jyff1(maxfin)
      dimension psnewf(nxf,nyf),psoldf(nxf,nyf),pstarf(nxf,nyf),
     &          uf(nxf,nyf,nzf),vf(nxf,nyf,nzf),sigdotf(nxf,nyf,nzf)
      dimension div(mxfz)
c
c----Set up some parameters relating the grids
c
      do 5 if1 = 1,numfin
        if (ifgptr(if1) .ne. ifine) go to 5
        ixff0(if1) = (ixfb(if1) - ixfb(ifine))*nhf(ifine) + 2
        ixff1(if1) = (ixfe(if1) - ixfb(ifine) + 1)*nhf(ifine) + 1
        jyff0(if1) = (jyfb(if1) - jyfb(ifine))*nhf(ifine) + 2
        jyff1(if1) = (jyfe(if1) - jyfb(ifine) + 1)*nhf(ifine) + 1
        if (ixff0(if1).le.2 .or. ixff1(if1).ge.nxf .or.
     &      jyff0(if1).le.2 .or. jyff1(if1).ge.nyf) then
          write(*,*) ' erroneous grid parameters in fvrtwd'
          write(*,*) ' ixff0, ixff1, jyff0, jyff1: ',ixff0,ixff1,
     &                 jyff0, jyff1
          write(*,*) 'if1, ifine: ',if1,ifine
          stop
        endif
   5  continue
c
c     loop through grid
c
      do 100 j = 2,nyf-1
        do 200 i = 2,nxf-1
c
c-----Skip cells covered by another finer grid
c
          do 10 if1 = 1,numfin
            if (ifgptr(if1).eq.ifine) then
              if (i.ge.ixff0(if1) .and. i.le.ixff1(if1)
     &            .and. j.ge.jyff0(if1) .and. j.le.jyff1(if1))
     &            go to 200
            endif
   10     continue
c
c     local time rate of change of P* is based on change in input surface
c     pressure; convert sfc pressure from mb to Pa, and datadt from min to s
c
c         pppt = 100.*(psnewf(i,j) - psoldf(i,j))/(60.*datadt)
c
c     loop through all layers to calculate divergence; convert winds from
c     km/min to m/s, and pstar from mb to Pa
c
          sum = 0.
          do 300 k=nzf,1,-1
            dsigma = sigmaf(k,ifine) - sigmaf(k+1,ifine)
            if (lstagr) then
              uip = 16.67*uf(i,j,k)
              uim = 16.67*uf(i-1,j,k)
              vjp = 16.67*vf(i,j,k)
              vjm = 16.67*vf(i,j-1,k)
              vmid = 16.67*(vf(i,j,k) + vf(i,j-1,k))/2.
            else
              uip = 16.67*(uf(i,j,k) + uf(i+1,j,k))/2.
              uim = 16.67*(uf(i,j,k) + uf(i-1,j,k))/2.
              vjp = 16.67*(vf(i,j,k) + vf(i,j+1,k))/2.
              vjm = 16.67*(vf(i,j,k) + vf(i,j-1,k))/2.
              vmid = 16.67*vf(i,j,k)
            endif
            pip = 100.*pstarf(i,j)
            pim = pip
            pjp = pip
            pjm = pip
            if (uip.lt.0.) pip = 100.*pstarf(i+1,j)
            if (uim.ge.0.) pim = 100.*pstarf(i-1,j)
            if (vjp.lt.0.) pjp = 100.*pstarf(i,j+1)
            if (vjm.ge.0.) pjm = 100.*pstarf(i,j-1)
            divu = (pip*uip - pim*uim)/(1000.*delfx(j,ifine))
            divv = (pjp*vjp - pjm*vjm)/(1000.*delfy(ifine))
            divlat = -100.*pstarf(i,j)*vmid*tphioaf(j,ifine)/1000.
            div(k) = (divu + divv + divlat)*dsigma
            sum = sum - div(k)
  300     continue
          pppt = sum
c
c     calculate vertical sigma velocity
c
          sum = 0.
          sigdotf(i,j,nzf) = 0.
          psij = 100.*pstarf(i,j)
          do 301 k = nzf-1,1,-1
            sum = sum - div(k+1) 
            sigdotf(i,j,k) = sum/psij - sigmaf(k+1,ifine)*pppt/psij
  301     continue
  200   continue
  100 continue
c
      return
      end
