c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine getbc(time,ndate,iunit,ieof)
c
c-----ATDM v1.00
c
c     Reads a time interval off a UAM type boundary file and 
c     return parameters 
c
c-----95/04/21:
c     -from UAM-V Revision 1.2  94/02/10  20:56:28  tcm
c     -now scaling bc conc from ppm (gasses) to umol/m3, or from ug/m3
c      (PM) to umol/m3
c-----95/06/02:
c     -removed units conversion (added 95/04/21) as all BC are now ppm
c      and internal units are ppm
c
c     Input arguments:
c               time:           model time    
c               ndate:          model date
c               iunit:          boundary condition file unit number
c     Output arguments:
c               ieof:           end of file error flag
c     Files read:
c               iunit:          boundary condition file
c     Files written:
c               6:              standard output file unit (*)
c     Routines called:
c               jtocal    caltoj
c     Called by:
c               update
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comuam.cmd'
      include 'ibccon.cmd'
      include 'comchem.cmd'
c
      dimension bcrd(mxz,mxxx)
      character*4 mdum(10)
c
      ieof = 0
c
c  find correct time and read boundary concs
c  (if ndate is calendar date, convert file date; but if ndate
c   is julian date, convert file date to julian)
c
      if (ndate .gt. 99999) then
        if (ibcdy1 .lt. 99999) then
          call jtocal(ibcdy1,id1)
          call jtocal(ibcdy2,id2)
        else
          id1 = ibcdy1
          id2 = ibcdy2
        endif
      else
        if (ibcdy1 .gt. 99999) then
          call caltoj(ibcdy1,id1)
          call caltoj(ibcdy2,id2)
        else
          id1 = ibcdy1
          id2 = ibcdy2
        endif
      endif
      tchk = bctim1*100. + 2400.*(id1-ndate)
      tcend = bctim2*100. + 2400.*(id2-ndate)
      if (time .ge. tchk-1. .and. time .le. tcend-1.) return
c
   10 read (iunit,end=900) ibcdy1, bctim1, ibcdy2, bctim2
      write(*,'(a,i8,f10.2,a,i8,f10.2,a,i4)')
     &     ' Read interval ',ibcdy1, bctim1, ' to ', ibcdy2, bctim2,
     &     ' on unit ', iunit
c
c  set all concentrations to numerical lower bound
c
      do 30 k=1,nlayer
        do 28 l=1,nospec
          do 26 j=1,mxxx
            do 24 i=1,4
              bc(j,i,l,k) = bdnl(l)
   24       continue
   26     continue
   28   continue
   30 continue
c
c
      do 200 l=1,nbcsp
        lint = kbcsp(l)
        if ( lint .gt. 0 ) ll=jind(lint)
c       ll = jind(lint)
        do 190 nn=1,4
          ncel = noy
          if (nn .gt. 2) ncel = nox
          nbc = 5 - nn
          read(iunit) iseg,(mdum(ii),ii=1,10),nedgno,
     &                ((bcrd(ii,jj),ii=1,nlayer),jj=1,ncel)
          if (lint .eq. 0) go to 190
          do 150 j=1,ncel
            if (nbc .eq. 1) then
              ixx = j
              jyy = jend(j)
            else if (nbc .eq. 2) then
              ixx = j
              jyy = jstr(j)
            else if (nbc .eq. 3) then
              ixx = iend(j)
              jyy = j
            else if (nbc .eq. 4) then
              ixx = istr(j)
              jyy = j
            endif
            do 140 k=1,nlayer
              if (ixx .ge. 1 .and. ixx .le. nox .and.
     &            jyy .ge. 1 .and. jyy .le. noy .and.
     &            bcrd(k,j) .ge. 0.) then
                bc(j,nbc,lint,k) = bcrd(k,j)
              else
                bc(j,nbc,lint,k) = -9.
              endif
  140       continue
  150     continue
  190   continue
  200 continue
c
      if (kno2s .gt. 0) then
        if (kno2 .gt. 0) then
          do nn=1,4
            ncel = noy
            if (nn .gt. 2) ncel = nox
            nbc = 5 - nn
            do j=1,ncel
              do k=1,nlayer
                bc(j,nbc,kno2s,k) = bc(j,nbc,kno2,k)
              enddo
            enddo
          enddo
        endif
      endif
c
c  check time and if not correct, go back for another interval
c  (if ndate is calendar date, convert file date; but if ndate
c   is julian date, convert file date to julian)
c
      if (ndate .gt. 99999) then
        if (ibcdy1 .lt. 99999) then
          call jtocal(ibcdy1,id1)
          call jtocal(ibcdy2,id2)
        else
          id1 = ibcdy1
          id2 = ibcdy2
        endif
      else
        if (ibcdy1 .gt. 99999) then
          call caltoj(ibcdy1,id1)
          call caltoj(ibcdy2,id2)
        else
          id1 = ibcdy1
          id2 = ibcdy2
        endif
      endif
      tchk = bctim1*100. + 2400.*(id1-ndate)
      tcend = bctim2*100. + 2400.*(id2-ndate)
      if (time .ge. tcend) go to 10
      if (time .lt. tchk-1.) then
        write(*,*) ' Time error on BOUNDARY file',
     &             ' Current time: ',time, ndate,
     &             ' File time: ',bctim2,ibcdy2
        stop
      endif
c
      return
c
  900 ieof = 1
      return
c
      end
