c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine getpt(time,ndate,iunit,nptsp,ieof)
c
c-----ATDM v1.00
c
c     Reads a time interval off a UAM type point-source emissions file and 
c     returns parameters 
c
c-----95/04/21:
c     -from UAM-V Revision 1.3  94/08/04  14:29:34  tcm
c     -made provisions for emissions of gasses (mol/hr) and PM (g/hr):
c      convert all input emission rates to mol/min
c-----95/10/30:
c     -added dtdata factor to emission rates in the case that they
c      are in terms of n-hourly rates rather than hourly
c
c     Input arguments:
c               time:           model time     
c               ndate:          model date
c               iunit:          point source emissions file unit number
c               nptsp:          number of emitted species
c     Output arguments:
c               ieof:           end of file error flag
c     Routines called:
c               caltoj    jtocal
c     Called by:
c               atdm      setup
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'stack.cmd'
      include 'iptcon.cmd'
      include 'oneday.cmd'
      include 'comchem.cmd'
      include 'comuam.cmd'
      include 'com3.cmd'
c
      character*4 mdum(10)
c
      fact = 1./datadt
      pi = 3.1415927
      ieof = 0
c
c  find correct time and read point-source emissions
c  (if ndate is calendar date, convert file date; but if ndate
c   is julian date, convert file date to julian)
c
      if (ndate .gt. 99999) then
        if (iptdy1 .lt. 99999) then
          call jtocal(iptdy1,id1)
          call jtocal(iptdy2,id2)
        else
          id1 = iptdy1
          id2 = iptdy2
        endif
      else
        if (iptdy1 .gt. 99999) then
          call caltoj(iptdy1,id1)
          call caltoj(iptdy2,id2)
        else
          id1 = iptdy1
          id2 = iptdy2
        endif
      endif
      tchk = pttim1*100. + 2400.*(id1-ndate)
      tcend = pttim2*100. + 2400.*(id2-ndate)
      if (time .ge. tchk-1. .and. time .le. tcend-1.) return
c
   50 read (iunit,end=900) iptdy1, pttim1, iptdy2, pttim2
      write(*,'(a,i8,f10.2,a,i8,f10.2,a,i4)')
     &     ' Read interval ',iptdy1, pttim1, ' to ', iptdy2, pttim2,
     &     ' on unit ', iunit
      if (l1eday) then
        if ( iptdy1 .gt. 99999 ) then
          call caltoj(iptdy1,iid1)
        else
          iid1=iptdy1
        endif
        if ( iptdy2 .gt. 99999 ) then
          call caltoj(iptdy2,iid2)
        else
          iid2=iptdy2
        endif
        if ( ndate .gt. 99999 ) then
          call caltoj(ndate,nnd1)
        else
          nnd1=ndate
        endif
        ndif = iid2-iid1
        iid1 = nnd1
        iid2 = iid1 + ndif
        iptdy1 = iid1
        iptdy2 = iid2
c        ndif = iptdy2 - iptdy1
c        iptdy1 = ndate
c        iptdy2 = iptdy1 + ndif
      endif
c
c  number of source on a record is not allowed to change
c
      write (*,*) 'nopts before the read of nn', nopts
      read (iunit,end=900) idum, nn
      if (nn .ne. nopts) then
        write(*,'(a)') ' ILLEGAL REDEFINITION OF NO. OF SOURCES.'
        write(*,'(a,i5)') ' ORIGINAL:  ',nopts
cgem        write(*,'(a,i5)') ' NEW:       ',nopts
        write(*,'(a,i5)') ' NEW:       ',nn
        stop
      endif
c
c  skip the location record
c
      read (iunit,end=900) (idum,idum,idum,flowrat(n),dum,n=1,nopts)
c
c  if flowrat > 0, convert flow rate to new exit velocity in m/s 
c
      do 80 i = 1,nopts
         if (flowrat(i).gt.0.) then
            vexit(i) = flowrat(i)/(3600.*pi*(stkd(i)/2)**2)
         endif
   80 continue
c
c  emissions record
c
      do 100 l=1,nptsp
        read(iunit,end=900) idum,(mdum(ii),ii=1,10),
     &                     (ptemis(l,i),i=1,nopts)
c
c  convert emission rates from moles/n-hour to moles/min for gasses
c                     and from g/n-hour to moles/min for PM
c
        ll = kemit(l)
        if (ll.eq.0) goto 100
        lll = jind(ll)
c       write(*,*) 'In getpt, l, ll, lll, ngas, molwt(lll): '
c       write(*,*) l, ll, lll, ngas, molwt(lll)
        do 90 i = 1,nopts
          if (ll.le.ngas) then
             ptemis(l,i) = ptemis(l,i)*fact
          else
             ptemis(l,i) = ptemis(l,i)*fact/molwt(lll)
          endif
   90   continue
c
  100 continue
c
c  check time and if not correct, go back for another interval
c  (if ndate is calendar date, convert file date; but if ndate
c   is julian date, convert file date to julian)
c
      if (ndate .gt. 99999) then
        if (iptdy1 .lt. 99999) then
          call jtocal(iptdy1,id1)
          call jtocal(iptdy2,id2)
        else
          id1 = iptdy1
          id2 = iptdy2
        endif
      else
        if (iptdy1 .gt. 99999) then
          call caltoj(iptdy1,id1)
          call caltoj(iptdy2,id2)
        else
          id1 = iptdy1
          id2 = iptdy2
        endif
      endif
      tchk = pttim1*100. + 2400.*(id1-ndate)
      tcend = pttim2*100. + 2400.*(id2-ndate)
      if (time .ge. tcend) go to 50
      if (time .lt. tchk-1.) then
        write(*,*) ' Time error on PTSOURCE file',
     &             ' Current time: ',time, ndate,
     &             ' File time: ',pttim2,iptdy2
        stop
      endif
c
      return
c
  900 ieof = 1
      return
c
      end
