c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine grdsp
c
c-----ATDM v1.00
c
c     Determines grid cell sizes for coarse domain and all fine grids
c
c-----95/02/22:
c     -from UAM-V Revision 1.2  94/02/10  20:56:49  tcm
c     -removed code calculating coordinates for non-gridded receptors
c
c     Files written:
c               6:              standard output file unit (*)      
c               8:              diagnostic file 
c     Called by:
c               setup
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'comuam.cmd'
      include 'latlon.cmd'
      include 'com3.cmd'
c
c     dkm - number of km in 1 deg lat
c     dtr - degrees to radians
c     a   - Earth radius (km)
c
      data dkm/111.1338/, dtr/0.01745329/
      data round/5.e-2/
      data a/6367.5/
c
c     if using cartesian coordinate system then dlong and dlat are
c     actually grid spacing in km
c
      if (lcart) then
        deltay=dlat
        do 5 j=1,noy
          deltax(j)=dlong
          tphioa(j)=0.0
    5   continue
c
      else
c
c     check for consitency in grid definition
c
        dum=wlong+dlong*float(nox)
        if (abs(dum-elong).gt.round) go to 800
        dum=slat+dlat*float(noy)
        if (abs(dum-nlat).gt.round) go to 800
c
c     calculate grid spacing in km
c
        deltay=dkm*dlat
        do 10 j=1,noy
          alat=(float(j)-0.5)*dlat+slat
          deltax(j)=dkm*dlong*cos(alat*dtr)
          tphioa(j)=tan(alat*dtr)/a
   10   continue
c
      endif
      write(*,*) 'deltax(1), deltay: ', deltax(1), deltay
c
c   calculate cell sizes on fine grid
c
      do 26 ifine=1,numfin
c
        if(lcart) then
          do 23 j=1,nfy(ifine)
            tphioaf(j,ifine)=0.
            delfx(j,ifine) = deltax(1)/float(nhf(ifine))
   23     continue
          delfy(ifine) = deltay/float(nhf(ifine))
c
        else
c
          do 25 j=jyfb(ifine),jyfe(ifine) 
            alat=(float(j)-1.)*dlat +slat
            do 24 jj=1,nhf(ifine)
              jf=(j-jyfb(ifine))*nhf(ifine)+jj+1
              alatf=alat+(float(jj)-0.5)*dlat/float(nhf(ifine))
              delfx(jf,ifine)=dkm*dlong*cos(alatf*dtr)/float(nhf(ifine))
              tphioaf(jf,ifine)=tan(alatf*dtr)/a
   24       continue
   25     continue
          alat=(float(jyfb(ifine))-1.)*dlat +slat
          alatf=alat-0.5*dlat/float(nhf(ifine))
          delfx(1,ifine)=dkm*dlong*cos(alatf*dtr)/float(nhf(ifine))
          alat=float(jyfe(ifine))*dlat +slat
          alatf=alat+0.5*dlat/float(nhf(ifine))
          delfx(nfy(ifine),ifine)=dkm*dlong*cos(alatf*dtr)
     &                                       /float(nhf(ifine))
          delfy(ifine) = deltay/float(nhf(ifine))
c
        endif
        write(*,*) 'ifine, delfx(1,ifine), delfy(ifine): '
        write(*,*) ifine, delfx(1,ifine), delfy(ifine)
   26 continue
      return
  800 continue
c
c     grid definition input is inconsistent
c
      write(8,3000) wlong,elong,dlong,nox,dum,slat,nlat,dlat,noy,dum
      write(6,3000) wlong,elong,dlong,nox,dum,slat,nlat,dlat,noy,dum
 3000 format(' $$ERROR GRID DEFINITION INPUT IS INCONSISTENT'
     &,3e12.4,i5,e12.4,/,3e12.4,i5,e12.4)
      stop1
      end
