c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
c
      subroutine hgchem(i,j,k,ihg,deltat,airden,lwch2o,atoh,atho2,
     &                  atno3,ath2o2,ato3,so2av,atcl2,atpec,lprnt)
c
c In this subroutine, it is assumed that for any tagged mercury, there are 
c three Hg species (Hg0, HG(II) and HgP). Every Hg species is also assumed to
c be the total Hg in that form i.e. there is no phase distinction.       
c
c The output of this subroutine is aqueous and gas phases of Hg0 and Hg(II) 
c and Hg(P) which will be passed over to wetdep subroutine where the final out
c put will be the total Hg0, Hg(II) and Hg(P) 
c     Chemistry driver for the coarse domain
c     
c-----99/12/21:
c     
c     Input arguments:
c     deltat:         model time step (min)
c     time:           model time
c     idate:          model date
c     idebug:         diagnostic debug flag
c     Files written:
c     6:              standard output file unit (*)     
c     Called by:
c     rchem
c     
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'ohtbl.cmd'
      include 'aerosl.cmd'
      include 'balanc.cmd'
      include 'ctlflg.cmd'
      include 'latlon.cmd'
c
      dimension ahg0(mxhg,mxz),ghg0(mxhg,mxz),ahg2(mxhg,mxz),ghg2(mxhg,mxz)
      real*4 lwch2o
      logical lprnt
      data ph /4.5/
      data xk6 /2.09e6/, xk7 /1.99e6/, xka /3.16e-8/ !Lin&Pehkonen
      data t33, t108 /5.01e-4, 1.58e-11/   ! 10**-3.3,10**-10.8
      data xs1, xs2 /1.3e-2, 6.6e-8/  !Seinfeld & Pandis
      data e1, e2 /2.1e13, 1.0e10/   !sulfite eq., Van Loon, Muenthe
      data e5, e6, e9, e10, e11, e12, e13 
     &    /4.27e+10,1.74e+22,1.78e+18,2.00e+7,1.e+14,1.e+15,3.98e+15/
      data solhgo /2.44e-4/  ! Solubility of HgO in mole/L
      data pecfac, pecbas /0.55, 450./  ! max fraction of hg2 adsorbed 
c                                         to soot.  conc of pec
c                                         in ugm/(mole of air) at
c                                         which adsorption is max.
c
      if (khg0(ihg) .eq. 0 .or. 
     &    khg2(ihg) .eq. 0 .or. 
     &    khgp(ihg) .eq. 0 ) then
        write(*,*) 'Bad subscripts in hgchem'
        write(*,*) 'ihg, khg0(ihg), khg2(ihg), khgp(ihg): '
        write(*,*) ihg, khg0(ihg), khg2(ihg), khgp(ihg)
      endif
      if (lprnt) then 
        write(*,*) 'Entering hgchem'
        write(*,*)' conc(i,j,khg0(ihg),k),conc(i,j,khg2(ihg),k),conc(i,j,khgp(ihg),k): '
        write(*,*) conc(i,j,khg0(ihg),k),conc(i,j,khg2(ihg),k),conc(i,j,khgp(ihg),k)
        write(*,*) 'i,j,k,ihg,deltat,airden,lwch2o,atoh,atho2: '
        write(*,*) i,j,k,ihg,deltat,airden,lwch2o,atoh,atho2
        write(*,*) 'atno3,ath2o2,ato3,so2av,atcl2: '
        write(*,*) atno3,ath2o2,ato3,so2av,atcl2
      endif
      hplus = 10**(-ph)
      ohm = 10**(ph-14.)
      hplus2 = hplus*hplus
      a0c = hplus/(hplus+xka)
      a1c = xka/(hplus+xka)
      xso3 = 1./(1. + hplus/xs2 + hplus2/(xs1*xs2))
      so32m = heffso*so2av*1.e-6*press(i,j,k)/1013. * xso3
      if (lprnt) then 
        write(*,*)' hplus,ph,a0c,xka,a1c,xso3,xs2,xs1 : '
        write(*,*) hplus,ph,a0c,xka,a1c,xso3,xs2,xs1 
        write(*,*)' so32m,heffso,so2av,press(i,j,k): '
        write(*,*) so32m,heffso,so2av,press(i,j,k)
      endif
c
c   reduce oh and ho2 gas phase concentrations if in cloud
c   (use lwch2o > 0.1 to determine presence of cloud)
c
      if (lwch2o .gt. 0.1) then
        atoh = atoh/2.        ! Jacob, 2000
        atho2 = atho2/4.
      endif
c     if (lwch2o .eq. 0.0) go to 5
c
c converting Hg mixing ratio (ppm) into concentration (molecules/cm3)
c
      convfac = airden*1.0e-6
c
      thg0 = conc(i,j,khg0(ihg),k)*convfac
      thg2 = conc(i,j,khg2(ihg),k)*convfac
c
      conc(i,j,khgp(ihg),k) = conc(i,j,khgp(ihg),k)*convfac
c
c  calculate dissolved HgO
c
      xlw = lwch2o/1.e9     !  Liquid water in L/cm3
      if (xlw .gt. 0.) then
        ahgp = amin1(conc(i,j,khgp(ihg),k)/6.02e23/xlw,solhgo)
      else
        ahgp = 0.
      endif
      dhgp = ahgp
      conc(i,j,khgp(ihg),k) = conc(i,j,khgp(ihg),k) - dhgp*6.02e23*xlw
c
      athcl = 0.001
      if (lprnt) then 
        write(*,*)' convfac, airden, thg0,conc(i,j,khg0(ihg),k),thg2: '
        write(*,*) convfac, airden, thg0,conc(i,j,khg0(ihg),k),thg2
        write(*,*)' conc(i,j,khg2(ihg),k),conc(i,j,khgp(ihg),k),athcl: '
        write(*,*) conc(i,j,khg2(ihg),k),conc(i,j,khgp(ihg),k),athcl
        write(*,*) 'dhgp, xlw, solhgo, lwch2o: '
        write(*,*) dhgp, xlw, solhgo, lwch2o
      endif
c
c Additional Henry law constants (M atm-1), at 298, for Hg related chemistry
c
      tfac = 1./298. - 1./temper(i,j,k)
      ekh  = ekh0(jind(khg0(ihg)))*exp(tdep(jind(khg0(ihg)))*tfac)
      ek1d = ek1d0(jind(khg0(ihg)))*exp(tdep1d(jind(khg0(ihg)))*tfac)
      ek2d = ek2d0(jind(khg0(ihg)))*exp(tdep2d(jind(khg0(ihg)))*tfac)
      ekhhg0 = ekh + ekh*ek1d/hplus + ekh*ek1d*ek2d/hplus2
      ekh  = ekh0(jind(khg2(ihg)))*exp(tdep(jind(khg2(ihg)))*tfac)
      ek1d = ek1d0(jind(khg2(ihg)))*exp(tdep1d(jind(khg2(ihg)))*tfac)
      ek2d = ek2d0(jind(khg2(ihg)))*exp(tdep2d(jind(khg2(ihg)))*tfac)
      ekhhgcl = ekh + ekh*ek1d/hplus + ekh*ek1d*ek2d/hplus2
      ekhhcl = 727.*exp(-2020.*tfac)  !HCl   Seinfeld and Pandis, 1998
      ekhoh  = 25.*exp(-5250.*tfac)     ! from aqbdat.f
      ekhho2 = 2.0e+3*exp(-6600.*tfac)  ! from aqbdat.f
      ekh  = ekh0(0)*exp(tdep(0)*tfac)
      ek1d = ek1d0(0)*exp(tdep1d(0)*tfac)
      ek2d = ek2d0(0)*exp(tdep2d(0)*tfac)
      ekho3 = ekh + ekh*ek1d/hplus + ekh*ek1d*ek2d/hplus2
      if (lprnt) then 
        write(*,*)' tfac,temper(i,j,k),ekhoh, ekhho2,ekho3: '
        write(*,*) tfac,temper(i,j,k),ekhoh, ekhho2,ekho3
        write(*,*) 'ekhhg0,ekhhgcl,ekhhcl,ekhoh,ekhho2,ekho3: '
        write(*,*) ekhhg0,ekhhgcl,ekhhcl,ekhoh,ekhho2,ekho3
      endif
c      
c Partition the gas and aqueous phase components from total Hg(0) and Hg(II)
c (here we are assuming that whenever Hg(II) enters in this subroutine, it is in the gas phase as HgCl2 and when it exits, it will be in the gas phase HgCl2)
c Here the aqueous and gas phase concentrations are in molecules cm-3.
c
      frachg0 = (1.0e-6)*ekhhg0*0.08206*temper(i,j,k)*lwch2o
      frachg2 = (1.0e-6)*ekhhgcl*0.08206*temper(i,j,k)*lwch2o
      fraco3 = (1.0e-6)*ekho3*0.08206*temper(i,j,k)*lwch2o
      fracoh = (1.0e-6)*ekhoh*0.08206*temper(i,j,k)*lwch2o
      fracho2 = (1.0e-6)*ekhho2*0.08206*temper(i,j,k)*lwch2o
      frachcl = (1.0e-6)*ekhhcl*0.08206*temper(i,j,k)*lwch2o
      if (lprnt) then 
        write(*,*)' frachg0,frachg2,fraco3,fracoh,fracho2,frachcl: '
        write(*,*) frachg0,frachg2,fraco3,fracoh,fracho2,frachcl
      endif
c
      ghg0(ihg,k) = thg0/(1.0+frachg0)
      ahg0(ihg,k) = ghg0(ihg,k)*ekhhg0
     &              *0.08206*temper(i,j,k)*(1.e-6/6.02e+20)
      ghg2(ihg,k) = thg2/(1.0+frachg2)
      ahg2(ihg,k) = ghg2(ihg,k)*ekhhgcl
     &              *0.08206*temper(i,j,k)*(1.e-6/6.02e+20)+dhgp
      aclminus = athcl*1.e-6*(press(i,j,k)/1013.)*ekhhcl/(1.0+frachcl)
      if (lprnt) then 
        write(*,*)' ghg0(ihg,k),ahg0(ihg,k),ghg2(ihg,k),ahg2(ihg,k),aclminus: '
        write(*,*) ghg0(ihg,k),ahg0(ihg,k),ghg2(ihg,k),ahg2(ihg,k),aclminus
      endif
c
c     (Lin&Pehkonen, 1999, and 1998 JGR: )
      if (lwch2o .gt. 0. .and. aclminus .gt. 1.e-10) then
        t = temper(i,j,k)-273.
        ekhcl20 = 5.67e-8*t**3+2.50e-5*t**2-3.59e-3*t+1.49e-1
        ekhcl2 = ekhcl20*(1.+t33/(aclminus*hplus)+t108/(aclminus*hplus))
        fraccl2 = (1.0e-6)*ekhcl2*0.08206*temper(i,j,k)*lwch2o
      else
        fraccl2 = 0.
      endif
      if (lprnt) then 
        write(*,*) ekhcl2,fraccl2
      endif
c
c aqueous phase concentrations for o3,oh,ho2 are in (mole/liter)
c NOTE: RETAIN THEM IN THIS UNIT AS THE RATE CONSTANTS FOR AQUEOUS PHASE 
c REACTIONS ARE IN moles-1 L s-1)
c
      go3 = convfac*ato3/(1.0+fraco3)
      ao3 = ato3*1.e-6*(press(i,j,k)/1013.)*ekho3/(1.0+fraco3)
      goh = convfac*atoh/(1.0+fracoh)
      aoh = atoh*1.e-6*(press(i,j,k)/1013.)*ekhoh/(1.0+fracoh)
      gho2 = convfac*atho2/(1.0+fracho2)
      aho2 = atho2*1.e-6*(press(i,j,k)/1013.)*ekhho2/(1.0+fracho2)
      acl2 = atcl2*1.e-6*(press(i,j,k)/1013.)*ekhhcl/(1.0+fraccl2)
      bb = 1.+e5*ohm+e6*ohm**2+e9*ohm*aclminus+e10*aclminus
     &     +e11*aclminus**2+e12*aclminus**3+e13*aclminus**4
      hgso3 = e1*so32m*ahg2(ihg,k)/(e1*e2*so32m**2 + 
     &                              e1*so32m + bb)
      if (lprnt) then 
        write(*,*)' go3,ao3,goh,aoh,gho2,aho2,acl2,hgso3: '
        write(*,*) go3,ao3,goh,aoh,gho2,aho2,acl2,hgso3
      endif
c
c gas phase chemistry of ghg0
c rate constants for gas phase reactions of Hg0
c
      ghg0o3 = 3.0e-20 * 60.       ! Hg0 + O3 
c
c  zero no3 rate based on recommendation of C. Seigneur and R. Bullock
c     ghg0no3 = 4.0e-15 * 60.      ! Hg0 + NO3
      ghg0no3 = 0.0                ! Hg0 + NO3
      ghg0h2o2 = 8.5e-19 * 60.     ! Hg0 + H2O2
c
      ghg0loss = (ghg0o3*go3)+(ghg0no3*convfac*atno3)+
     *           (ghg0h2o2*convfac*ath2o2)
c     ghg0(ihg,k) = ghg0(ihg,k)*exp(-ghg0loss*deltat)
      s1 = ghg0(ihg,k)*exp(-ghg0o3*go3*deltat)
      do3hg0 = ghg0(ihg,k) - s1
      s2 = s1*exp(-ghg0h2o2*convfac*ath2o2*deltat)
      dh2o2hg0 = s1 - s2
      s3 = s2*exp(-ghg0no3*convfac*atno3*deltat)
      dno3hg0 = s2 - s3
      ghg0(ihg,k) = s3
      if (lprnt) then 
        write(*,*)' ghg0loss,s1,do3hg0,s2,dh2o2hg0,s3,dno3hg0,ghg0(ihg,k): '
        write(*,*) ghg0loss,s1,do3hg0,s2,dh2o2hg0,s3,dno3hg0,ghg0(ihg,k)
      endif
c
c aqueous phase reactions of ahg0
c rate constants for aqueous phase reactions of Hg0 and HgII
c
      ahg0o3 = 4.7e+7 * 60.                ! Hg0 + O3 
      ahg0oh = 2.0e+9 * 60.                ! Hg0 + OH
      ahg2ho2 = 1.7e+4 * 60.               ! HgII + HO2
      ahg0cl2 = (xk6*a0c + xk7*a1c) * 60.  ! Hg0 + Cl2
      ahg2so3 = temper(i,j,k)*exp((31.971*temper(i,j,k)-12595.)
     &                                       /temper(i,j,k)) * 60.
c                                      ^   ! HgSO3 -> Hg0
      if (lprnt) then 
        write(*,*)' ahg0o3,ahg0oh,ahg2ho2,ahg0cl2,ahg2so3: '
        write(*,*) ahg0o3,ahg0oh,ahg2ho2,ahg0cl2,ahg2so3
      endif
c
      ahg0gain = ahg2ho2*aho2*ahg2(ihg,k) + ahg2so3*hgso3
      ahg0loss = (ahg0o3*ao3)+(ahg0oh*aoh)+(ahg0cl2*acl2)
c     ahg0(ihg,k) = ahg0(ihg,k)+(ahg0gain-ahg0loss*ahg0(ihg,k))*deltat
      s1 = ahg0(ihg,k)*exp(-ahg0o3*ao3*deltat)
      ado3hg0 = ahg0(ihg,k) - s1
      s2 = s1*exp(-ahg0oh*aoh*deltat)
      adohhg0 = s1 - s2
      s3 = s2*exp(-ahg0cl2*acl2*deltat)
      adcl2hg0 = s2 - s3
      if (lprnt) then 
        write(*,*)' ahg0gain,ahg0loss,s1,ado3hg0,s2,adohhg0,s3,adcl2hg0: '
        write(*,*) ahg0gain,ahg0loss,s1,ado3hg0,s2,adohhg0,s3,adcl2hg0
      endif
c
c  calculate fraction of hg2 available for reaction in aqueous phase
c
      adsfac = 1. - amin1(atpec/pecbas * pecfac, pecfac)
c
c aqueous phase formation of Hg(II)
c
c     ahg2(ihg,k) = ahg2(ihg,k) + (ahg0loss*ahg0(ihg,k)-ahg0gain)*deltat
      s1 = ahg2(ihg,k)*adsfac*exp(-ahg2ho2*aho2*deltat)
      adho2hg2 = ahg2(ihg,k) - s1
      s2 = s1*exp(-ahg2so3*hgso3*deltat)
      adso3hg2 = s1 - s2
      ahg0(ihg,k) = ahg0(ihg,k) - (ado3hg0 + adohhg0 + adcl2hg0)
     &                          + (adho2hg2 + adso3hg2)
      ahg2(ihg,k) = ahg2(ihg,k) -  (adho2hg2 + adso3hg2)
     &                          + (ado3hg0 + adohhg0 + adcl2hg0)
      if (lprnt) then 
        write(*,*)' s1,adho2hg2,s2,adso3hg2,ahg0(ihg,k),ahg2(ihg,k): '
        write(*,*) s1,adho2hg2,s2,adso3hg2,ahg0(ihg,k),ahg2(ihg,k)
      endif
c
c there is no significant gas phase loss reaction of Hg(II), but there is a 
c gas phase formation of Hg(II) in the form of Hg(OH)2
c
      ghg2(ihg,k)=ghg2(ihg,k)+dh2o2hg0
c
c Production of Hg(II) particulate. Whenever Hg reacted with O3, NO3 and other
c reactants, the formed product HgO is considered as the soild phase Hg(II)
c
      phggain = do3hg0 + dno3hg0
c
      conc(i,j,khgp(ihg),k) = conc(i,j,khgp(ihg),k) + phggain
      if (lprnt) then 
        write(*,*)' ghg2(ihg,k),phggain,conc(i,j,khgp(ihg),k): '
        write(*,*) ghg2(ihg,k),phggain,conc(i,j,khgp(ihg),k)
      endif
c
c recombine gas phase and aqueous phase concentration to give the total conc
c
      conc(i,j,khg0(ihg),k) = ghg0(ihg,k)+(6.02e+23)*xlw*ahg0(ihg,k)
      conc(i,j,khg2(ihg),k) = ghg2(ihg,k)+(6.02e+23)*xlw*ahg2(ihg,k)
C
c convert the concentration unit back to mixing ratio (ppm)      
c
      conc(i,j,khg0(ihg),k) = amax1(conc(i,j,khg0(ihg),k)/convfac,
     &                                             bdnl(khg0(ihg)))
      conc(i,j,khg2(ihg),k) = amax1(conc(i,j,khg2(ihg),k)/convfac,
     &                                             bdnl(khg2(ihg)))
      conc(i,j,khgp(ihg),k) = amax1(conc(i,j,khgp(ihg),k)/convfac,
     &                                             bdnl(khgp(ihg)))
c
 5    continue
c
      if (lprnt) then 
        write(*,*) 'Leaving hgchem'
        write(*,*)' conc(i,j,khg0(ihg),k),conc(i,j,khg2(ihg),k),conc(i,j,khgp(ihg),k): '
        write(*,*) conc(i,j,khg0(ihg),k),conc(i,j,khg2(ihg),k),conc(i,j,khgp(ihg),k)
      endif
c
      return
c
      end
