c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine micsurf(nm,gsun,ustrw,wsrf,tcellk,rhum,lstrcl,
     &                  lsnocl,flndcl,raincl,rs,pcell)
c
c    ATDM
c    
c    96/09/12:dfg
c    -from UAM-V Revision 1.3  1996/06/14  23:07:22  tcm
c    -move species ispd
c    -add PM species that used for REMSAD.
c    -moved calculation of RCL and RGS to near top of 270 loop so that
c     it is calculated each landuse iteration independent of water,
c     dew-wetted, or dry surface
c    -redefine DRATIO1 and FREACT1 as local variables.
c    98/02/27:dfg
c    -added new method for calculating NH3 surface resistance.
c     References: Duyzer, 1994; Duyzer et al., 1994. 
c    03/02/20:tcm
c    -revised wet surface treatments to use Henry's law to estimate
c     surface resistance for soluble species.  Eliminated specific
c     set of ammonia resistance to zero.  Eliminated special values
c     of resistance for ammonia for certain forest canopy land use, etc.c
c
c
c     gaseous depositing species are assumed to be soluble and/or reactive
c     note that the indices here are not the same as for the concentrations
c
c       kt(1)   so2
c       kt(2)   o3
c       kt(3)   h2o2
c       kt(4)   hno3
c       kt(5)   hno2
c       kt(6)   no2
c       kt(7)   no
c       kt(8)   co2     * (not a state species)
c       kt(9)   nh3     *
c       kt(10)  pan
c       kt(11)  hcho
c       kt(12)  hcooh
c       kt(13)  hcl     *
c       kt(14)  mhp
c       kt(15)  paa
c       kt(16)  oh      *
c       kt(17)  ho2     *
c       kt(18)  hno4
c       kt(19)  mgly
c       kt(20)  cres
c       kt(21)  meoh
c       kt(22)  ald2
c       kt(23)  open
c--- add more gas species for REMSAD PM species. 
c       kt(24)  voc
c       kt(25)  NAPH
c       kt(26)  POM1
c       kt(27)  POM2
c       kt(28)  HG0 
c       kt(29)  HG2G
c       kt(30)  DIOXIN
c       kt(31)  ATRAZ
c       kt(32)  SULF
c  also,kn2o5   n2o5
c
c  Input arguments:
c     nm = month
c     gsun = solar insolation (w/m2) at ground
c     ustrw = friction velocity of water
c     wsrf = 10 m wind speed (m/sec)
c     tcellk = ambient cell temperature (k)
c     rhum = ambient relative humidity (%)
c     pcell = ambient cell pressure (mb)
c     lstrcl = stress code for this cell
c     lsnocl = snow code for this cell
c     flndcl = fractional land use values for this cell
c     raincl = rainfall rate this cell (inches/hour)
c
c  Output arguments:
c     rs = the array of surface resistances (species dependent)
c
c  Called by:
c     surlyr   surlyrf
c
c**********************************************************************
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
c
c******   local arrays
      dimension rmin(mlus,5),rcuts(mlus,5),rcls(mlus,5),rgss(mlus,5),
     &          rac(mlus,5),rclo(mlus,5),rgso(mlus,5)
      dimension rsurf(mlus,nhen),rstom(mlus,nhen),rmes(nhen),
     &          rcut(mlus,nhen), rcl(mlus,nhen),rgs(mlus,nhen),
     &          rcutwet(mlus,nhen),rstrf(mlus,nhen)
      dimension dratio1(nhen),freact1(nhen)
      dimension rs(mxgas,mlus), wsrf(mlus)
      dimension tmax(mlus), tmin(mlus), topt(mlus)
      dimension flndcl(mlus)
      dimension hheff(nhen), hheff45(nhen)
      dimension rstmp(nhen,mlus)
      logical ldew, lfirst
      save lfirst
c
      data lfirst /.true./
c
c  dratio11(i) = diffusivity(h2o)/diffusivity(i)
c  calculated using the molecular weights: SQRT(Mi/Mh2o)
c  (species order as in this routine).
c
      data dratio1/1.9,1.6,1.4,1.9,1.6,1.6,1.3,1.6,1.0,2.6,1.3,1.6,
     &  1.4,1.6,2.0,1.0,1.4,2.1,2.0,2.4,1.3,1.6,2.5,1.83,2.67,3.14,
     &  3.74,3.34,3.34,4.23,3.46,2.33/
c
c  freact1 is a reactivity parameter
c  (species order as in this routine).
c
      data freact1/0.,1.,1.,0.,0.1,0.1,0.,0.,0.,0.1,0.,0.,0.,0.3,0.5,
     &  1.,0.1,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0./
c
c  the following resistances are from Wesely (1989)
c
      data rmin/
     & 99999,60.  ,120. ,70.  ,130,100,99999,99999,80.  ,100. ,150. ,
     & 99999,99999,99999,99999,250,500,99999,99999,99999,99999,99999,
     & 99999,99999,99999,99999,250,500,99999,99999,99999,99999,99999,
     & 99999,99999,99999,99999,400,800,99999,99999,99999,99999,99999,
     & 99999,120. ,240. ,140. ,250,190,99999,99999,160. ,200. ,300. /
c
      data rcuts/
     & 99999,2000.,2000.,2000.,2000,2000.,99999,99999,2500,2000,4000,
     & 99999,9000.,9000.,9000.,4000,8000.,99999,99999,9000,9000,9000,
     & 99999,99999,9000.,9000.,4000,8000.,99999,99999,9000,9000,9000,
     & 99999,99999,99999,99999,6000,9000.,99999,99999,9000,9000,9000,
     & 99999,4000.,4000.,4000.,2000,3000.,99999,99999,4000,4000,8000/
c
      data rac/
     & 100,200,100,2000,2000,2000,0.5,0.5,300,150,200,
     & 100,150,100,1500,2000,1700,0.5,0.5,200,120,140,
     & 100,10.,100,1000,2000,1500,0.5,0.5,100,50.,120,
     & 100,10.,10.,1000,2000,1500,0.5,0.5,50.,10.,50.,
     5 100,50.,80.,1200,2000,1500,0.5,0.5,200,60,120./
c
      data rgss/
     & 400,150,350,500,500,100,0.5,1000,0.5,220,400,
     & 400,200,350,500,500,100,0.5,1000,0.5,300,400,
     & 400,150,350,500,500,200,0.5,1000,0.5,200,400,
     & 100,100,100,100,100,100,0.5,1000,100,100,50.,
     & 500,150,350,500,500,200,0.5,1000,0.5,250,400/
c
      data rgso/
     & 300,150.,200.,200.,200.,300.,2000,400,1000,180.,200.,
     & 300,150.,200.,200.,200.,300.,2000,400,800.,180.,200.,
     & 300,150.,200.,200.,200.,300.,2000,400,1000,180.,200.,
     & 600,3500,3500,3500,3500,3500,2000,400,3500,3500,3500,
     & 300,150.,200.,200.,200.,300.,2000,400,1000,180.,200./
c
      data rcls/
     & 99999,2000.,2000.,2000,2000,2000,99999,99999,2500,2000.,4000,
     & 99999,9000.,9000.,9000,2000,4000,99999,99999,9000,9000.,9000,
     & 99999,99999,9000.,9000,3000,6000,99999,99999,9000,9000.,9000,
     & 99999,99999,99999,9000,200.,400.,99999,99999,9000,99999,9000,
     & 99999,4000.,4000.,4000,2000,3000,99999,99999,4000,4000.,8000/
c
      data rclo/
     & 99999,1000,1000,1000,1000,1000,99999,99999,1000,1000,1000,
     & 99999,400.,400.,400.,1000,600.,99999,99999,400.,400.,400.,
     & 99999,1000,400.,400.,1000,600.,99999,99999,800.,600.,600.,
     & 99999,1000,1000,400.,1500,600.,99999,99999,800.,1000,800.,
     & 99999,1000,500.,500.,1500,700.,99999,99999,600.,800.,800./
c
      data tmax /40.,40.,40.,40.,40.,40.,40.,40.,40.,40.,40./
      data tmin / 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0./
      data topt /20.,20.,20.,20.,20.,20.,20.,20.,20.,20.,20./
 
c
c      write (*,*) 'nm=', nm
c      write (*,*) 'gsun=', gsun
c      write (*,*) 'ustrw=', ustrw
c      write (*,*) 'wsrf=', wsrf
c      write (*,*) 'tcellk=', tcellk
c      write (*,*) 'rhum=', rhum
c      write (*,*) 'lstrcl=', lstrcl
c      write (*,*) 'lsnocl=', lsnocl
c      write (*,*) 'flndcl=',flndcl
c      write (*,*) 'raincl=', raincl
c      write (*,*) 'rs=', rs
c      write (*,*) 'pcell=', pcell
cdiag
c       write(*,*) 'just enter micsurf routine'
c       write(*,*) 'nhen ',nhen
c       write(*,*) 'knh3, knaph, kpom1, kpom2'
c       write(*,*) knh3,knaph,kpom1,kpom2
c       write(*,*) 'khg0,khg2g,kdioxin,katraz'
c       write(*,*) khg0,khg2g,kdioxin,katraz
        call flush(6)
c
c   first time through only, set up species mapping
c
      if (lfirst) then
        do 25 l=1,mxgas
          if (kno .eq. l) lmap(l) = 7
          if (kno2 .eq. l) lmap(l) = 6
          if (kno2s .eq. l) lmap(l) = 6
          if (ko3 .eq. l) lmap(l) = 2
          if (kcarb .eq. l) lmap(l) = 11
          if (kvoc .eq. l) lmap(l) = 24
          if (kpna .eq. l) lmap(l) = 18
          if (kpan .eq. l) lmap(l) = 10
          if (khno2 .eq. l) lmap(l) = 5
          if (kh2o2 .eq. l) lmap(l) = 3
          if (khno3 .eq. l) lmap(l) = 4
          if (kso2 .eq. l) lmap(l) = 1
          if (ksv1 .eq. l) lmap(l) = 12
          if (ksv2 .eq. l) lmap(l) = 12
          if (ksv3 .eq. l) lmap(l) = 12
          if (ksv4 .eq. l) lmap(l) = 12
          if (knh3 .eq. l) lmap(l) = 9
          if (knaph .eq. l) lmap(l) = 25
          if (kpom1 .eq. l) lmap(l) = 26
          if (kpom2 .eq. l) lmap(l) = 27
          do nn = 1, mxhg
             if (khg0(nn) .eq. l) lmap(l) = 28
             if (khg2(nn) .eq. l) lmap(l) = 29
          end do
          if (kdiox .eq. l) lmap(l) = 30
          if (katraz .eq. l) lmap(l) = 31
          if (ksulf .eq. l) lmap(l) = 32
   25   continue
        lfirst = .false.
      endif
c
c    ** need to determine iseasn from month and snow cover **
c       iseasn = 1  summer
c                2  early autumn
c                3  late autumn after frost
c                4  winter with snow on ground
c                5  spring 
c 
       if(nm .ge. 11)then 
         iseasn=3
       elseif(nm .eq. 10)then
         iseasn=2
       elseif(nm.ge.6)then
         iseasn=1
       elseif(nm.ge.3)then
         iseasn=5
       else
         iseasn=3
       endif
       if(lsnocl.gt.0) iseasn = 4
c
c  call tdrate to get henry's law constants at the surface temp
c  ** still need to adjust for pressure!! **
c
cdiag
c     call flush(6)
c
      call tdrate(tcellk,pcell)
cdiag
c     call flush(6)
c
c   Calculate effective henry's Law coefficients f(T,pH)
c   assume leaf surface ph=7 (recomended by Wesely).
c
       hplus = 1.e-7
       hplus2 = hplus*hplus
c   
c   get h+ for pH 4.5 also, for use in rain wetted calculations
c
       hp45 = 10.**(-4.5)
       hp452 = hp45*hp45
c
cdiag      
       call flush(6)
c
       do 100 i = 1,nhen
         ttmp1 = kt(i)*aqrk1(i)/hplus
         ttmp2 = kt(i)*aqrk1(i)*aqrk2(i)/hplus2
         hheff(i) = kt(i) + ttmp1 + ttmp2
         ttmp1 = kt(i)*aqrk1(i)/hp45
         ttmp2 = kt(i)*aqrk1(i)*aqrk2(i)/hp452
         hheff45(i) = kt(i) + ttmp1 + ttmp2
c         if (i .eq. 32) then
c            write (*,*) 'hheff=', i, kt(i), ttmp1, ttmp2, aqrk1(i), aqrk2(i) 
c         end if
cdiag
c        call flush(i)
  100 continue
c
c  calculate solar correction factor for stomatal resistance
c
      gofs = 1. + (200./(gsun+0.1))**2
      rdc=100.*(1.+1000./(gsun+10.))
      heffso = hheff(1)
c
      tcell = tcellk - 273.0
cdiag
      call flush(6)
c
      do 205 lu = 1,mlus
c
c      calculate temperature correction factor for stomatal resistance
c
        if (tcell .lt. tmin(lu)+.01 .or.
     &      tcell .gt. tmax(lu)-.01) then
          goft = 1.e4
        else
          tmxmn = tmax(lu) - tmin(lu)
          tmxo = tmax(lu) - topt(lu)
          tomn = topt(lu) - tmin(lu)
          goft = (tomn/(tcell - tmin(lu)))
     &           * (tmxo/(tmax(lu) - tcell))**(tmxo/tmxmn)
        endif
c
cdiag
c       call flush(6)
c
c***DFG
c  Calculate stomatal resistance for NH3 for the barren land)
c  The index for NH3 is 9 and for the barren land is 8.
c
        rsnh3=dratio1(9)*rmin(8,iseasn)*gofs*goft       
c***DFG
c
        do 200 l = 1,nhen
cdiag
          call flush(6)
c
          rcut(lu,l)=rcuts(lu,iseasn)/(hheff(l)/heffso + freact1(l))
c
c       for now, stressed vegetation is set to 10 times stomatal
c       resistance otherwise calculated.  this may not be the 
c       most appropriate multiplier.
c
          if (lstrcl .eq. 1) then
            rstrf(lu,l)=dratio1(l)*rmin(lu,iseasn)*gofs*goft*10.
            if (l.eq.9) rstrf(lu,l)=rsnh3
          else
            rstom(lu,l)=dratio1(l)*rmin(lu,iseasn)*gofs*goft
          endif
cdiag
c         call flush(6)
  200   continue
c
cdiag 
  205 continue
c
      do 280 l=1,nhen
        rmes(l) = 1./(hheff(l)/3000. + 100.*freact1(l))
        do 270 lu=1,mlus
          if(flndcl(lu) .lt. 1.0e-3) then
            rsurf(lu,l) = 1.e10
            go to 270
          endif
          rcl(lu,l)=1./(hheff(l)/(heffso*rcls(lu,iseasn))+freact1(l)/
     &                rclo(lu,iseasn))
          rgs(lu,l)=1./(hheff(l)/(heffso*rgss(lu,iseasn))+freact1(l)/
     &                rgso(lu,iseasn))
c
c
c   check for presence of dew
c
          rtest = (100. - rhum) * (wsrf(lu)+0.6)
          ldew = .false.
          if (rtest .lt. 19.) ldew = .true.
c
c     note that the order of the following checks is important.
c     that is, water surface takes priority over rain wetted which
c     takes priority over dew wetted which takes priority over 
c     regular surface.
c
c        water surface
c
          if (lu .eq. 7) then
            rsurf(lu,l) = 1./(hheff(l)*0.082*tcellk*4.8e-4*ustrw)
c
c        rain wetted surface
c
          else if (raincl .ge. 0.01) then
            if (l .eq. 1) then
              if (lu .eq. 1) then
                rcutwet(lu,l) = 50.
              else
                rcutwet(lu,l) = 1./(1./5000. + 1./(3.*rcut(lu,l)))
              endif
            else if (l .eq. 2) then
              rcutwet(lu,l) = 1./(1./1000. + 1./(3.*rcut(lu,l)))
            else
              rcutwet(lu,l) = 1./(1./(3.*rcut(lu,l))+1e-7*hheff(l) +
     &                    freact1(l)/rcutwet(lu,2))
            endif
            rsurf(lu,l)=1./(1./(3.*rstom(lu,l)+rmes(l))+1/rcutwet(lu,l)
     &                +1/(rdc+rcl(lu,l))+1/(rac(lu,iseasn)+rgs(lu,l)))
c
c  calculate resistance based on solubility and use smaller of this
c  and the above resistance
c
            rs2 = 1./(hheff45(l)*0.082*tcellk*4.8e-4*ustrw)
            rsurf(lu,l) = amin1(rsurf(lu,l),rs2)
c
c        dew wetted surface
c
          else if (ldew) then
            if (l .eq. 1) then
              if (lu .eq. 1) then
                rcutwet(lu,l) = 50.
              else
                rcutwet(lu,l) = 100.
              endif
            else if (l .eq. 2) then
              rcutwet(lu,l) = 1./(1./3000. + 1./(3.*rcut(lu,l)))
            else
              rcutwet(lu,l) = 1./(1./(3.*rcut(lu,l))+1e-7*hheff(l) +
     &                    freact1(l)/rcutwet(lu,2))
            endif
            rsurf(lu,l)=1./(1./(3.*rstom(lu,l)+rmes(l))+1/rcutwet(lu,l)
     &                +1/(rdc+rcl(lu,l))+1/(rac(lu,iseasn)+rgs(lu,l)))
c
c  calculate resistance based on solubility and use smaller of this
c  and the above resistance
c
            rs2 = 1./(hheff(l)*0.082*tcellk*4.8e-4*ustrw)
            rsurf(lu,l) = amin1(rsurf(lu,l),rs2)
c
c        regular dry surface
c
          else
            if (lstrcl .le. 0) then
              rsurf(lu,l)=1./(1./(rstom(lu,l)+rmes(l))+1./rcut(lu,l)+
     &           1./(rdc+rcl(lu,l))+1./(rac(lu,iseasn)+rgs(lu,l)))
            else if (lstrcl .eq. 1) then
              rsurf(lu,l)=1./(1./(rstrf(lu,l)+rmes(l))+1./rcut(lu,l)+
     &           1./(rdc+rcl(lu,l))+1./(rac(lu,iseasn)+rgs(lu,l)))
            else if (lstrcl .ge. 2) then
              rsurf(lu,l)=1./(1./(10000.+rmes(l))+1./rcut(lu,l)+
     &           1./(rdc+rcl(lu,l))+1./(rac(lu,iseasn)+rgs(lu,l)))
            endif
c
          endif
  270   continue
  280 continue
c
c
      do 305 n=1,mlus
        do 301 l=1,ngas
c
c  set rsurface very high for non-depositing species 
c
c  added default tcm 8/18/00
          rs(l,n) = 1.e20
c
          ld = lmap(l)
          if (ld .gt. 0) rs(l,n) = rsurf(n,ld)
c          if (n .eq. 7) then
c             write (*,*) 'within micsurf, n,l,rs(l,n)', n,l,rs(l,n),ld,lmap(l)
c          end if
cdiag
c         write(*,*) 'l,lmap,ld,n ',l,lmap(l),id, n, rs(l,n)
c         call flush(6)
c
  301   continue
  305 continue
cdiag
c      call flush(6)
c
c      write (*,*) 'end of micsurf'
c      write (*,*) 'nm=', nm
c      write (*,*) 'gsun=', gsun
c      write (*,*) 'ustrw=', ustrw
c      write (*,*) 'wsrf=', wsrf
c      write (*,*) 'tcellk=', tcellk
c      write (*,*) 'rhum=', rhum
c      write (*,*) 'lstrcl=', lstrcl
c      write (*,*) 'lsnocl=', lsnocl
c      write (*,*) 'flndcl=',flndcl
c      write (*,*) 'raincl=', raincl
c      write (*,*) 'rs=', rs
c      write (*,*) 'pcell=', pcell
c
      return
      end
