c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine phot(klyr,nlayer,zenith,zht,avterr,cloud,rato3,ratno2)
c
c-----ATDM v1.00
c
c     Determines the NO2 and O3 photolysis rate constants (1/min)
c     as a function of height and solar zenith angle -- adjustments are made
c     for cloud cover following RADM (Chang, et al., JGR Vol. 92, 1987)
c
c-----95/08/16:
c     C Emery & A Kuklin SAI
c-----95/08/30:
c     -removed cloud averaging by sigma depth; now averaging by physical depth
c-----95/10/06:
c     -now passed zht (m AGL) and avterr (m MSL); fixed erroneous depth
c      calculations
c-----95/10/17:
c     -revised calculation of transmission coefficient trans to be a function
c      of optical depth and cloud cover in each layer, rather than layer-
c      weighted cloud cover and total optical depth; fixed some coding errors
c-----96/10/22:dfg
c     -set zenith >= 87 to set photolysis rate equal to zero.
c-----96/11/20:DFG
c     -removed the fcld calculations which follows method from Chang et al i
c      (1987).
c     -use cloud adjustment factors from UAM-V.  Set cloud cover to be the
c      maximum value from current layer to the top.
c
c     Input arguments:
c               klyr:           current layer index
c               nlayer:         number of layers in column
c               zenith:         solar zenith angle (degrees)
c               zht:            profile of layer interface heights (m AGL)
c               avterr:         terrain elevation (m MSL)
c               cloud:          fractional cloud cover profile (0-1)
c     Output arguments:
c               rato3:          o3 photolysis rate (1/min)
c               ratno2:         no2 photolysis rate (1/min)
c     Called by:
c               rchem     rchemf
c
      dimension zht(nlayer), cloud(nlayer)
      dimension za(10), alt(16),cldadj(11)
      dimension photno2(10,16), photo3(10,16)
      data za /0.,10.,20.,30.,40.,50.,60.,70.,78.,86./
      data alt /0.,.15,.36,.64,.98,.138,.184,.235,.291,3.53,4.21,6.,8.,
     &          10.,12.,14./
      data cwat /1./, rhoh2o /1.e6/, rdrop /1.e-5/
      data alfno2 /1.2/, alfo3 /0.7/
c
c  cloud adjustment factors from RADM (entered, 3/13/95, tcm)
      data cldadj/1.0,0.96,0.92,0.88,0.84,0.80,0.76,0.72
     &               ,0.68,0.64,0.59/

c
      data photno2
     1/.5142,.5089,.4926,.4646,.4235,.3668,.2908,.1913,.09841,.02578,
     2 .5522,.5469,.5306,.5025,.4611,.4037,.3255,.2208,.1190,.03405,
     3 .5746,.5693,.5533,.5257,.4847,.4275,.3489,.2414,.1333,.03908,
     4 .5954,.5903,.5746,.5476,.5074,.4508,.3722,.2625,.1483,.04398,
     5 .6150,.6100,.5947,.5683,.5289,.4732,.3950,.2836,.1637,.04884,
     6 .6340,.6291,.6142,.5885,.5499,.4951,.4174,.3048,.1798,.05386,
     7 .6528,.6481,.6336,.6084,.5707,.5167,.4397,.3262,.1965,.05918,
     8 .6714,.6668,.6527,.6281,.5911,.5381,.4617,.3475,.2136,.06478,
     9 .6901,.6856,.6718,.6477,.6115,.5593,.4836,.3690,.2313,.07077,
     O .7091,.7047,.6912,.6677,.6322,.5810,.5061,.3912,.2500,.07737,
     1 .7281,.7238,.7107,.6878,.6531,.6029,.5290,.4143,.2700,.08479,
     2 .7709,.7669,.7546,.7332,.7007,.6532,.5826,.4700,.3213,.1067,
     3 .8081,.8044,.7931,.7733,.7433,.6992,.6330,.5255,.3776,.1378,
     4 .8363,.8329,.8225,.8042,.7765,.7358,.6745,.5738,.4313,.1770,
     5 .8572,.8541,.8445,.8277,.8021,.7646,.7081,.6149,.4806,.2234,
     6 .8726,.8697,.8607,.8451,.8215,.7869,.7349,.6490,.5245,.2743/
c
      data photo3
     1/.1964E-2,.1891E-2,.1681E-2,.1362E-2,.9810E-3,.5969E-3,.2772E-3,
     & .7931E-4,.1557E-4,.1104E-5,
     2 .2150E-2,.2072E-2,.1848E-2,.1506E-2,.1095E-2,.6774E-3,.3245E-3,
     & .9999E-4,.2318E-4,.2376E-5,
     3 .2278E-2,.2197E-2,.1963E-2,.1607E-2,.1176E-2,.7348E-3,.3581E-3,
     & .1142E-3,.2817E-4,.3211E-5,
     4 .2411E-2,.2326E-2,.2084E-2,.1713E-2,.1261E-2,.7959E-3,.3939E-3,
     & .1292E-3,.3331E-4,.4067E-5,
     5 .2548E-2,.2460E-2,.2208E-2,.1822E-2,.1350E-2,.8598E-3,.4317E-3,
     & .1450E-3,.3857E-4,.4943E-5,
     6 .2690E-2,.2600E-2,.2339E-2,.1937E-2,.1443E-2,.9273E-3,.4718E-3,
     & .1617E-3,.4408E-4,.5854E-5,
     7 .2840E-2,.2746E-2,.2475E-2,.2057E-2,.1542E-2,.9986E-3,.5145E-3,
     & .1796E-3,.4985E-4,.6805E-5,
     8 .2993E-2,.2896E-2,.2615E-2,.2181E-2,.1643E-2,.1073E-2,.5591E-3,
     & .1983E-3,.5579E-4,.7773E-5,
     9 .3150E-2,.3050E-2,.2759E-2,.2308E-2,.1748E-2,.1150E-2,.6059E-3,
     & .2181E-3,.6193E-4,.8758E-5,
     O .3313E-2,.3210E-2,.2909E-2,.2441E-2,.1858E-2,.1231E-2,.6558E-3,
     & .2393E-3,.6842E-4,.9771E-5,
     1 .3482E-2,.3374E-2,.3064E-2,.2579E-2,.1972E-2,.1317E-2,.7091E-3,
     & .2623E-3,.7533E-4,.1081E-4,
     2 .3886E-2,.3771E-2,.3438E-2,.2915E-2,.2256E-2,.1532E-2,.8467E-3,
     & .3236E-3,.9372E-4,.1337E-4,
     3 .4295E-2,.4174E-2,.3819E-2,.3262E-2,.2553E-2,.1765E-2,.1001E-2,
     & .3970E-3,.1167E-3,.1621E-4,
     4 .4684E-2,.4557E-2,.4185E-2,.3597E-2,.2844E-2,.1998E-2,.1163E-2,
     & .4789E-3,.1444E-3,.1944E-4,
     5 .5085E-2,.4952E-2,.4562E-2,.3945E-2,.3149E-2,.2246E-2,.1340E-2,
     & .5736E-3,.1789E-3,.2354E-4,
     6 .5541E-2,.5402E-2,.4993E-2,.4342E-2,.3499E-2,.2533E-2,.1548E-2,
     & .6899E-3,.2241E-3,.2930E-4/
c
c-----Determine table indices for given zenith and zht
c
      if (zenith.ge.87.) then
        rato3 = 0.
        ratno2 = 0.
        return
      endif
      do 10 n = 1,9
        if (zenith.ge.za(n) .and. zenith.lt.za(n+1)) then
          zdif1 = abs(zenith-za(n))
          zdif2 = abs(zenith-za(n+1))
          if (zdif1.le.zdif2) then
            iz = n
          else
            iz = n + 1
          endif
          goto 11
        endif
  10  continue
      iz = 10
  11  continue
      zmid = zht(klyr)/2. + avterr
      if (klyr.gt.1) zmid = (zht(klyr) + zht(klyr-1))/2. + avterr
      zmid = zmid/1000.
      do 20 n = 1,15
        if (zmid.ge.alt(n) .and. zmid.lt.alt(n+1)) then
          zdif1 = abs(zmid-alt(n))
          zdif2 = abs(zmid-alt(n+1))
          if (zdif1.le.zdif2) then
            ia = n
          else
            ia = n + 1
          endif
          goto 21
        endif
  20  continue
      ia = 16
  21  continue
c
c-----Load temporary clear sky photolysis rates
c
      ratno2 = photno2(iz,ia)
      rato3 = photo3(iz,ia)
c
c-----Adjust for cloud cover using the constant factor for each coverage.
c
      cldmax = 0.
      do 85 kk = klyr,nlayer
        cldmax = amax1(cldmax,cloud(kk))
  85  continue
c
      icld=int(cldmax*10.0+1.5)
      if (icld.gt.11) icld=11
      if (icld.lt.1) icld=1
c
      fcldno2=cldadj(icld)
      fcldo3=cldadj(icld)
c

c     Check for clouds above current layer
c
c     lblw = 0
c     linc = 0
c     labv = 0
c     zang = 3.1415927*amin1(60.,zenith)/180.
c     if (klyr.eq.nlayer) goto 40
c     tau = 0.
c     do 30 k = klyr+1,nlayer
c       dz = zht(k) - zht(k-1)
c       if (cloud(k).ge.0.1)
c    &    tau = tau + cloud(k)*3.*cwat*dz/(2.*rhoh2o*rdrop)
c 30  continue
c     if (tau.gt.5.) then
c       trans = (5. - exp(-tau))/(4. + 3.*tau*(1-0.86))
c       fcldno2 = 1.6*trans*cos(zang)
c       fcldo3 = fcldno2
c       goto 60
c     endif
c
c-----No clouds above; check for clouds in current layer
c
c 40  if (cloud(klyr).ge.0.1) then
c       fcldno2 = cloud(klyr)*1.4*cos(zang)
c       fcldo3 = fcldno2
c       goto 60
c     endif
c
c-----No clouds in current layer; check for clouds below current level
c
c     if (klyr.eq.1) return
c     tau = 0.
c     do 50 k = 1,klyr-1
c       dz = zht(k)
c       if (k.gt.1) dz = zht(k) - zht(k-1)
c       if (cloud(k).ge.0.1)
c    &    tau = tau + cloud(k)*3.*cwat*dz/(2.*rhoh2o*rdrop)
c 50  continue
c     if (tau.le.5.) return
c     trans = (5. - exp(-tau))/(4. + 3.*tau*(1-0.86))
c     fcldno2 = 1. + alfno2*(1. - trans)*cos(zang)
c     fcldo3 = 1. + alfo3*(1. - trans)*cos(zang)
c
c-----Clouds are present; modify clear-sky photolysis rates
c
  60  continue
      ratno2 = ratno2*fcldno2
      rato3 = rato3*fcldo3
c
      return
      end
