c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine rchem(deltat,time,idate,idebug)
c     
c-----ATDM v1.00
c     
c     Chemistry driver for the coarse domain
c     
c-----95/06/16:
c     C Emery SAI
c-----95/10/05:
c     -revised calculation of PM10 for toxic partitioning to be in units
c     of ug/m3 rather than ppm
c     -reworked aqueous chemistry code, cleaned up some errors
c-----95/10/06:
c     -removed zmsl array; passing PHOT zcell and avterr
c-----95/10/18:
c     -restructured for more efficient handling of lwet and rain 
c-----95/10/26:
c     -revised calculation of Hg0 -> HgII aqueous reaction
c-----95/12/08:gem
c     -fixed index on bdnl from k to l
c-----95/12/11:gem
c     -changed pm10=0. to pm10=4. (bckgrnd pm10)
c-----95/12/13:
c     -added check against lower bnd after call to nitrat (now replaced by remeql3: adopted from CMAQ)
c-----95/12/22:
c     -added branch around convmix for PM runs, calls wetdep from here
c     (quick fix to avoid problems in convmix with strong convective
c     columns with low cloud base)
c-----96/08/01:
c     -moved check of ierr from convmix inside if-then-else loop
c-----97/03/24:DFG
c     -passed k to nitrat subroutine. (nitrat is now removed by remeql3)
c-----97/06/25:DFG
c     -passed pressure (i.e. pecll(k)) to nitrat subroutine.(nitrat is now removed by remeql3)
c     
c     Rebuilt to contain the cloud sub-model; for those columns with
c     precipitation ( > 0.01 in/hr), the vertical concentration profile
c     is split between an in-cloud profile and non-cloud profile.  Cloud 
c     mixing and wet deposition act on the in-cloud profile over multiple
c     sub-steps, after which the two profiles are recombined.  PM and/or
c     toxic chemistry is then performed on the combined profile.
c     
c     Input arguments:
c     deltat:         model time step (min)
c     time:           model time
c     idate:          model date
c     idebug:         diagnostic debug flag
c     Files written:
c     6:              standard output file unit (*)     
c     Routines called:
c     jtocal    remeql3   toxpart   convmix   strtmix  solar
c     phot      getoh     partchm   toxchm
c     Called by:
c     atdm
c     
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'ohtbl.cmd'
      include 'aerosl.cmd'
      include 'balanc.cmd'
      include 'ctlflg.cmd'
      include 'latlon.cmd'
c     
      dimension aradcon(mxrads,mxz)
      dimension cincld(mxz,mxspec), cnocld(mxz,mxspec)
      dimension acincld(mxz),acnocld(mxz)
      dimension zcell(mxz),tcell(mxz),pcell(mxz),qcell(mxz),rhcell(mxz)
      dimension ccell(mxz)
      dimension wi(mxz), delhg0(mxz), delgso4(mxz)
      dimension pm(7), tox(5)
      dimension lqdh2o(mxz)
      dimension cnew(mxz)
      dimension cold(mxz)
      real*4 lwch2o(mxx,mxy,mxz),clwch2o(mxx,mxy,mxz)
      logical lprnt, lpro3
      real nh3, nh4n, nh4s, lv, mfac
      real*4 lqdh2o
      save ndone
      data eps /0.622/, lv /2.5e6/, e0 /6.11/, rv /461./
      data tdepo3 /-2300./, rkhg0 /2.31e+8/
c      data lwc /1./, ekho30 /1.13e-2/, tdepo3 /-2300./, rkhg0 /2.31e+8/
      data pm /7*0./, tox /5*0./
      data pi /3.1415927/
      data ndone/0/
c     
      write(*,3001) deltat
 3001 format(' enter rchem  ',f10.2)
c     
c-----Bit nine of idebug (i.e., the 256 location) 
c     will turn on debug writes in rchem.
c     
      idiag = and(idebug,256)
c     lprnt = .false.
c     if (idiag.ne.0) lprnt = .true. 
c     
c-----Extract date
c     
      if (idate.lt.100000) then
         call jtocal(idate,icdate)
         nyr = icdate/10000
         nmth = icdate/100 - nyr*100
         nday = icdate - nmth*100 - nyr*10000
      else
         nyr = idate/10000
         nmth = idate/100 - nyr*100
         nday = idate - nmth*100 - nyr*10000
      endif
      nyr = nyr + 1900
c
c for given time interval, calculate the liquid water content for entire domain
c     
      do i = 1, nox
         do j = 1, noy
            do k = 1, nlayer
               call alwc(clh2o(i,j,k),rnh2o(i,j,k),press(i,j,k),
     *                  temper(i,j,k),lwch2o(i,j,k),clwch2o(i,j,k))
            end do
         end do
      end do
c     
c-----Loop on grids
c     
      do 100 j = 1,noy
         if (istr(j).ge.nox) goto 100
         i1 = istr(j) + 1
         i2 = iend(j) - 1
         do 99 i = i1,i2
c
            do ijk = 1, nlayer
               lqdh2o(ijk)= clwch2o(i,j,ijk)
            end do
c
            if (j.le.jstr(i)) goto 99
            if (j.ge.jend(i)) goto 99
c     
c-----Skip cells covered by fine grid
c     
            do 10 ifine = 1,numfin
               if (i.ge.ixfb(ifine) .and. i.le.ixfe(ifine) .and.
     &              j.ge.jyfb(ifine) .and. j.le.jyfe(ifine)) goto 99
 10         continue
c     
c-----Load some ambient column profiles
c     
            lprnt = .false.
            if (i.eq.21 .and. j.eq.32. .and. idiag.ne.0) lprnt = .true.
            if (lprnt) write(*,*)'z,t,p,q,c,rh'
            do 20 k = 1,nlayer
               zcell(k) = h(i,j,k)
               tcell(k) = temper(i,j,k)
               pcell(k) = press(i,j,k)
               qcell(k) = ch2o(i,j,k)
               ccell(k) = cloud(i,j,k)
               h2o = 1.e6*qcell(k)*28.8/18. 
               ev = qcell(k)*pcell(k)/(qcell(k) + eps)
               es = e0*exp((lv/rv)*(1./273. - 1./tcell(k)))
               rhcell(k) = 100.*ev/es
               if (lprnt) write(*,'(6e10.3)') zcell(k),tcell(k),
     &              pcell(k),qcell(k),ccell(k),rhcell(k)
 20         continue
            if ( lprnt ) then
               write(*,*) ' Enter rchem ...'
               do iaa=1,nospec
                  write(*,'(6e12.5)') (conc(21,32,iaa,kaa),kaa=1,nlayer)
               enddo
            endif
c     
c-----Determine nitrate equilibrium if this is a PM chemistry run, or if
c     its a Toxic chemistry run with nitrates
c     
               do 30 k = 1,nlayer
                  if (lpmchm) then
                     if (lpmchm .or. (ltxchm .and. khno3.ne.0)) then
c
c               aso4 and gso4 should not be modified by remeql3
c               so calculation of fractions and redistribution
c               of sulfate has been removed.
c               since prior versions of REMSAD have assumed
c               that all pariculate nitrate is nh4no3,
c               we will assume that here also.  This means
c               that we do not need to calculate the fraction
c               of nh4 as nitrate (frnh4n) before remeql3.
c               The assumption that this fraction would be
c               the same after remeql3 is not correct anyway.
c               (tcm, 10/18/00)
                     tso4  = conc(i,j,kaso4,k) + conc(i,j,kgso4,k)
                     thno3 = conc(i,j,khno3,k)
                     tnh3o = conc(i,j,knh3,k)
                     pno3  = conc(i,j,kpno3,k)
                     panh4 = conc(i,j,knh4n,k) + conc(i,j,knh4s,k)
Ctcm                 frnh4n= conc(i,j,knh4n,k)/panh4
Ctcm                 paso4 = conc(i,j,kaso4,k) + conc(i,j,kgso4,k)
Ctcm                 fraso4= conc(i,j,kaso4,k)/paso4
                     call remeql3(tso4,thno3,tnh3o,rhcell(k),tcell(k),
     &                    pcell(k),pno3,panh4,paso4,i,j,k)
c     
c     check against lower bound for these species
c     
                     conc(i,j,khno3,k) = amax1(thno3,bdnl(khno3))
                     conc(i,j,knh3,k)  = amax1(tnh3o,bdnl(knh3))
Ctcm                 conc(i,j,kaso4,k) = amax1(paso4*fraso4,bdnl(kaso4))
Ctcm                 conc(i,j,kgso4,k) = amax1(paso4*(1-fraso4),bdnl(kgso4))
                     conc(i,j,kpno3,k) = amax1(pno3,bdnl(kpno3))
Ctcm                 conc(i,j,knh4n,k) = amax1(panh4*frnh4n,bdnl(knh4n))
Ctcm                 conc(i,j,knh4s,k) = amax1(panh4*(1-frnh4n),bdnl(knh4s))
                     conc(i,j,knh4n,k) = amax1(pno3,bdnl(knh4n))
                     conc(i,j,knh4s,k) = amax1(panh4-pno3,bdnl(knh4s))
                  endif
               end if
c     
c-----Partition certain toxic species between vapor and particle phase
c     
               if (ltxchm) then
                  pm10 = 4.
                  mfac = cpfac*(1013.25/pcell(k))*(tcell(k)/273.)
                  do 35 l = ngas+1,nospec
                     if (l.eq.kpom1p .or. l.eq.kpom2p .or. l.eq.kdioxp .or.
     &                    l.eq.katrazp) goto 35
                     lsp = jind(l)
                     pm10 = pm10 + molwt(lsp)*conc(i,j,l,k)/mfac
 35               continue
c     if (lprnt) write(*,*)'PM10: ',pm10
                  if (kpom1.ne.0)
     &                 call toxpart(pm10,tcell(k),conc(i,j,kpom1,k),
     &                 conc(i,j,kpom1p,k),1)
                  if (kpom2.ne.0)
     &                 call toxpart(pm10,tcell(k),conc(i,j,kpom2,k),
     &                 conc(i,j,kpom2p,k),2)
                  if (kdiox.ne.0)
     &                 call toxpart(pm10,tcell(k),conc(i,j,kdiox,k),
     &                 conc(i,j,kdioxp,k),3)
                  if (katraz.ne.0)
     &                 call toxpart(pm10,tcell(k),conc(i,j,katraz,k),
     &                 conc(i,j,katrazp,k),4)
               endif
 30         continue
c
            ndone = ndone + 1
c
            do k = 1, nlayer
               airfact = press(i,j,k)/(1013.25*0.08206*temper(i,j,k))
               airden = airfact*(1.0e-3)*(6.02e+23)
               if (lo3) then
                 ato3 = ozone(i,j,k)
                 ath2o2 = hydro(i,j,k)
                 atoh = ohcon(i,j,k)
                 atho2 = ho2con(i,j,k)
                 atno3 = no3con(i,j,k)
                 so2av = aso2con(i,j,k)
                 atpec = peccon(i,j,k)
               else
                 ato3 = conc(i,j,ko3,k)
                 ath2o2 = conc(i,j,kh2o2,k)
                 atpec = conc(i,j,kpec,k)
                 atoh = aradcon(koh,k)
                 atho2 = aradcon(kho2,k)
                 atno3 = aradcon(kno3,k)
               endif
               atcl20 = cl2con(i,j)
               hmax = 2000.
               if (atcl20 .gt. 1.e-4) hmax = 1000.
               h2 = h(i,j,k)
               if (k .gt. 1) then
                 h1 = h(i,j,k-1)
               else
                 h1 = 0.
               endif
               call solar(ctlat(i,j),-ctlon(i,j),tz,nyr,nmth,nday,time,zcoef)
               zenith = (180./pi)*acos(zcoef)
               if (zenith .lt. 85.) atcl20 = 0.0
               if (h1 .gt. hmax) then
                 atcl2 = 0.
               else
                 atcl22 = (hmax - amin1(hmax,h2))/hmax * atcl20
                 atcl21 = (hmax - h1)/hmax * atcl20
                 atcl2 = (amin1(hmax,h2)-h1)/(h2-h1)*.5*(atcl22+atcl21)
               endif
c
               lprnt = .false.
               if (mod(ndone,100000) .eq. 0) then
                 if (mod(k,4) .eq. 1) lprnt = .true.
               endif
               do ihg = 1,mxhg
                 if (khg0(ihg) .ne. 0) then
                   call hgchem(i,j,k,ihg,deltat,airden,lwch2o(i,j,k),
     &                        atoh,atho2,atno3,ath2o2,ato3,so2av,atcl2,
     &                        atpec,lprnt)
                 endif
               end do
               lprnt = .false.
            end do
c     
c-----Do cloud mixing/wet dep in a loop over species
c     Perform cloud aqueous chemical conversions while loading cloud 
c     and non-cloud concentration profiles
c     
            fracp = amax1(fconv(i,j),fstrt(i,j))
            if (lprnt) write(*,*)'fracp: ',fracp
c     
            do l = 1,nospec
               do k = 1,nlayer
                  cnocld(k,l) = conc(i,j,l,k)
                  cincld(k,l) = conc(i,j,l,k)
               enddo
            enddo
c     
            do 40 l = 1,nospec
               lsp = jind(l)
               if (lprnt) write(*,*)' l,lsp :',l,lsp
               do 41 k = 1,kfrz(i,j)-1
                  frac1 = amax1((ccell(k) - fracp),0.)/
     &                 (1. - fracp + 1.e-10) 
                  frac2 = 0.
                  if (fracp.ne.0.) then
                     if (k.ge.kcldb(i,j) .and. k.le.kcldt(i,j)) then
                        frac2 = 1.
                     else
                        frac2 = amin1(ccell(k),fracp)/fracp
                     endif
                  endif
c     
                  tfac = 1./298. - 1./tcell(k)
                  patm = pcell(k)/1013.25
                  if (fracp.eq.0. .and. ccell(k).eq.0.) goto 41
                  if (lwet) wi(k) = wrat(i,j,k)
c
                  if (l.eq.kgso4) then
                     if (kgso4.lt.kaso4)
     &                    delgso4(k) = -fnucl*conc(i,j,l,k)
                     cnocld(k,l) = cnocld(k,l) + frac1*delgso4(k)
                     cincld(k,l) = cincld(k,l) + frac2*delgso4(k)
                     if (lprnt) then
                        write(*,*)'l,k,frac1,frac2,kfrz',
     &                       l,k,frac1,frac2,kfrz(i,j)
                        write(*,*)'GSO4:delc,conc',delgso4(k),conc(i,j,l,k)
                        write(*,*)'nocld,incld',cnocld(k,l),cincld(k,l)
                     endif
                  elseif (l.eq.kaso4) then
                     if (kgso4.gt.kaso4)
     &                    delgso4(k) = -fnucl*conc(i,j,kgso4,k)
                     cnocld(k,l) = cnocld(k,l) - frac1*delgso4(k)
                     cincld(k,l) = cincld(k,l) - frac2*delgso4(k)
                     if (lprnt) then
                        write(*,*)'ASO4:delc',delgso4(k)
                        write(*,*)'nocld,incld',cnocld(k,l),cincld(k,l)
                     endif
                  endif
 41            continue
c     
c-----------------------------------------------------------------------
c-----If there is rain in this column, invoke the cloud sub-model
c     
               do k = 1, nlayer
                  acincld(k) = cincld(k,l)
                  acnocld(k) = cnocld(k,l)
               end do
c
               if (l .eq. ko3) then
                  lpro3 = .false.
               else
                  lpro3 = .false.
               end if
               deltadp = 0.
               if (lwet .and. rain(i,j).ge.0.01) then
c     
c-----Convective precipitation
c     
                  if (fconv(i,j).gt.0.) then
c                    if ( ltxchm .or. lpmchm ) then
                        call convmix(lprnt,nlayer,l,kcldb(i,j),kcldt(i,j),
     &                       kfrz(i,j),deltat,deltax(j),deltay,pstar(i,j),
     &                       xmd(i,j),rain(i,j),sigma,zcell,tcell,pcell,
     &                       wi,acincld,deltadp,idebug,ierr,lqdh2o,i,j,lpro3)
                        if (ierr.ne.0) then
                           write(*,*)'Stopping in RCHEM at i,j',i,j
                           write(*,*)'Too many convective sub-steps'
                           stop
                        endif
c                    endif
                     deltadp = fconv(i,j)*deltadp
c     
c-----Stratiform precipitation
c     
                  elseif (fstrt(i,j).gt.0.) then
                     call strtmix(lprnt,nlayer,l,kcldb(i,j),kcldt(i,j),
     &                    kfrz(i,j),deltat,deltax(j),deltay,rain(i,j),
     &                    pstar(i,j),sigma,zcell,tcell,pcell,qcell,
     &                    wi,acincld,deltadp,idebug,lqdh2o,i,j,lpro3)
                     deltadp = fstrt(i,j)*deltadp
                  else
                     write(*,*)'ERROR (RCHEM): '
                     write(*,*)'Non-zero rain, but zero cloud fractions!'
                     write(*,*)'i,j,rain,fconv,fstrt'
                     write(*,*) i,j,rain(i,j),fconv(i,j),fstrt(i,j)
                     stop
                  endif
                  dpflow(l) = dpflow(l) - deltadp
                  wposit(i,j,l) = wposit(i,j,l) + molwt(lsp)*deltadp/
     &                 (deltay*deltax(j))
c                 if (i.eq. 40 .and. j .eq. 40) then
c                   write(*,*) 'In rchem, l, lsp, molwt(lsp): '
c                   write(*,*) l, lsp, molwt(lsp)
c                 endif
c     
               endif
c     
c-----Recombine in-cloud profiles with non-cloud profiles
c     
               do k = 1, nlayer
                  cincld(k,l) = acincld(k)
                  cnocld(k,l) = acnocld(k)
               end do
c
c              dmass = 0.
c              mfac = cpfac*(1013.25/pcell(k))*(tcell(k)/273.)
c              mfac = pstar(i,j)/(10.*9.8*28.8)
               do 45 k = 1,nlayer
c                 dsigma = sigma(k) - sigma(k+1)
c                 vfac = 1.e6*deltax(j)*deltay*dsigma
c                 cnew(k)=fracp*cincld(k,l) + (1. - fracp)*cnocld(k,l)
c                 dmass = dmass + (conc(i,j,l,k)-cnew(k))*1.e6/mfac
c                 dmass = dmass + (conc(i,j,l,k)-cnew(k))*mfac*vfac
c                 cold(k) = conc(i,j,l,k)
c                 conc(i,j,l,k) = cnew(k)
                  conc(i,j,l,k) = fracp*cincld(k,l) + (1. - fracp)*cnocld(k,l)
                  if (lprnt .and. fracp.ne.0.) then
                     write(*,*)'k,incld,nocld',k,cincld(k,l),cnocld(k,l)
                     write(*,*)'conc',conc(i,j,l,k)
                  endif
 45            continue
c              if (deltadp .gt. 0.) then
c              if (abs(dmass-deltadp)/deltadp .gt. .01) then
c              if (dmass .gt. 1.e-5) then
c              if ((cnew(1)-cold(1))/cold(1) .gt. 0.2) then
c                write(*,*) 'Wet dep not equivalent to conc change'
c                write(*,*) 'i,j,l,fracp: '
c                write(*,*) i,j,l,fracp
c                write(*,*) 'cold(k): '
c                write(*,*) (cold(k),k=1,nlayer)
c                write(*,*) 'cnew(k): '
c                write(*,*) (cnew(k),k=1,nlayer)
c                write(*,*) 'acincld(k): '
c                write(*,*) (acincld(k),k=1,nlayer)
c                write(*,*) 'acnocld(k): '
c                write(*,*) (acnocld(k),k=1,nlayer)
c                write(*,*) 'deltadp, dmass: '
c                write(*,*) deltadp, dmass
c                write(*,*) 'mfac, cpfac: '
c                write(*,*) mfac, cpfac
c                write(*,*) 'pcell(k),tcell(k): '
c                write(*,*) (pcell(k),tcell(k),k=1,nlayer)
c              endif
c              endif
c     
 40         continue
c     
c     if ( lprnt ) then
c     write(*,*) ' After cloud module ...'
c     do iaa=1,nospec
c     write(*,'(6e12.5)') (conc(21,32,iaa,kaa),kaa=1,nlayer)
c     enddo
c     endif
c-----------------------------------------------------------------------
c-----Perform chemistry for PM
c     
c-----Calculate solar zenith angle
c     
            if (lprnt) then
               write(*,*)'lat,lon,zone,yr,mo,dy,t'
               write(*,*) ctlat(i,j),ctlon(i,j),tz,nyr,nmth,nday,time
            endif
               call solar(ctlat(i,j),-ctlon(i,j),tz,nyr,nmth,nday,time,zcoef)
               zenith = (180./pi)*acos(zcoef)
c     if (lprnt) write(*,*)'zenith,zcoef ',zenith,zcoef
               
c-----Do chemistry in each layer
c     
               do 50 k = 1,nlayer
c     
c-----Obtain current NO2 and O3 photolysis rates from lookup table
c     
                  call phot(k,nlayer,zenith,zcell,avterr(i,j),ccell,
     &                 rato3,ratno2)
                  if (lprnt) write(*,*)'k,rato3,ratno2',i,j,k,rato3,ratno2
c     
c-----Load ambient concentrations
c     
                  o3 = ozone(i,j,k)
                  h2o = 1.e6*qcell(k)*28.8/18. 
                  if (lpmchm) then
                     cnox = conc(i,j,kno,k) + conc(i,j,kno2,k)
                     voc = conc(i,j,kvoc,k)
c     
c-----Get OH concentration from lookup table
c     
                     if (lprnt) write(*,*)'o3,h2o,cnox,voc',o3,h2o,cnox,voc
                     if (pcell(k).gt.700.) then
                        call getoh(1.,oh01,rato3,zenith,tcell(k),h2o,o3,cnox,
     &                       voc,oh,ierr)
                     else
                        call getoh(2.,oh02,rato3,zenith,tcell(k),h2o,o3,cnox,
     &                       voc,oh,ierr)
                     endif
                     if (ierr.eq.1) then
                        write(*,*)'Stopping in RCHEM: bad OH',i,j
                        stop
                     endif
                     if (lprnt) write(*,*)'oh',oh
c     
c-----Load PM array and call PM chemistry routine
c     
                  pm(1) = conc(i,j,kso2,k)
                  pm(2) = conc(i,j,kgso4,k)
                  pm(3) = conc(i,j,kaso4,k)
                  pm(4) = conc(i,j,kno,k)
                  pm(5) = conc(i,j,kno2,k)
                  pm(6) = conc(i,j,kvoc,k)
                  pm(7) = 0.
                  if (khno3.ne.0) pm(7) = conc(i,j,khno3,k)
c     
                  call partchm(pm,tcell(k),pcell(k),h2o,rhcell(k),o3,ratno2,
     &                 oh,zcoef,deltat,lprnt)
c     
                  conc(i,j,kso2,k) = pm(1)
                  conc(i,j,kgso4,k) = pm(2)
c                  conc(i,j,kaso4,k) = pm(3)
                  conc(i,j,kno,k) = pm(4)
                  conc(i,j,kno2,k) = pm(5)
                  conc(i,j,kvoc,k) = pm(6)
                  if (khno3.ne.0) conc(i,j,khno3,k) = pm(7)
c     
c-----Determine nitrate equilibrium if this is a PM chemistry run, or if
c     its a Toxic chemistry run with nitrates
c     
                  if (lpmchm .or. (ltxchm .and. khno3.ne.0)) then
c
c               aso4 and gso4 should not be modified by remeql3
c               so calculation of fractions and redistribution
c               of sulfate has been removed.
c               since prior versions of REMSAD have assumed
c               that all pariculate nitrate is nh4no3,
c               we will assume that here also.  This means
c               that we do not need to calculate the fraction
c               of nh4 as nitrate (frnh4n) before remeql3.
c               The assumption that this fraction would be
c               the same after remeql3 is not correct anyway.
c               (tcm, 10/18/00)
                     tso4  = conc(i,j,kaso4,k) + conc(i,j,kgso4,k)
                     thno3 = conc(i,j,khno3,k)
                     tnh3o = conc(i,j,knh3,k)
                     pno3  = conc(i,j,kpno3,k)
                     panh4 = conc(i,j,knh4n,k) + conc(i,j,knh4s,k)
Ctcm                 frnh4n= conc(i,j,knh4n,k)/panh4
Ctcm                 paso4 = conc(i,j,kaso4,k) + conc(i,j,kgso4,k)
Ctcm                 fraso4= conc(i,j,kaso4,k)/paso4
                     call remeql3(tso4,thno3,tnh3o,rhcell(k),tcell(k),
     &                    pcell(k),pno3,panh4,paso4,i,j,k)
c     
c     check against lower bound for these species
c     
                     conc(i,j,khno3,k) = amax1(thno3,bdnl(khno3))
                     conc(i,j,knh3,k)  = amax1(tnh3o,bdnl(knh3))
Ctcm                 conc(i,j,kaso4,k) = amax1(paso4*fraso4,bdnl(kaso4))
Ctcm                 conc(i,j,kgso4,k) = amax1(paso4*(1-fraso4),bdnl(kgso4))
                     conc(i,j,kpno3,k) = amax1(pno3,bdnl(kpno3))
Ctcm                 conc(i,j,knh4n,k) = amax1(panh4*frnh4n,bdnl(knh4n))
Ctcm                 conc(i,j,knh4s,k) = amax1(panh4*(1-frnh4n),bdnl(knh4s))
                     conc(i,j,knh4n,k) = amax1(pno3,bdnl(knh4n))
                     conc(i,j,knh4s,k) = amax1(panh4-pno3,bdnl(knh4s))
                  endif
               end if
c     
c-----------------------------------------------------------------------
c-----Perform chemistry for toxics
c     
               if (.not.ltxchm) goto 70
c     
c-----Partition certain toxic species between vapor and particle phase
c     and load TOX array
c     
               pm10 = 4.
               mfac = cpfac*(1013.25/pcell(k))*(tcell(k)/273.)
               do 60 l = ngas+1,nospec
                  if (l.eq.kpom1p .or. l.eq.kpom2p .or. l.eq.kdioxp .or.
     &                 l.eq.katrazp) goto 60
                  lsp = jind(l)
                  pm10 = pm10 + molwt(lsp)*conc(i,j,l,k)/mfac
 60            continue
c     
               if (knaph.ne.0) tox(1) = conc(i,j,knaph,k)
               if (kpom1.ne.0) then
                  call toxpart(pm10,tcell(k),conc(i,j,kpom1,k),
     &                 conc(i,j,kpom1p,k),1)
                  tox(2) = conc(i,j,kpom1,k)
               endif
               if (kpom2.ne.0) then
                  call toxpart(pm10,tcell(k),conc(i,j,kpom2,k),
     &                 conc(i,j,kpom2p,k),2)
                  tox(3) = conc(i,j,kpom2,k)
               endif
               if (kdiox.ne.0) then
                  call toxpart(pm10,tcell(k),conc(i,j,kdiox,k),
     &                 conc(i,j,kdioxp,k),3)
                  tox(4) = conc(i,j,kdiox,k)
               endif
               if (katraz.ne.0) then
                  call toxpart(pm10,tcell(k),conc(i,j,katraz,k),
     &                 conc(i,j,katrazp,k),4)
                  tox(5) = conc(i,j,katraz,k)
               endif
c     
c-----Call TOX chemistry routine, then
c     repartition certain toxic species between vapor and particle phase
c     
c     if (lprnt) write(*,*)'TOX: ',tox
               call toxchm(deltat,tox,tcell(k))
c     if (lprnt) write(*,*)'TOX: ',tox
c     
               if (knaph.ne.0) 
     &              conc(i,j,knaph,k) = tox(1)
               if (kpom1.ne.0) then
                  conc(i,j,kpom1,k) = tox(2)
                  call toxpart(pm10,tcell(k),conc(i,j,kpom1,k),
     &                 conc(i,j,kpom1p,k),1)
               endif
               if (kpom2.ne.0) then
                  conc(i,j,kpom2,k) = tox(3)
                  call toxpart(pm10,tcell(k),conc(i,j,kpom2,k),
     &                 conc(i,j,kpom2p,k),2)
               endif
               if (kdiox.ne.0) then
                  conc(i,j,kdiox,k) = tox(4)
                  call toxpart(pm10,tcell(k),conc(i,j,kdiox,k),
     &                 conc(i,j,kdioxp,k),3)
               endif
               if (katraz.ne.0) then
                  conc(i,j,katraz,k) = tox(5)
                  call toxpart(pm10,tcell(k),conc(i,j,katraz,k),
     &                 conc(i,j,katrazp,k),4)
               endif
c     
 70            do 75 l = 1,nospec
                  conc(i,j,l,k) = amax1(conc(i,j,l,k),bdnl(l))
 75            continue
 50         continue
c     if ( lprnt ) then
c     write(*,*) ' Leave rhcem ...'
c     do iaa=1,nospec
c     write(*,'(6e12.5)') (conc(21,32,iaa,kaa),kaa=1,nlayer)
c     enddo
c     endif
c     c
c-----End loop on I and J
c     
 99      continue
 100  continue
c     
      return
      end
      
