c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
c
      subroutine rchem2(deltat,time,idate,idebug)
c
c-----ATDM v1.00
c
c     Chemistry driver for the coarse domain (for use Micro CB4 only).
c
c-----95/06/16:
c     C Emery SAI
c-----95/10/05:
c     -revised calculation of PM10 for toxic partitioning to be in units
c      of ug/m3 rather than ppm
c     -reworked aqueous chemistry code, cleaned up some errors
c-----95/10/06:
c     -removed zmsl array; passing PHOT zcell and avterr
c-----95/10/18:
c     -restructured for more efficient handling of lwet and rain 
c-----95/10/26:
c     -revised calculation of Hg0 -> HgII aqueous reaction
c-----95/12/08:gem
c     -fixed index on bdnl from k to l
c-----95/12/11:gem
c     -changed pm10=0. to pm10=4. (bckgrnd pm10)
c-----95/12/13:
c     -added check against lower bnd after call to nitrat
c-----95/12/22:
c     -added branch around convmix for PM runs, calls wetdep from here
c      (quick fix to avoid problems in convmix with strong convective
c       columns with low cloud base)
c-----96/08/01:
c     -moved check of ierr from convmix inside if-then-else loop
c-----96/09/13:dfg
c     -added call to tdrate, photol, step4 for solve micro CB4 chemical
c      mechanism.
c     -removed call to partchm for pm chemistry.
c     -moved calculation of cloud mixing/wet dep to the end of chemistry
c      loop.
c     -added one concentration array con(mxspec).
c     -added ozone(i,j,k)=conc(i,j,ko3,k)
c     -added include file comchem2.cmd
c-----97/03/24:DFG
c     -passed k to nitrat subroutine.
c-----97/03/25:DFG
c     -add call to subroutine sulfate for sulfate chemistry calculation.
c-----97/06/25:DFG
c     -passed pressure (i.e. pecll(k)) to nitrat subroutine.
c-----97/12/20:DFG
c     -set ddtt=deltat in responding to unit of per min for k's.
c-----99/4/14: DFG
c     -added radical array for new chemistry solver to use.
c     -added radical concentration (RCON) to passing input to step4.
c-----99/4/22: DFG
c     -added loops for saving OH separately when necessary.
c
c     Rebuilt to contain the cloud sub-model; for those columns with
c     precipitation ( > 0.01 in/hr), the vertical concentration profile
c     is split between an in-cloud profile and non-cloud profile.  Cloud 
c     mixing and wet deposition act on the in-cloud profile over multiple
c     sub-steps, after which the two profiles are recombined.  PM and/or
c     toxic chemistry is then performed on the combined profile.
c
c     Input arguments:
c               deltat:         model time step (min)
c               time:           model time
c               idate:          model date
c               idebug:         diagnostic debug flag
c     Files written:
c               6:              standard output file unit (*)     
c     Routines called:
c               jtocal    nitrat    toxpart   convmix   strtmix
c               solar     tdrate    photol    step4     toxchm
c     Called by:
c               atdm
c
      include 'cpumar.cmd'
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'comchem2.cmd'
      include 'ohtbl.cmd'
      include 'aerosl.cmd'
      include 'balanc.cmd'
      include 'ctlflg.cmd'
      include 'latlon.cmd'
      include 'com3.cmd'
      include 'kweight.cmd'
c
      dimension aradcon(mxrads,mxz)
      dimension cincld(mxz,mxspec), cnocld(mxz,mxspec)
      dimension acincld(mxz),acnocld(mxz)
      dimension zcell(mxz),tcell(mxz),pcell(mxz),qcell(mxz),rhcell(mxz)
      dimension ccell(mxz)
      dimension wi(mxz), delhg0(mxz), delgso4(mxz)
      dimension pm(7), tox(5)
      dimension lqdh2o(mxz)
      real*4 lwch2o(mxx,mxy,mxz),clwch2o(mxx,mxy,mxz)
      dimension con(mxspec),rcon(mxrads)
      dimension tarray(2)
c     dimension fackoh(mxz)
      real*4 hf(3)
      real svg(4), svp(4), mwv(4)
      integer iprt,jprt
      logical lprnt, lpro3
      real*4 lqdh2o
      common /idcell/irchem,jrchem,krchem,tmchm
      real nh3, nh4n, nh4s, lv, lwc, mfac
      save ilast, jlast
      save ndone
c
      data eps /0.622/, lv /2.5e6/, e0 /6.11/, rv /461./
      data tdepo3 /-2300./, rkhg0 /2.31e+8/
c      data lwc /1./, ekho30 /1.13e-2/, tdepo3 /-2300./, rkhg0 /2.31e+8/
      data pm /7*0./, tox /5*0./
      data pi /3.1415927/
      data iprt,jprt /88,11/
      data ndone/0/
      data nsv /4/               ! number of soa species
c
      write(*,3001) deltat
 3001 format(' enter rchem2  ',f10.2)
c
      cpusoa = 0.
      ilast = 1
      jlast = 1
      cpumar = 0.0
      do i = 1, nox
         do j = 1, noy
            do k = 1, nlayer
               do is = 1, mxspec
                  if (conc(i,j,is,k) .lt. 0.0) then
                     write (*,*) 'rchem2',i,j,k,is,conc(i,j,is,k)
                     stop
                  end if
               end do
            end do
         end do
      end do
c
c      write (*,*) 'in rchem', (bdnl(l), l = 1, nospec)
c
c-----Bit nine of idebug (i.e., the 256 location) 
c     will turn on debug writes in rchem.
c
      idiag = and(idebug,256)
      tmchm = time
c     lprnt = .false.
      if (idiag.ne.0) lprnt = .true. 
c
c     initializing array con for step4 use.
      do l=1,mxspec
        con(l)=0.
      end do
c
c-----Extract date
c
      if (idate.lt.100000) then
        call jtocal(idate,icdate)
        nyr = icdate/10000
        nmth = icdate/100 - nyr*100
        nday = icdate - nmth*100 - nyr*10000
      else
        nyr = idate/10000
        nmth = idate/100 - nyr*100
        nday = idate - nmth*100 - nyr*10000
      endif
      nyr = nyr + 1900
c
c for given time interval, calculate the liquid water content for entire domain
c
c      write (*,*) 'before alwc'
c
      do i = 1, nox
         do j = 1, noy
            do k = 1, nlayer
               call alwc(clh2o(i,j,k),rnh2o(i,j,k),press(i,j,k),
     *              temper(i,j,k),lwch2o(i,j,k),clwch2o(i,j,k))
            end do
         end do
      end do
c      write (*,*) 'after alwc'
c
c***
c    Convert time step to in hour.
c***DFG          ddtt=deltat/60.
c
           ddtt=deltat
c
c-----Loop on grids
c
      do 100 j = 1,noy
        jrchem = j
        if (istr(j).ge.nox) goto 100
        i1 = istr(j) + 1
        i2 = iend(j) - 1
        do 99 i = i1,i2
c           write (*,*) 'i  j', i,j
c     
            do ijk = 1, nlayer
               lqdh2o(ijk)= clwch2o(i,j,ijk)
            end do
c
          irchem=i
          if (j.le.jstr(i)) goto 99
          if (j.ge.jend(i)) goto 99
c
c-----Skip cells covered by fine grid
c
          do 10 ifine = 1,numfin
            if (i.ge.ixfb(ifine) .and. i.le.ixfe(ifine) .and.
     &          j.ge.jyfb(ifine) .and. j.le.jyfe(ifine)) goto 99
  10      continue
c
c-----Load some ambient column profiles
c
          lprnt = .false.
          if (i.eq.iprt .and. j.eq.jprt .and. idiag.ne.0) lprnt = .true.
          if (i.eq.50 .and. j.eq.50 .and. idiag.ne.0) lprnt = .true.
          if (lprnt) write(*,*)'z,t,p,q,c,rh'
cgem
c          write(*,*)'z,t,p,q,c,rh'
cgem
c          write (*,*) 'before 20'
          do 20 k = 1,nlayer
            zcell(k) = h(i,j,k)
            tcell(k) = temper(i,j,k)
            pcell(k) = press(i,j,k)
            qcell(k) = ch2o(i,j,k)
            ccell(k) = cloud(i,j,k)
            h2o = 1.e6*qcell(k)*28.8/18. 
            ev = qcell(k)*pcell(k)/(qcell(k) + eps)
            es = e0*exp((lv/rv)*(1./273. - 1./tcell(k)))
            rhcell(k) = 100.*ev/es
            if (lprnt) write(*,'(6e10.3)') zcell(k),tcell(k),
     &                        pcell(k),qcell(k),ccell(k),rhcell(k)
cgem
c            write(*,'(6e10.3)') zcell(k),tcell(k),
c     &                        pcell(k),qcell(k),ccell(k),rhcell(k)
cgem
  20      continue
c          write (*,*) 'after 20',i,j
 	  if ( lprnt ) then
 	     write(*,*) ' Enter rchem ...'
 	     do iaa=1,nospec
   	  write(*,'(6e12.5)') (conc(iprt,jprt,iaa,kaa),kaa=1,nlayer)
 	     enddo
 	   endif
c
c-----Determine nitrate equilibrium if this is a PM chemistry run, or if
c     its a Toxic chemistry run with nitrates
c
           adum1 = etime(tarray)
           cpumar1 = tarray(1)
           do 30 k = 1,nlayer
            if (lpmchm .or. (ltxchm .and. khno3.ne.0)) then
c
c               aso4 and gso4 should not be modified by remeql3
c               since prior versions of REMSAD have assumed
c               that all pariculate nitrate is nh4no3,
c               we will assume that here also.
c               (tcm, 10/18/00)
                  tso4  = conc(i,j,kaso4,k) + conc(i,j,kgso4,k)
                  thno3 = conc(i,j,khno3,k)
                  tnh3o = conc(i,j,knh3,k)
                  pno3  = conc(i,j,kpno3,k)
                  panh4 = conc(i,j,knh4n,k) + conc(i,j,knh4s,k)
                  call remeql3(tso4,thno3,tnh3o,rhcell(k),tcell(k),
     &                 pcell(k),pno3,panh4,paso4,i,j,k)
c
c     check against lower bound for these species
c
                  conc(i,j,khno3,k) = amax1(thno3,bdnl(khno3))
                  conc(i,j,knh3,k)  = amax1(tnh3o,bdnl(knh3))
                  conc(i,j,kpno3,k) = amax1(pno3,bdnl(kpno3))
                  conc(i,j,knh4n,k) = amax1(pno3,bdnl(knh4n))
                  conc(i,j,knh4s,k) = amax1(panh4-pno3,bdnl(knh4s))
            endif
c
c-----Partition certain toxic species between vapor and particle phase
c
            if (ltxchm) then
              pm10 = 4.
              mfac = cpfac*(1013.25/pcell(k))*(tcell(k)/273.)
              do 35 l = ngas+1,nospec
                if (l.eq.kpom1p .or. l.eq.kpom2p .or. l.eq.kdioxp .or.
     &              l.eq.katrazp) goto 35
                lsp = jind(l)
                pm10 = pm10 + molwt(lsp)*conc(i,j,l,k)/mfac
  35          continue
c             if (lprnt) write(*,*)'PM10: ',pm10
              if (kpom1.ne.0)
     &          call toxpart(pm10,tcell(k),conc(i,j,kpom1,k),
     &                       conc(i,j,kpom1p,k),1)
              if (kpom2.ne.0)
     &          call toxpart(pm10,tcell(k),conc(i,j,kpom2,k),
     &                       conc(i,j,kpom2p,k),2)
              if (kdiox.ne.0)
     &          call toxpart(pm10,tcell(k),conc(i,j,kdiox,k),
     &                       conc(i,j,kdioxp,k),3)
              if (katraz.ne.0)
     &          call toxpart(pm10,tcell(k),conc(i,j,katraz,k),
     &                       conc(i,j,katrazp,k),4)
            endif
   30     continue
           adum2 = etime(tarray)
           cpumar2 = tarray(1)
           cpumar = cpumar+(cpumar2-cpumar1)
c          write (*,*) 'after 30', i, j
c
c-----Calculate solar zenith angle
c
c          write(*,*) i,j,ctlat(i,j),ctlon(i,j),tz,nyr,nmth,nday,time,zcoef
          if (lprnt) then
            write(*,*)'lat,lon,zone,yr,mo,dy,t'
            write(*,*) ctlat(i,j),ctlon(i,j),tz,nyr,nmth,nday,time
          endif
          call solar(ctlat(i,j),-ctlon(i,j),tz,nyr,nmth,nday,time,zcoef)
          zenith = (180./pi)*acos(zcoef)
c         if (lprnt) write(*,*)'zenith,zcoef ',zenith,zcoef
c
c-----Do chemistry in each layer
c
c          write (*,*) 'before 50', i,j
c
          adum3 = etime(tarray)
          cpumar3 = tarray(1)
          do 50 k = 1,nlayer
            krchem=k
            rh=rhcell(k)
c
c*** Call tdrate and rate_coeff to get rate constants and do chemistry when
c    mechanism is specified.
c
          h2o = 1.e6*qcell(k)*28.8/18. 
          bh2o = 1.e6*qcell(k)*28.8/18. 
          call tdrate(tcell(k),pcell(k))
          call rate_coeff(tcell(k),pcell(k),bh2o,pcell(1))
          if (i.eq.2.and.j.eq.2.and.k.eq.7) then
c         write (*,*) 'after tdrate in rchem2', i,j,k,(rk(irxn),irxn=1,64)
          end if
c
c-----Obtain current photolysis rates from lookup table
c
          call photol(rk,rkn,k,nlayer,zenith,zcell,
     &              avterr(i,j),ccell,ldark)
          if (lprnt) then
            write (*,*) 'after photol', i,j
            write(*,*) 'k, nlayer, zenith, zcell, avterr(i,j), ccell:'
            write(*,*) k, nlayer, zenith, zcell, avterr(i,j), ccell
            write(*,*) 'ldark = ', ldark
            write(*,*) 'n, rk, rkn:'
            do n=1,mxgrx
               write(*,*) n, rk(n), rkn(n)
            enddo
          endif
c
c     Load layer-average concentrations into con array.
c
            do 31 l=1,nors
              con(l) = conc(i,j,l,k) 
 31         continue
c
c  load radical concentrations
c
            llx = i + nox*(j-1) + nox*noy*(k-1)
            do ll=1,mxrads
              rcon(ll)=radcon(ll,llx)
            end do
c
c   Save the previous concentration of SULF.
c
          rsulf1=conc(i,j,ksulf,k)
c
          aromf = 0.
          if (katol .gt. 0) aromf = qarea(i,j,katol)
c                                                          
c     Call step4 to solve chemistry.
c
          itest = itest + 1
          idex = i
          jdex = j
          kdex = k
          testtime = time
          testidate = idate
          lprnt = .false.
c         if ( (idex .eq. nox/2 .and. jdex .eq. noy/2) .or.
c    &          (jdex .lt. jlast) ) then
c         if ( idex .eq. 91 .and. jdex .eq. 18) then
c         if ( idex .eq. iprt .and. jdex .eq. jprt) then
c           write(*,*) 'Before step4, time, idex,jdex,kdex,tcell: '
c           write(*,*) time, idex,jdex,kdex,tcell(k)
c           write(*,*) 'ddtt = ', ddtt
c           write(*,*) 'h2o = ', h2o
c           write(*,*) 'Con: ', con
c           write(*,*) 'Rcon: ', rcon
c           lprnt = .true.
c         endif
          call step4(ddtt,con,tcell(k),lprnt,rcon,time,idate)
 1002     format (100(1x,e10.4))
c         if ( (idex .eq. nox/2 .and. jdex .eq. noy/2) .or.
c    &          (jdex .lt. jlast) ) then
c         if ( idex .eq. 91 .and. jdex .eq. 18) then
c         if ( idex .eq. iprt .and. jdex .eq. jprt) then
c           write(*,*) 'After step4, time, idex,jdex,kdex: '
c           write(*,*) time, idex,jdex,kdex
c           write(*,*) 'Con: ', con
c           write(*,*) 'Rcon: ', rcon
c         endif
c         ilast = idex
c         jlast = jdex
c         lprnt = .false.
c         fackoh(k) = rcon(koh)
c         if (k .eq. 15) then
c             write (*,*) 'yes rchem2'
c            rcon(koh) = fackoh(14)
c            oh = fackoh(14)
c         end if
c          write (*,*) 'after step4', i,j, rcon(koh), oh
c
c     Load back conc array.
c
          do 32 l=1,nors
            conc(i,j,l,k) = con(l)
 32       continue
          do l=1,mxrads
            radcon(l,llx)=rcon(l)
            aradcon(l,k) = rcon(l)
          enddo
c
c   save oh concentration separately if necessary.  dfg,4/14/99
c
          if (lmoh) ohcon(i,j,k)=rcon(koh)
          if (lmoh) ho2con(i,j,k)=rcon(kho2)
          if (lmoh) no3con(i,j,k)=rcon(kno3)
c
c   Save the new concentration of SULF.
c
          rsulf2=conc(i,j,ksulf,k)
Cc
          ozone(i,j,k)=conc(i,j,ko3,k)
          hydro(i,j,k)=conc(i,j,kh2o2,k)

c-----Load SO2, GSO4, ASO4 concentrations to PM array and 
c     Calculate sulfate concentrations
c
          pm(1) = conc(i,j,kso2,k)
          pm(2) = conc(i,j,kgso4,k)
          pm(3) = conc(i,j,kaso4,k)
          pm(4) = conc(i,j,kh2o2,k)
          pm(5) = conc(i,j,ko3,k)
          rsulf = rsulf2 - rsulf1
c
c-----calculate eff. henry's law constants for h2o2 and so2
c
          hplus = 10.**(-4.5)
          hplus2 = hplus*hplus
          do nh = 1,3
            ttmp1 = kt(nh)*aqrk1(nh)/hplus
            ttmp2 = kt(nh)*aqrk1(nh)*aqrk2(nh)/hplus2
            hf(nh) = kt(nh) + ttmp1 + ttmp2
          enddo
c
c-----Call sulfate chemistry routine.
c
          fnonpr = cfnonpr(i,j,k)
          vmr = clh2o(i,j,k)*1.293e-3*pcell(k)/1013.*273./tcell(k)
          call sulfate(pm,so2av,rsulf,vmr,tcell(k),pcell(k),
     &                 hf,deltat,lprnt,ccell(k),fnonpr)
c          write (*,*) 'after sulfate',i,j
c
c-----Load SO2, GSO4, ASO4 concentrations back to CONC array and 
c
            conc(i,j,kso2,k) = pm(1)
            conc(i,j,kgso4,k) = pm(2)
            conc(i,j,kaso4,k) = pm(3)
            conc(i,j,kh2o2,k) = amax1(pm(4),bdnl(kh2o2))
            conc(i,j,ko3,k) = amax1(pm(5),bdnl(ko3))
            aso2con(i,j,k)=so2av
c
c-----Determine nitrate equilibrium if this is a PM chemistry run, or if
c     its a Toxic chemistry run with nitrates
c
            if (lpmchm .or. (ltxchm .and. khno3.ne.0)) then
c
c               aso4 and gso4 should not be modified by remeql3
c               since prior versions of REMSAD have assumed
c               that all pariculate nitrate is nh4no3,
c               we will assume that here also. 
c               (tcm, 10/18/00)
                  tso4  = conc(i,j,kaso4,k) + conc(i,j,kgso4,k)
                  thno3 = conc(i,j,khno3,k)
                  tnh3o = conc(i,j,knh3,k)
                  pno3  = conc(i,j,kpno3,k)
                  panh4 = conc(i,j,knh4n,k) + conc(i,j,knh4s,k)
                  call remeql3(tso4,thno3,tnh3o,rhcell(k),tcell(k),
     &                 pcell(k),pno3,panh4,paso4,i,j,k)
                  adum = etime(tarray)
c
c     check against lower bound for these species
c
                  conc(i,j,khno3,k) = amax1(thno3,bdnl(khno3))
                  conc(i,j,knh3,k)  = amax1(tnh3o,bdnl(knh3))
                  conc(i,j,kpno3,k) = amax1(pno3,bdnl(kpno3))
                  conc(i,j,knh4n,k) = amax1(pno3,bdnl(knh4n))
                  conc(i,j,knh4s,k) = amax1(panh4-pno3,bdnl(knh4s))
            endif
c
c     calculate soa partitioning
c
c           write(*,*) 'Calling soapart'
            aaa = etime(tarray)
            if (ksv1 .gt. 0) then
              svg(1) = conc(i,j,ksv1,k)
              svg(2) = conc(i,j,ksv2,k)
              svg(3) = conc(i,j,ksv3,k)
              svg(4) = conc(i,j,ksv4,k)
              svp(1) = conc(i,j,ksoa1,k)
              svp(2) = conc(i,j,ksoa2,k)
              svp(3) = conc(i,j,ksoa3,k)
              svp(4) = conc(i,j,ksoa4,k)
              mwv(1) = molwt(jind(ksv1))
              mwv(2) = molwt(jind(ksv2))
              mwv(3) = molwt(jind(ksv3))
              mwv(4) = molwt(jind(ksv4))
              call soapart(conc(i,j,kpoc,k),rcon(koh),
     &                     pcell(k),tcell(k),deltat,svg,svp,
     &                     mwv,cpfac,nsv)
              conc(i,j,ksv1,k) = svg(1)
              conc(i,j,ksv2,k) = svg(2)
              conc(i,j,ksv3,k) = svg(3)
              conc(i,j,ksv4,k) = svg(4)
              conc(i,j,ksoa1,k) = svp(1)
              conc(i,j,ksoa2,k) = svp(2)
              conc(i,j,ksoa3,k) = svp(3)
              conc(i,j,ksoa4,k) = svp(4)
            endif
            bbb = etime(tarray)
            cpusoa = bbb - aaa + cpusoa
c           write(*,*) 'back from soapart'
c
c          write (*,*) 'before toxic chemistry', i,j
c
c-----------------------------------------------------------------------
c-----Perform chemistry for toxics
c
            if (.not.ltxchm) goto 70
c
c-----Partition certain toxic species between vapor and particle phase
c     and load TOX array
c
            pm10 = 4.
            mfac = cpfac*(1013.25/pcell(k))*(tcell(k)/273.)
c          write (*,*) 'before 60',i,j
            do 60 l = ngas+1,nospec
              if (l.eq.kpom1p .or. l.eq.kpom2p .or. l.eq.kdioxp .or.
     &            l.eq.katrazp) goto 60
              lsp = jind(l)
              pm10 = pm10 + molwt(lsp)*conc(i,j,l,k)/mfac
  60        continue
c
c            write (*,*) 'after 60', i,j
c
            if (knaph.ne.0) tox(1) = conc(i,j,knaph,k)
            if (kpom1.ne.0) then
              call toxpart(pm10,tcell(k),conc(i,j,kpom1,k),
     &                     conc(i,j,kpom1p,k),1)
              tox(2) = conc(i,j,kpom1,k)
            endif
            if (kpom2.ne.0) then
              call toxpart(pm10,tcell(k),conc(i,j,kpom2,k),
     &                     conc(i,j,kpom2p,k),2)
              tox(3) = conc(i,j,kpom2,k)
            endif
            if (kdiox.ne.0) then
              call toxpart(pm10,tcell(k),conc(i,j,kdiox,k),
     &                     conc(i,j,kdioxp,k),3)
              tox(4) = conc(i,j,kdiox,k)
            endif
            if (katraz.ne.0) then
              call toxpart(pm10,tcell(k),conc(i,j,katraz,k),
     &                     conc(i,j,katrazp,k),4)
              tox(5) = conc(i,j,katraz,k)
            endif
c
c-----Call TOX chemistry routine, then
c     repartition certain toxic species between vapor and particle phase
c
c           if (lprnt) write(*,*)'TOX: ',tox
c            write (*,*) 'before toxchem rchem2', i,j,k
            call toxchm(deltat,tox,tcell(k))
c            write (*,*) 'after toxchem', i,j
c           if (lprnt) write(*,*)'TOX: ',tox
c
            if (knaph.ne.0) 
     &        conc(i,j,knaph,k) = tox(1)
            if (kpom1.ne.0) then
              conc(i,j,kpom1,k) = tox(2)
              call toxpart(pm10,tcell(k),conc(i,j,kpom1,k),
     &                     conc(i,j,kpom1p,k),1)
            endif
            if (kpom2.ne.0) then
              conc(i,j,kpom2,k) = tox(3)
              call toxpart(pm10,tcell(k),conc(i,j,kpom2,k),
     &                     conc(i,j,kpom2p,k),2)
            endif
            if (kdiox.ne.0) then
              conc(i,j,kdiox,k) = tox(4)
              call toxpart(pm10,tcell(k),conc(i,j,kdiox,k),
     &                   conc(i,j,kdioxp,k),3)
            endif
            if (katraz.ne.0) then
              conc(i,j,katraz,k) = tox(5)
              call toxpart(pm10,tcell(k),conc(i,j,katraz,k),
     &                     conc(i,j,katrazp,k),4)
            endif
c
   70       do 75 l = 1,nospec
c              if (l .eq. 8) then
c                 write (*,*) 'in 75', i,j,k,conc(i,j,l,k),bdnl(l),khno2,hno2
c              end if
              conc(i,j,l,k) = amax1(conc(i,j,l,k),bdnl(l))
   75       continue
   50     continue
          adum4 = etime(tarray)
          cpumar4 = tarray(1)
          cpumar = cpumar+(cpumar4-cpumar3)
c          write (*,*) 'after 50',i,j
c
            ndone = ndone + 1
            do k = 1, nlayer
c               write (*,*) i,j,k,temper(i,j,k)
               airfact = press(i,j,k)/(1013.25*0.08206*temper(i,j,k))
               airden = airfact*(1.0e-3)*(6.02e+23)
               ato3 = conc(i,j,ko3,k)
               ath2o2 = conc(i,j,kh2o2,k)
               atoh = aradcon(koh,k)
               atho2 = aradcon(kho2,k)
               atno3 = aradcon(kno3,k)
               so2av = aso2con(i,j,k)
               atpec = conc(i,j,kpec,k)
               atcl20 = cl2con(i,j)
               hmax = 2000.
               if (atcl20 .gt. 1.e-4) hmax = 1000.
               h2 = h(i,j,k)
               if (k .gt. 1) then
                 h1 = h(i,j,k-1)
               else
                 h1 = 0.
               endif
               call solar(ctlat(i,j),-ctlon(i,j),tz,nyr,nmth,nday,time,zcoef)
               zenith = (180./pi)*acos(zcoef)
               if (zenith .lt. 85.) atcl20 = 0.0
               if (h1 .gt. hmax) then
                 atcl2 = 0.
               else
                 atcl22 = (hmax - amin1(hmax,h2))/hmax * atcl20
                 atcl21 = (hmax - h1)/hmax * atcl20
                 atcl2 = (amin1(hmax,h2)-h1)/(h2-h1)*.5*(atcl22+atcl21)
               endif
c
               lprnt = .false.
               if (mod(ndone,100000) .eq. 0) then
                 if (mod(k,4) .eq. 1) lprnt = .true.
               endif
               do ihg = 1,nnhg
                  call hgchem(i,j,k,ihg,deltat,airden,lwch2o(i,j,k),
     &                        atoh,atho2,atno3,ath2o2,ato3,so2av,atcl2,
     &                        atpec,lprnt)
               end do
               lprnt = .false.
            end do

c            end do
c
c-----Do cloud mixing/wet dep in a loop over species
c     Perform cloud aqueous chemical conversions while loading cloud 
c     and non-cloud concentration profiles
c
          fracp = amax1(fconv(i,j),fstrt(i,j))
          if (lprnt) write(*,*)'fracp: ',fracp
          do 40 l = 1,nospec
c             write (*,*) 'in 40', l
            lsp = jind(l)
            if (lprnt) write(*,*)' l,lsp :',l,lsp
            do 41 k = 1,nlayer
c               if (l .eq. 8) then
c                  write (*,*) i,j,k,l,conc(i,j,l,k)
c               end if
              cnocld(k,l) = conc(i,j,l,k)
              cincld(k,l) = conc(i,j,l,k)
cgem
              if ( lprnt ) then
		 write(*,*) 'l,k,cin,cno :',l,k,cincld(k,l),cnocld(k,l)
	      endif
cgem
              if (fracp.eq.0. .and. ccell(k).eq.0.) goto 41
              if (lwet) wi(k) = wrat(i,j,k)
c
              if (k.lt.kfrz(i,j)) then
                frac1 = amax1((ccell(k) - fracp),0.)/
     &                        (1. - fracp + 1.e-10)
                frac2 = 0.
                if (fracp.ne.0.) then
                  if (k.ge.kcldb(i,j) .and. k.le.kcldt(i,j)) then
                    frac2 = 1.
                  else
                    frac2 = amin1(ccell(k),fracp)/fracp
                  endif
                endif
c
c                write (*,*) 'kgso4 kaso4', kgso4,kaso4, l
c
                if (l.eq.kgso4) then
                  if (kgso4.lt.kaso4)
     &              delgso4(k) = -fnucl*conc(i,j,l,k)
                  cnocld(k,l) = cnocld(k,l) + frac1*delgso4(k)
                  cincld(k,l) = cincld(k,l) + frac2*delgso4(k)
                  if (lprnt) then
                    write(*,*)'l,k,frac1,frac2,kfrz',
     &                         l,k,frac1,frac2,kfrz(i,j)
                    write(*,*)'GSO4:delc,conc',delgso4(k),conc(i,j,l,k)
                    write(*,*)'nocld,incld',cnocld(k,l),cincld(k,l)
                  endif
                elseif (l.eq.kaso4) then
                  if (kgso4.gt.kaso4)
     &              delgso4(k) = -fnucl*conc(i,j,kgso4,k)
                  cnocld(k,l) = cnocld(k,l) - frac1*delgso4(k)
                  cincld(k,l) = cincld(k,l) - frac2*delgso4(k)
                  if (lprnt) then
                    write(*,*)'ASO4:delc',delgso4(k)
                    write(*,*)'nocld,incld',cnocld(k,l),cincld(k,l)
                  endif
                endif
              endif
  41        continue
c
c            write (*,*) 'after 41'
cgem
c              if ( lprnt ) then
c		 write(*,*) ' Finished cloud split :'
c		 write(*,*) 'l,cin,cno :',
c     &              l,(cincld(k,l),cnocld(k,l),k=1,nlayer)
c	      endif
cgem
c-----------------------------------------------------------------------
c-----If there is rain in this column, invoke the cloud sub-model
c
               do k = 1, nlayer
                  acincld(k) = cincld(k,l)
                  acnocld(k) = cnocld(k,l)
               end do
c
               if (l .eq. ko3) then
                  lpro3 = .false.
               else
                  lpro3 = .false.
               end if
               if (lpro3) then
                  write (84,*) 'rchem2',i,j,lpro3,lwet,rain(i,j)
               end if
               deltadp = 0.0
            if (lwet .and. rain(i,j).ge.0.01) then
c
c-----Convective precipitation
c
            if (fconv(i,j).gt.0.) then
              if ( ltxchm .or. lpmchm ) then
c                 write (*,*) 'call convmix'
                call convmix(lprnt,nlayer,l,kcldb(i,j),kcldt(i,j),
     &                     kfrz(i,j),deltat,deltax(j),deltay,pstar(i,j),
     &                     xmd(i,j),rain(i,j),sigma,zcell,tcell,pcell,
     &                     wi,acincld,deltadp,idebug,ierr,lqdh2o,i,j,lpro3)
                if (ierr.ne.0) then
                  write(*,*)'Stopping in RCHEM at i,j',i,j
                  write(*,*)'Too many convective sub-steps'
                  stop
                endif
              endif
              deltadp = fconv(i,j)*deltadp
c
c-----Stratiform precipitation
c
            elseif (fstrt(i,j).gt.0.) then
c               write (*,*) 'call strtmix'
              call strtmix(lprnt,nlayer,l,kcldb(i,j),kcldt(i,j),
     &                     kfrz(i,j),deltat,deltax(j),deltay,rain(i,j),
     &                     pstar(i,j),sigma,zcell,tcell,pcell,qcell,
     &                     wi,acincld,deltadp,idebug,lqdh2o,i,j,lpro3)
c
              deltadp = fstrt(i,j)*deltadp
            else
              write(*,*)'ERROR (RCHEM): '
              write(*,*)'Non-zero rain, but zero cloud fractions!'
              write(*,*)'i,j,rain,fconv,fstrt'
              write(*,*) i,j,rain(i,j),fconv(i,j),fstrt(i,j)
              stop
            endif
            dpflow(l) = dpflow(l) - deltadp
            wposit(i,j,l) = wposit(i,j,l) + molwt(lsp)*deltadp/
     &                      (deltay*deltax(j))
c
            endif
 2001       format (a,2(1x,i3),10(1x,e10.4))
            if (lpro3) then
               if (deltadp .ne. 0.0) then
                  write (80,2001) 'coar',i,j,deltadp,acincld(1),wposit(i,j,l),fconv(i,j),fstrt(i,j)
               end if
            end if
c     
cgem
              if ( lprnt ) then
		 write(*,*) ' Finished cloud mixing :'
		 write(*,*) 'l,cin,cno :',
     &              l,(cincld(k,l),cnocld(k,l),k=1,nlayer)
	      endif
cgem
c
c-----Recombine in-cloud profiles with non-cloud profiles
c
               do k = 1, nlayer
                  cincld(k,l) = acincld(k)
                  cnocld(k,l) = acnocld(k)
               end do
c
c            write (*,*) 'before 45'
            do 45 k = 1,nlayer
              conc(i,j,l,k) = fracp*cincld(k,l) + (1. - fracp)*cnocld(k,l)
              if (lprnt .and. fracp.ne.0.) then
                write(*,*)'k,incld,nocld',k,cincld(k,l),cnocld(k,l)
                write(*,*)'conc',conc(i,j,l,k)
              endif
  45        continue
  40      continue
c
c          write (*,*) 'after 40'
c
 	  if ( lprnt ) then
 	     write(*,*) ' After cloud module ...'
 	     do iaa=1,nospec
  	   write(*,'(6e12.5)')(conc(iprt,jprt,iaa,kaa),kaa=1,nlayer)
 	     enddo
 	   endif
c-----------------------------------------------------------------------
c	  if ( lprnt ) then
c	     write(*,*) ' Leave rhcem ...'
c	     do iaa=1,nospec
c		write(*,'(6e12.5)') (conc(21,32,iaa,kaa),kaa=1,nlayer)
c	     enddo
c	   endif
cc
c-----End loop on I and J
c
   99   continue
  100 continue
c
       write (*,*) 'end of rchem2', ko3
       write (*,*) 'cpusoa = ', cpusoa
c      stop
      return
      end
