c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine restrt(time,ndate)
c
c-----ATDM v1.00
c
c     Reads a UAM formatted instantaneous file for model restart
c
c-----95/06/01:
c     -from UAM-V Revision 1.2  94/02/10  20:59:43  tcm
c
c     Input arguments:
c               time:           model time
c               ndate:          model date
c     Files read:
c               7:              ATDM control file
c               98:             coarse grid instantaneous restart file
c               99:             fine grid instantaneous restart file
c     Files written:
c               6:              standard output file unit (*)
c               8:              diagnostic file
c               66:             output message file
c     Routines called:
c               caltoj    fgread
c     Called by:
c               setup
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'com1f.cmd'
      include 'comuam.cmd'
      include 'com3.cmd'
      include 'comchem.cmd'
c
      character*90 fn
      character*80 msgrs
      character*10 mchk1, mchk2
      character*4 mrsspc(10,mxspec)
      character*4 ifile(10), ifid(60)
      logical lrstop
      nskip = 0
c
      write(*,3309)
 3309 format(' entering restrt')
      write (*,*) 'begin of restart', time, ndate
      lrstop = .false.
c
      if ( ndate .gt. 99999) then
        call caltoj(ndate,nn)
        ndate = nn
      endif
      write (*,*) 'ndate=', ndate
c
c  open instantaneous file for restart
c
      read(7,7777) fn
 7777 format(a)
      write(*,3310) fn
      write(8,3310) fn
 3310 format(' opening unit 98 for restart file=',a)
      open(unit=98,file=fn,form='unformatted',status='old')
c
c  read headers
c
      read(98) (ifile(i),i=1,10),(ifid(i),i=1,60),ndum,
     &             nd2,idd1,tb,idd2,te
      write(*,*) 'restart', (ifile(i),i=1,10),(ifid(i),i=1,60)
      write (*,*) 
      write (*,*) ndum, nd2,idd1,tb,idd2,te
      read(98) f1,f2, utmzon, xu, yu, dx, dy,
     &          nx2, ny2, nl2, i1, i2, f3, f4, f5
      write(*, *) 'restart',f1,f2, utmzon, xu, yu, dx, dy,
     &          nx2, ny2, nl2, i1, i2, f3, f4, f5
      if (nl2 .ne. nlayer .or. nd2 .ne. nospec) then
        write(*,'(a,/,a,2i5,/,a,2i5)') 
     &       ' bad number of layers or species on restart file',
     &       ' on restart file: ',nl2, nd2,
     &       ' in current run: ',nlayer,nospec
        write(8,'(a,/,a,2i5,/,a,2i5)') 
     &       ' bad number of layers or species on restart file',
     &       ' on restart file: ',nl2, nd2,
     &       ' in current run: ',nlayer,nospec
        stop
      endif
      if (nx2 .ne. nox .or. ny2 .ne. noy) then
        write(*,'(a,/,a,2i5,/,a,2i5)') 
     &       ' bad x/y dimensions on restart file',
     &       ' on restart file: ',nx2, ny2,
     &       ' in current run: ',nox, noy
        write(8,'(a,/,a,2i5,/,a,2i5)') 
     &       ' bad x/y dimensions on restart file',
     &       ' on restart file: ',nx2, ny2,
     &       ' in current run: ',nox, noy
        stop
      endif
      read (98)  i1,i2,ndum,ndum2
      read (98) ((mrsspc(m,l),m=1,10),l=1,nospec)   
      do 15 nn=1,nospec
        write(mchk1,'(10a1)') (mspec(i,nn),i=1,10)
        write(mchk2,'(10a1)') (mrsspc(i,nn),i=1,10)
        if (mchk1 .ne. mchk2) then
          write(*,'(a)') 
     &     ' coarse grid restart species do not match current species'
          write(8,'(a)') 
     &     ' coarse grid restart species do not match current species'
          write(8,'(a)') '  restart     current'
          do 155 nnn=1,nospec
            write(8,'(2x,10a1,2x,10a1)')
     &         (mrsspc(i,nnn),i=1,10), (mspec(i,nnn),i=1,10)
  155     continue
        endif
   15 continue
      write (*,*) 'restart for fine grid', numfin

      if (numfin .gt. 0) then
        read(7,7777) fn
        write(*,3315) fn
        write(8,3315) fn
 3315   format(' opening unit 99 for fine grid restart file=',a)
        open(unit=99,file=fn,form='unformatted',status='old')
        read(99) msgrs
        write(*,'(a,/,2x,a)') ' fine grid restart file labelled:',
     &                          msgrs
        write(8,'(a,/,2x,a)') ' fine grid restart file labelled:',
     &                          msgrs
        read(99) nrsfin, nrsspc
        if (nrsfin .ne. numfin .or. nrsspc .ne. nospec) then
          write(*,'(a,/,a,2i5,/,a,2i5)') 
     &         ' bad number of fine grids or species on restart file',
     &         ' on restart file: ',nrsfin,nrsspc,
     &         ' in current run: ',numfin,nospec
          write(8,'(a,/,a,2i5,/,a,2i5)') 
     &         ' bad number of fine grids or species on restart file',
     &         ' on restart file: ',nrsfin,nrsspc,
     &         ' in current run: ',numfin,nospec
          stop
        endif
        read(99)((mrsspc(i,l),i=1,10),l=1,nrsspc)
        do 17 nn=1,nospec
          write(mchk1,'(10a1)') (mspec(i,nn),i=1,10)
          write(mchk2,'(10a1)') (mrsspc(i,nn),i=1,10)
          if (mchk1 .ne. mchk2) then
            write(*,'(a)') 
     &       ' fine grid restart species do not match current species'
            write(8,'(a)') 
     &       ' fine grid restart species do not match current species'
            write(8,'(a)') '  restart     current'
            do 175 nnn=1,nospec
              write(8,'(2x,10a1,2x,10a1)')
     &             (mrsspc(i,nnn),i=1,10), (mspec(i,nnn),i=1,10)
  175       continue
          endif
   17   continue
        do 18 nn=1,nrsfin
          read(99) ixfbrs, jyfbrs,
     &             ixfers, jyfers, nhfrs,
     &             nvfrs, nfxrs, nfyrs, 
     &             nfzrs
          if (ixfbrs .ne. ixfb(nn) .or.
     &        jyfbrs .ne. jyfb(nn) .or.
     &        ixfers .ne. ixfe(nn) .or.
     &        jyfers .ne. jyfe(nn) .or.
     &        nhfrs .ne. nhf(nn) .or.
     &        nvfrs .ne. nvf(nn) .or.
     &        nfxrs .ne. nfx(nn) .or.
     &        nfyrs .ne. nfy(nn) .or.
     &        nfzrs .ne. nfz(nn)) then
            write(*,'(a,a)') 'restart and current fine grid ',
     &          'definitions do not match'
            write(8,'(a,a)') 'restart and current fine grid ',
     &          'definitions do not match'
            write(8,'(a,9i5)') 'restart: ', ixfbrs, jyfbrs, ixfers,
     &           jyfers, nhfrs, nvfrs, nfxrs, nfyrs, nfzrs
            write(8,'(a,9i5)') 'current: ', ixfb(nn), jyfb(nn), 
     &           ixfe(nn), jyfe(nn), nhf(nn), nvf(nn), nfx(nn), 
     &           nfy(nn), nfz(nn)
            stop
          endif
   18   continue
      endif
   10 continue
      write (*,*) 'after 10'
c
c  modified for restart capability using UAM format file
c
      nskip=nskip+1
      write (*,*) 'before read in 98'
      read(98,end=800,iostat=irst) ndu1, tmu1, idate, tmu2
      write (*,*) ndu1,tmu1,idate,tmu2
      t2 = 100.*tmu2
      write(*,3055) t2,idate
      write (*,*) 'coarse grid time and date=', t2, idate
      write (*,*) 'irst=',irst
      if (irst .ne. 0) then
        write(*,*) ' error reading time record, ',
     &       ' skipping one, trying again'
        read(98)
        go to 10
      endif
 3055 format(' in restrt reading ',f10.1,i10)
      if ( idate .gt. 99999) then
        call caltoj(idate,nn)
        idate = nn
      endif
      do 26 k=1,nospc1
        do 25 l=1,nlayer
          read(98) idum, (ifile(i),i=1,10),
     &              ((conc(i,j,k,l),i=1,nox),j=1,noy)
   25   continue
   26 continue
c
c  read date and time from fine grid instantaneous file
c
      if (numfin .gt. 0) then 
        read(99) tf, ndf
        write (*,*) 'fine grid time and date=', tf, ndf
        if ( ndf .gt. 99999) then
        call caltoj(ndf,nn)
        ndf = nn
      endif
      if (abs(tf-t2) .gt. .5 .or. ndf .ne. idate) then
        write(*,*) ' coarse and fine grid restart files out of synch'
        write(*,*) ' coarse:  ',t2, idate
        write(*,*) ' fine:    ',tf,ndf
        stop
      endif
      do 30 ifine=1,numfin
        call fgread(99,nfx(ifine),nfy(ifine),nfz(ifine),nospec,
     &            cncf(1,1,1,ifine))
   30 continue
      endif
c
      write (*,*) 'end of loop 30'
      tchk= (aint(t2+0.1)-aint(time+0.1))+ 2400.*(idate - ndate)
      write(*,*)'in restrt tchk = ',tchk,t2,idate,time,ndate
      if ( tchk .lt. 0.0) goto 10
      if ( tchk .gt. 0.0) goto 800
c
c     have instantenous concentrations
c
      write(66,3200) nskip,t2,idate
      write(*,3200) nskip,t2,idate
 3200 format(' in restrt ',i5,' outputs skipped ',f10.0,i10)
c
      close(98)
      if (numfin .gt. 0) close(99)
      return
  800 continue
      write(66,3100) time,ndate,t2,idate
      write(*,3100) time,ndate,t2,idate
 3100 format(' $$ error eof in restrt ',2(f10.2,i10))
      stop1
      end
