c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine rvmmap(nxx,nyy,iout,idebug)
c
c-----ATDM v1.00
c
c     Calculate parameters for nested variable-grid map
c
c-----95/02/22:
c     -from UAM-V Revision 1.2  94/02/10  20:59:52  tcm
c     -removed chemistry aggregation code
c     -removed unused code that was commented out
c-----95/03/31 (v0.02):
c     -modifications made following UAM-V to correct vertical nesting code
c-----95/04/05:
c     -added writes of header info to fine grid deposition files
c     -removed check of nxx*nyy*nzz < maxagg
c-----95/06/22:
c     -removed bug that resulted in improper specification of fine grid 
c      pointers for nests more than 3 fine grids deep
c-----95/10/26:
c     -removed print of vertical subdivisions panel as this is now meaningless
c-----99/4/22:DFG
c     -added a block to write headers for OH,O3,HO2,H2O2,NO3 fine grid files
c      if necessary.
c
c     Locations of fine grids are read from control file in setup.
c  
c     Fine grid regions must be rectangles and must use the same
c     number of subdivisions throughout a given fine grid. 
c
c     Three maps will be printed in the diagnostic file showing where
c     the fine grids are located.  In the example below, there are 3 fine 
c     grids.  The first is at the upper left with two subdivisions
c     per coarse cell, another next to it also with two subdivisions
c     and a third with 5 subdivisions at the lower right. 
c  
c     Horizontal subdivisions:
c 
c     0000000000000000
c     0002222000000000
c     0002222bbbb00000
c     0002222bbbb00000
c     0000000000000550
c     0000000000000550
c     0000000000000000
c
c     Vertical subdivisions:
c
c     0000000000000000
c     0001111000000000
c     0001111111100000
c     0001111111100000
c     0000000000000110
c     0000000000000110
c     0000000000000000
c
c     Grid id numbers:
c
c     0000000000000000
c     0002222000000000
c     0002222333300000
c     0002222333300000
c     0000000000000110
c     0000000000000110
c     0000000000000000
c
c     Input arguments:
c               nxx:            number of coarse grid cells in x-direction
c               nyy:            number of coarse grid cells in y-direction
c               iout:           diagnostic output file unit number
c               idebug:         diagnostic debug flag
c     Files written:
c               iout:           diagnostic file
c               ifavg:          fine grid average concentration output file
c               ifdep:          fine grid deposition output file
c               ifinst:         fine grid instantaneous concentration file
c     Called by:
c               setup
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'ngmap.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'hedmsg.cmd'
c
      character*1 clast, vlast
      character*35 cdivid
      common /fgsav/ mfhrz(mxy), mfvrt(mxy)
      character*140 mfhrz, mfvrt
      logical lerr
c
      data cdivid/'123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      data ione /1/
c
c   fourth bit of idebug (i.e., 8's place) set will turn on
c   diagnostic writes in rvmmap
c
      idiag = and(idebug,8)
      lerr = .false.
      write(iout,2221)
 2221 format(' entered rvmmap')
c
c  fill in fine grid maps
c
c  initialize ingmap and ifngmp to zeros
c  initialize numgrd to zero (i.e., entire domain is coarse grid)
c  initialize fine grid maps to indicate all coarse grid
c  initialize kvmap to indicate one-to-one correspondence of layers
c             between all grids
c
        do 10 ifine=1,numfin
          do 6 j=1,mxfy
            do 5 i=1,mxfx
              ifngmp(i,j,ifine) = 0
    5       continue
    6     continue
          do 8 k=1,mxfz
            kvmap(k,ifine) = k
    8     continue
   10   continue
        do 15 j=1,mxy
          do 14 i=1,mxx
            ingmap(i,j) = 0
            numgrd(i,j) = 0
            mfhrz(j)(i:i) = '0'
            mfvrt(j)(i:i) = '0'
   14     continue
   15   continue
c
c  set up remaining parameters for fine grid
c  (number and location read in the routine setup)
c
        do 420 ifine=1,numfin
          ifgptr(ifine) = 0
          ifglvl(ifine) = 1
c
c         check if this fine grid is within any other fine grids
c         defined so far
c
          do 640 if0 = 1,numfin
            if (if0 .eq. ifine) go to 640
            if (ixfb(ifine) .ge. ixfb(if0) .and.
     &          ixfb(ifine) .le. ixfe(if0) .and.
     &          jyfb(ifine) .ge. jyfb(if0) .and.
     &          jyfb(ifine) .le. jyfe(if0) ) then
              if (ixfe(ifine) .lt. ixfb(if0) .and.
     &            ixfe(ifine) .gt. ixfe(if0) .and.
     &            jyfe(ifine) .lt. jyfb(if0) .and.
     &            jyfe(ifine) .gt. jyfe(if0) ) then
                write(iout,*) ' ***error***',
     &              ' improper nesting at ',ixfb(ifine),
     &              jyfb(ifine)
                lerr = .true.
              endif
              if (if0 .gt. ifine) then
                write(iout,*) ' ******error******'
                write(iout,*) ' Grid nests defined in improper order.'
                write(iout,*) ' Grid ',ifine,' is inside grid ',if0
                write(iout,*) ' Definition order must be from outer'
                write(iout,*) ' grid to inner grids.'
                lerr = .true.
              endif
              ifgptr(ifine) = if0
              ifglvl(ifine) = ifglvl(if0) + 1
            endif
  640     continue
c
  642     nfx(ifine) = (ixfe(ifine)-ixfb(ifine)+1)*nhf(ifine)+2
          nfy(ifine) = (jyfe(ifine)-jyfb(ifine)+1)*nhf(ifine)+2
          nvf(ifine) = (nfz(ifine)-1)/nlayer+1
          if (idiag .ne. 0) then
            write(iout,'(4x,3i4,4x,2i4)')
     &          nfx(ifine), nfy(ifine), nfz(ifine),
     &          nhf(ifine), nvf(ifine)
          endif
c
c          check all these parameters against the maxima
c
          if (nfx(ifine) .gt. mxfx .or. 
     &        nfy(ifine) .gt. mxfy .or.
     &        nfz(ifine) .gt. mxfz) then
            write(iout,'(a,3i4,/,a,3i4)')
     &        ' maximum x y or z exceeded for fine grid',
     &        nfx(ifine), nfy(ifine), nfz(ifine),
     &        ' max values are:',mxfx,mxfy,mxfz
            lerr = .true.
          endif
          if (nfx(ifine) .gt. mxxx .or.
     &        nfy(ifine) .gt. mxxx) then
            write(iout,'(a,2i4,a,i4,/,a)') 
     &        'x or y fine grid size of ',nfx(ifine), nfy(ifine),
     &        ' exceeds maximum of ',mxxx,
     &        ' allowed in common shas'
            lerr = .true.
          endif
          ixby = nfx(ifine)*nfy(ifine)
          if (ixby .gt. mxfxby) then
            write(iout,'(a,3i5,/,a,i5)') 
     &        ' max x by y dimension exceeded',
     &        nfx(ifine), nfy(ifine), ixby,
     &        ' max is ',mxfxby
            lerr = .true.
          endif
          if (nhf(ifine) .gt. maxdiv .or. 
     &        nvf(ifine) .gt. maxdiv ) then
            write(iout,'(a,2i4,/,a,i4)')
     &        ' maximum subdivisions exceeded for fine grid',
     &        nhf(ifine), nvf(ifine),
     &        ' max value is: ',maxdiv
            lerr = .true.
          endif
c
  420   continue
c
c   write out fine grid maps for user to check
c   also fill in map of nest interfaces
c
        write(iout,'(1x,i4,a)') numfin, ' fine grids defined.'
        write(iout,'(a)') ' the following definition of fine grids ',
     &                    ' will be used:',' '
        do 45 nn=1,numfin
          if (ifgptr(nn) .ne. 0) then
            if2 = ifgptr(nn)
            ixff0 = (ixfb(nn) - ixfb(if2))*nhf(if2) + 2
            ixff1 = (ixfe(nn) - ixfb(if2) + 1)*nhf(if2) + 1
            jyff0 = (jyfb(nn) - jyfb(if2))*nhf(if2) + 2
            jyff1 = (jyfe(nn) - jyfb(if2) + 1)*nhf(if2) + 1
            ii = ixff0 - 1
            ii2 = ixff1
            do 23 j=jyff0,jyff1
              ifngmp(ii,j,if2) = 1
              ifngmp(ii2,j,if2) = 1
   23       continue
            jj = jyff0 - 1
            jj2 = jyff1
            do 24 i=ixff0,ixff1
              ifngmp(i,jj,if2) = ifngmp(i,jj,if2) + 10
              ifngmp(i,jj2,if2) = ifngmp(i,jj2,if2) + 10
   24       continue
          else
            ii = ixfb(nn) - 1
            ii2 = ixfe(nn)
            do 25 j=jyfb(nn),jyfe(nn)
              ingmap(ii,j) = 1
              ingmap(ii2,j) = 1
   25       continue
            jj = jyfb(nn) - 1
            jj2 = jyfe(nn)
            do 26 i=ixfb(nn),ixfe(nn)
              ingmap(i,jj) = ingmap(i,jj) + 10
              ingmap(i,jj2) = ingmap(i,jj2) + 10
   26       continue
          endif
c
          if (nhf(nn) .lt. 35) then
            clast = cdivid(nhf(nn):nhf(nn))
          else
            clast = '*'
          endif
c         if (nvf(nn) .lt. 35) then
c           vlast = cdivid(nvf(nn):nvf(nn))
c         else
c           vlast = '*'
c         endif
          do 40 j=jyfb(nn), jyfe(nn)
            do 35 i=ixfb(nn), ixfe(nn)
              mfhrz(j)(i:i) = clast
c             mfvrt(j)(i:i) = vlast
              numgrd(i,j) = nn
   35       continue
   40     continue
   45   continue
c
        write(iout,'(/,a,/)') '     horizontal subdivisions:'
        do 47 j=nyy,1,-1
          write(iout,'(5x,120a1)') (mfhrz(j)(i:i),i=1,nxx)
   47   continue
c       write(iout,'(/,a,/)') '     vertical subdivisions:'
c       do 48 j=nyy,1,-1
c         write(iout,'(5x,120a1)') (mfvrt(j)(i:i),i=1,nxx)
c  48   continue
        write(iout,'(/,a,/)') '     map of grid id nos.:'
        do 49 j=nyy,1,-1
          write(iout,'(5x,120z1)') (numgrd(i,j),i=1,nxx)
   49   continue
c
c   look for fine grids that are adjacent to each other
c   set igloc and fhloc to point to row/col to use for
c   inflow from the adjacent grid
c   (igloc = 0 means no fine grid adjacent to this one,
c    otherwise the value is the fine grid that is adjacent)
c
        do 90 ifine=1,numfin
          do 53 iedg=1,2
            do 51 j=1,nfx(ifine)
              igloc(j,iedg,ifine) = 0
              fhloc(j,iedg,ifine) = 0.
   51       continue
            do 52 j=1,nfy(ifine)
              igloc(j,iedg+2,ifine) = 0
              fhloc(j,iedg+2,ifine) = 0.
   52       continue
   53     continue
c
c         --- check what sits on north edge of ifine
c
          iedg = 1
          do 56 ic=ixfb(ifine),ixfe(ifine)
            ic1 = (ic-ixfb(ifine))*nhf(ifine)+2
            ic2 = ic1 + nhf(ifine) - 1
            if0 = numgrd(ic,jyfe(ifine)+1)
            if (if0 .eq. 0) then
              fh1 = float(ic-1)
              hrat = 1./float(nhf(ifine))
              do 54 i=ic1,ic2
                igloc(i,iedg,ifine) = if0
                fhloc(i,iedg,ifine) = fh1 + float(i-ic1)*hrat
                jhloc(i,iedg,ifine) = jyfe(ifine)+1
   54         continue
            else
              fh1 = float((ic-ixfb(if0))*nhf(if0)+1)
              hrat = float(nhf(if0))/float(nhf(ifine))
              do 55 i=ic1,ic2
                igloc(i,iedg,ifine) = if0
                fhloc(i,iedg,ifine) = fh1 + float(i-ic1)*hrat
                jhloc(i,iedg,ifine) = (jyfe(ifine)+1-jyfb(if0))
     &                                 *nhf(if0)+2
   55         continue
            endif
   56     continue
c
c         --- check what sits on south edge of ifine
c
          iedg = 2
          do 66 ic=ixfb(ifine),ixfe(ifine)
            ic1 = (ic-ixfb(ifine))*nhf(ifine)+2
            ic2 = ic1 + nhf(ifine) - 1
            if0 = numgrd(ic,jyfb(ifine)-1)
            if (if0 .eq. 0) then
              fh1 = float(ic-1)
              hrat = 1./float(nhf(ifine))
              do 64 i=ic1,ic2
                igloc(i,iedg,ifine) = if0
                fhloc(i,iedg,ifine) = fh1 + float(i-ic1)*hrat
                jhloc(i,iedg,ifine) = jyfb(ifine)-1
   64         continue
            else
              fh1 = float((ic-ixfb(if0))*nhf(if0)+1)
              hrat = float(nhf(if0))/float(nhf(ifine))
              do 65 i=ic1,ic2
                igloc(i,iedg,ifine) = if0
                fhloc(i,iedg,ifine) = fh1 + float(i-ic1)*hrat
                jhloc(i,iedg,ifine) = (jyfb(ifine)-jyfb(if0))
     &                                 *nhf(if0)+1
   65         continue
            endif
   66     continue
c
c         --- check what sits on east edge of ifine
c
          iedg = 3
          do 76 jc=jyfb(ifine),jyfe(ifine)
            jc1 = (jc-jyfb(ifine))*nhf(ifine)+2
            jc2 = jc1 + nhf(ifine) - 1
            if0 = numgrd(ixfe(ifine)+1,jc)
            if (if0 .eq. 0) then
              fh1 = float(jc-1)
              hrat = 1./float(nhf(ifine))
              do 74 j=jc1,jc2
                igloc(j,iedg,ifine) = if0
                fhloc(j,iedg,ifine) = fh1 + float(j-jc1)*hrat
                jhloc(j,iedg,ifine) = ixfe(ifine)+1
   74         continue
            else
              fh1 = float((jc-jyfb(if0))*nhf(if0)+1)
              hrat = float(nhf(if0))/float(nhf(ifine))
              do 75 j=jc1,jc2
                igloc(j,iedg,ifine) = if0
                fhloc(j,iedg,ifine) = fh1 + float(j-jc1)*hrat
                jhloc(j,iedg,ifine) = (ixfe(ifine)+1-ixfb(if0))
     &                                 *nhf(if0)+2
   75         continue
            endif
   76     continue
c
c         --- check what sits on west edge of ifine
c
          iedg = 4
          do 86 jc=jyfb(ifine),jyfe(ifine)
            jc1 = (jc-jyfb(ifine))*nhf(ifine)+2
            jc2 = jc1 + nhf(ifine) - 1
            if0 = numgrd(ixfb(ifine)-1,jc)
            if (if0 .eq. 0) then
              fh1 = float(jc-1)
              hrat = 1./float(nhf(ifine))
              do 84 j=jc1,jc2
                igloc(j,iedg,ifine) = if0
                fhloc(j,iedg,ifine) = fh1 + float(j-jc1)*hrat
                jhloc(j,iedg,ifine) = ixfb(ifine)-1
   84         continue
            else
              fh1 = float((jc-jyfb(if0))*nhf(if0)+1)
              hrat = float(nhf(if0))/float(nhf(ifine))
              do 85 j=jc1,jc2
                igloc(j,iedg,ifine) = if0
                fhloc(j,iedg,ifine) = fh1 + float(j-jc1)*hrat
                jhloc(j,iedg,ifine) = (ixfb(ifine)-ixfb(if0))
     &                                 *nhf(if0)+1
   85         continue
            endif
   86     continue
c
c  diagnostics--- write out everything for this fine grid
c
          if (idiag .ne. 0) then
            write(iout,'(a,4i10)') 'Fine grid ',ifine, nfx(ifine), 
     &                         nfy(ifine), nfz(ifine)
            do 88 iedg=1,4
              nnn = nfx(ifine)
              if (iedg .gt. 2) nnn = nfy(ifine)
              write(iout,'(a,i3,a)') 'iedg = ',iedg,'  igloc:'
              write(iout,*) (igloc(j,iedg,ifine),j=1,nnn)
              write(iout,'(a)') '            fhloc:'
              write(iout,*) (fhloc(j,iedg,ifine),j=1,nnn)
              write(iout,'(a)') '            jhloc:'
              write(iout,*) (jhloc(j,iedg,ifine),j=1,nnn)
   88       continue
          endif
   90   continue
c
        if (lerr) then
          write(iout,*) ' Stopping due to errors in nest definition.'
          stop
        endif
c
c   write header info to fine grid average and inst files that
c   fully defines location and extent of each fine grid.
c   also write species simulated (in order).
c
        if (ifavg .gt. 0) then
          write(ifavg) msg
          write(ifavg) numfin, navspc
          write(ifavg) (mavspc(l),l=1,navspc)
        endif
c
        if (ifdep .gt. 0) then
          write(ifdep) msg
          write(ifdep) numfin, navspc
          write(ifdep) (mavspc(l),l=1,navspc)
        endif
c
        if (ifinst .gt. 0) then
          write(ifinst) msg
          write(ifinst) numfin, nospec
          write(ifinst)((mspec(i,l),i=1,10),l=1,nospec)
        endif
c
c--- same for OH output file if necessary.  dfg, 4/14/99
c
         if (ifohf .gt. 0) then
           write(ifohf) msg
           write(ifohf) numfin, 7
           write(ifohf) (msoh(i),i=1,10)
           write(ifohf) (msho2(i),i=1,10)
           write(ifohf) (msno3(i),i=1,10)
           write(ifohf) (msaso2(i),i=1,10)
           write(ifohf) (mspec(i,kh2o2),i=1,10)
           write(ifohf) (mspec(i,ko3),i=1,10)
           write(ifohf) (mspec(i,kpec),i=1,10)
        end if
c
c   loop over fine grids, writing grid definition
c
        do 4881 ifine=1,numfin
          if (ifavg .gt. 0) write(ifavg) ixfb(ifine), jyfb(ifine),
     &                        ixfe(ifine), jyfe(ifine), nhf(ifine),
     &                        nvf(ifine), nfx(ifine), nfy(ifine), 
     &                        ione, ifgptr(ifine), ifglvl(ifine)
          if (ifdep .gt. 0) write(ifdep) ixfb(ifine), jyfb(ifine),
     &                        ixfe(ifine), jyfe(ifine), nhf(ifine),
     &                        nvf(ifine), nfx(ifine), nfy(ifine), 
     &                        ione, ifgptr(ifine), ifglvl(ifine)
          if (ifinst .gt. 0) write(ifinst) ixfb(ifine), jyfb(ifine),
     &                        ixfe(ifine), jyfe(ifine), nhf(ifine),
     &                        nvf(ifine), nfx(ifine), nfy(ifine), 
     &                        nfz(ifine), ifgptr(ifine), ifglvl(ifine)
          if (ifohf .gt. 0 ) write(ifohf) ixfb(ifine), jyfb(ifine),
     &                        ixfe(ifine), jyfe(ifine), nhf(ifine),
     &                        nvf(ifine), nfx(ifine), nfy(ifine),
     &                        nfz(ifine), ifgptr(ifine), ifglvl(ifine)
 4881   continue
c
      return
      end
