c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
c
      subroutine soapart(poa,oh,press,temp,dt,svg,svp,mwv,cpfac,n)
c
c-----ATDM v7.00
c
c     Partitions semi-volatile organics into gas and particulate
c     fractions
c     (See Odum et al., 1996, Env. Sci & Tech., 30:2580-2585)
c
c     Input arguments:
c               poa:           Primary organic aerosol (ppm)
c               oh:            oh radical concentration (ppm)
c               press:         Pressure (mb)
c               temp:          Ambient temperature (K)
c               dt:            time step (min)
c               svg:           Gas-phase concentration (ppm): 
c               svp:           Particle-phase concentration (ppm): 
c               n:             Number of semi-volatiles (currently
c                               coded for 4.)
c     Output arguments:
c               svg:           Gas-phase concentration (ppm): 
c               svp:           Particle-phase concentration (ppm): 
c     Called by:
c               rchem     rchemf    vrtdif    fvrtdif
c
      dimension pl0(4), hv(4), tref(4)
      dimension svg(n), svp(n)
      dimension xk(4), xkt(4), rkoh(4), sv(4)
      dimension xk3(4), xk4(4)
      include 'parfin.cmd'
      include 'kweight.cmd'
      real*8 dd(0:5), soln(5), solni(5)
      real mwv(n)
      data xk /0.047,0.0016,0.123,0.00435/
c
c  temperature dependence of partitioning coefficients based on 
c  Sheehan and Bowman (2001) Environmental Science & Technology, 
c  Vol. 35, pp 2129-2135, entitled "Estimated Effects of Temperature 
c  on Secondary Organic Aerosol Concentrations"
c  updated 3/5/03, tcm
c
c
c  corrected calculation of 1st order term in equilibrium eq.
c  tcm, 3/29/03
c
c  enthalpy of vaporization in kJoules/mole
      data hv /70., 70., 70., 70./
      data tref /308., 308., 308., 308./
c R in kJ/mole/K
      data rkj /0.008314/
c  partitioning coefficients for products of the TERP reactions, 
c  in the order O products, OH products, O3 products, and NO3 products
      data xk3 / 0.126386, 0.098145, 0.142002, 0.125645/
      data xk4 / 0.004837, 0.003490, 0.004704, 0.004385/
      data rkoh / 0., 1.e4, 0., 1.e5 /
c
c   calculate weighted average xk(3) and xk(4)
c
c     xk(3) = 0.25*(xk3(1) + xk3(2) + xk3(3) + xk3(4))
c     xk(4) = 0.25*(xk4(1) + xk4(2) + xk4(3) + xk4(4))
      xk(3) = wtk3(1)*xk3(1) + wtk3(2)*xk3(2) 
     &         + wtk3(3)*xk3(3) + wtk3(4)*xk3(4)
      xk(4) = wtk4(1)*xk4(1) + wtk4(2)*xk4(2) 
     &         + wtk4(3)*xk4(3) + wtk4(4)*xk4(4)
c
c   adjust equilibrium k's for temperature
c
      do i=1,n
Cglg    
C        write (*,*) 'Semi Volatiles: i, hv, temp, xk '
C        write(*,*) i, hv(i), temp, xk(i)
Cglg
        xkt(i) = xk(i) * temp/tref(i) 
     &           * exp(hv(i)/rkj*(1./temp - 1./tref(i)))
Cglg
C        write (*,*) 'xk(i) = ', xk(i)
Cglg
      enddo
c
c   calculate reaction of sv1 and sv3 to form sv2 and sv4 respectively
c
c     write(*,*) 'in soapart'
      sv0 = svg(2)
      svg(2) = svg(2)*exp(-rkoh(2)*oh*dt)
      svg(1) = svg(1) + sv0 - svg(2)
      sv0 = svg(4)
      svg(4) = svg(4)*exp(-rkoh(4)*oh*dt)
      svg(3) = svg(3) + sv0 - svg(4)
c
c   convert concentrations to ugm/m3
c
c     write(*,*) 'calculating facm'
      facm = cpfac*(1013.25/press)*(temp/273.)
      poaug = poa/facm
c     write(*,*) 'facm, poaug, poa, n, mwv: '
c     write(*,*) facm, poaug, poa, n, mwv
      do i=1,n
        svg(i) = svg(i)*mwv(i)/facm
        svp(i) = svp(i)*mwv(i)/facm
      enddo
c     write(*,*) 'done converting units'
c
c  inserted and iteration loop here to ensure that we reach
c  equilibrium (tcm, 3/5/03)
c
      soalast = 0.
      do i=1,n
        soalast = soalast + svp(i)
      enddo
      do iter=1,10
c
c   we solve for the gas/aerosol by first solving for xm0, which
c   is the total organic aerosol mass
c
        xksum = 0.
        xksvsum = 0.
        xkprod = 1.
        do i=1,n
Cglg
C        write (*,*) 'i. xkt(i), xksum, xkprod = ',i,xkt(i),xksum,xkprod
Cglg
          xksum = xksum + xkt(i)
          xkprod = xkprod*xkt(i)
          sv(i) = svg(i) + svp(i)
          xksvsum = xksvsum + xkt(i)*sv(i)
        enddo
c       write(*,*) 'done calculating sum and product'
        dd(5) = xkprod
        dd(4) = xkt(1)*xkt(3)*xkt(4) + xkt(2)*xkt(3)*xkt(4) 
     &          + xkt(1)*xkt(2)*xkt(3) + xkt(1)*xkt(2)*xkt(4) 
     &          - xkprod*(poaug+sv(1)+sv(2)+sv(3)+sv(4))
        dd(3) = xkt(1)*xkt(2) + xkt(1)*xkt(3) + xkt(1)*xkt(4) 
     &          + xkt(2)*xkt(3) + xkt(2)*xkt(4) + xkt(3)*xkt(4) 
     &          - xkt(1)*xkt(2)*xkt(3)*(poaug+sv(1)+sv(2)+sv(3))
     &          - xkt(1)*xkt(2)*xkt(4)*(poaug+sv(1)+sv(2)+sv(4))
     &          - xkt(1)*xkt(3)*xkt(4)*(poaug+sv(1)+sv(3)+sv(4))
     &          - xkt(2)*xkt(3)*xkt(4)*(poaug+sv(2)+sv(3)+sv(4))
        dd(2) = xksum - xkt(1)*xkt(2)*(poaug+sv(1)+sv(2))
     &          - xkt(1)*xkt(3)*(poaug+sv(1)+sv(3))
     &          - xkt(1)*xkt(4)*(poaug+sv(1)+sv(4))
     &          - xkt(2)*xkt(3)*(poaug+sv(2)+sv(3))
     &          - xkt(2)*xkt(4)*(poaug+sv(2)+sv(4))
     &          - xkt(3)*xkt(4)*(poaug+sv(3)+sv(4))
        dd(1) = 1.- poaug*xksum - xksvsum
        dd(0) = -poaug
c       write(*,*) ' dd: ', dd
c       write(*,*) ' xk: ', xk
c       write(*,*) ' sv: ', sv
c       write(*,*) ' poaug: ', poaug
c       write(*,*) ' xkprod: ', xkprod
c       write(*,*) ' xksum: ', xksum
c       write(*,*) 'calling quintic'
        call quintic(dd,soln,solni,Nsol)
c       write(*,*) 'Nsol: ', nsol
        if (Nsol .eq. -9) then
c         write(*,*) 'Max iter exceeded in quintic. '
c         write(*,*) 'Setting soa mass to original and recalculating '
c         write(*,*) 'final distribution on that basis.'
c         write(*,*) 'dd: ', dd
c         write(*,*) 'svg: ', svg
c         write(*,*) 'svp: ', svp
c         write(*,*) 'xk: ', xk
c         write(*,*) 'poa, poaug: ', poa, poaug
          xm0 = poaug+svp(1)+svp(2)+svp(3)+svp(4)
        else if (Nsol .eq. 0) then
c         write(*,*) 'No solution in quintic'
c         write(*,*) 'Setting soa mass to original and recalculating '
c         write(*,*) 'final distribution on that basis.'
c         write(*,*) 'dd: ', dd
c         write(*,*) 'svg: ', svg
c         write(*,*) 'svp: ', svp
c         write(*,*) 'xk: ', xk
c         write(*,*) 'poa, poaug: ', poa, poaug
          xm0 = poaug+svp(1)+svp(2)+svp(3)+svp(4)
        else if (Nsol .eq. 1) then
          xm0 = soln(1)
        else if (Nsol .gt. 1) then
          is = 0
          do i=1,Nsol
            if (soln(i) .ge. 0.) then 
              if (soln(i) .le. poaug+sv(1)+sv(2)+sv(3)+sv(4)) then
                xm0 = soln(i)
                is = is + 1
              endif
            endif
          enddo
          if (is .ne. 1) then
Cglg          write(*,*) 'Ambiguous solution from quintic.'
Cglg          write(*,*) 'is = ', is
Cglg          write(*,*) ' soln: ', soln
Cglg          write(*,*) ' xkt  : ', xkt
Cglg          write(*,*) ' sv  : ', sv
Cglg          write(*,*) ' svp : ', svp 
Cglg          write(*,*) ' svg : ', svg 
            xm0 = poaug+svp(1)+svp(2)+svp(3)+svp(4)
c           stop
          endif
        endif
c
c   determine gas/particle fraction of each sv species
c
        soanow = 0.
        do i=1,n
          fx = xm0*xkt(i)/(1.+xm0*xkt(i))
          svp(i) = sv(i)*fx
          svg(i) = sv(i)*(1.-fx)
          soanow = soanow + svp(i)
        enddo
c       write(*,*) 'svg ugm/m3): ', svg
c       write(*,*) 'svp ugm/m3): ', svp
        if (abs(soanow-soalast) .lt. 0.1*soanow) go to 50
        soalast = soanow
      enddo
c
c   convert concentrations back to ppm
c
   50 do i=1,n
        svg(i) = svg(i)*facm/mwv(i)
        svp(i) = svp(i)*facm/mwv(i)
      enddo
c
c     write(*,*) 'leaving soapart'
      return
      end

C     ***QUINTIC************************************************25.03.98
C     Solution of a quintic equation by a hybrid method:
C     first real solution is obtained numerically by the Newton method,
C     the remaining four roots are obtained analytically by QUARTIC
C     NO WARRANTY, ALWAYS TEST THIS SUBROUTINE AFTER DOWNLOADING
C     ******************************************************************
C     dd(0:4)     (i)  vector containing the polynomial coefficients
C     sol(1:4)    (o)  results, real part
C     soli(1:4)   (o)  results, imaginary part
C     Nsol        (o)  number of real solutions 
C     ==================================================================
      subroutine quintic(dd,sol,soli,Nsol)
      implicit double precision (a-h,o-z)
      parameter( eps = 1.d-8 )
      dimension dd(0:5),sol(5),soli(5)
      dimension dd4(0:4),sol4(4),soli4(4)
      common /start/ xstart
      data maxitr /100/
C
      Nsol = 0
      if (dd(5).eq.0.d+0) then
C	write(6,*)'ERROR: NOT A QUINTIC EQUATION'
	return
      endif
C
C     Newton iteration of one real root 
      iter = 0
      xs = xstart
 4    sum  = dd(0)
      iter = iter + 1
      if (iter .gt. maxitr) then
c       write(*,*) 'Exceeded max iterations in quintic.'
        Nsol = -9
        return
      endif
      do 5 i=1,5
 5      sum = sum + dd(i)*xs**i
      sum1 = dd(1)
      do 6 i=1,4
 6      sum1 = sum1 + float(i+1)*dd(i+1)*xs**i
Cglg
      if (sum1 .eq. 0.0) then
         Nsol = 0
         return
      end if
Cglg
      xnew = xs - sum/sum1
      if (abs(xnew-xs).gt.eps) then
	xs = xnew
	goto 4
      endif
      e = xnew
C
C     "e" is one real root of quintic equation
C     reduce quintic to quartic equation using "e"
      dd4(4) = dd(5)
      do 8 i=4,1,-1
 8      dd4(i-1) = dd(i) + e*dd4(i)
c     do 9 i=0,4
c9      write(6,*) dd4(i)
      call quartic(dd4,sol4,soli4,Nsol4)
      sol(1)  = e
      soli(1) = 0.d+0
      do 10 i=1,4
        sol(i+1)  = sol4(i)
        soli(i+1) = soli4(i)
 10   continue
      Nsol = Nsol4 + 1
C
      return
      END


C     ***QUARTIC************************************************25.03.98
C     Solution of a quartic equation
C     ref.: J. E. Hacke, Amer. Math. Monthly, Vol. 48, 327-328, (1941)
C     NO WARRANTY, ALWAYS TEST THIS SUBROUTINE AFTER DOWNLOADING
C     ******************************************************************
C     dd(0:4)     (i)  vector containing the polynomial coefficients
C     sol(1:4)    (o)  results, real part
C     soli(1:4)   (o)  results, imaginary part
C     Nsol        (o)  number of real solutions 
C     ==================================================================
      subroutine quartic(dd,sol,soli,Nsol)
      implicit double precision (a-h,o-z)
      dimension dd(0:4),sol(4),soli(4)
      dimension AA(0:3),z(3)
C
      Nsol = 0
      if (dd(4).eq.0.d+0) then
	write(6,*)'ERROR: NOT A QUARTIC EQUATION'
	return
      endif
      a = dd(4)
      b = dd(3)
      c = dd(2)
      d = dd(1)
      e = dd(0)
C
      p = (-3.d+0*b**2 + 8.d+0*a*c)/(8.d+0*a**2)
      q = (b**3 - 4.d+0*a*b*c + 8.d+0*d*a**2)/(8.d+0*a**3)
      r = (-3.d+0*b**4 + 16.d+0*a*b**2*c - 64.d+0*a**2*b*d + 
     &      256.d+0*a**3*e)/(256.d+0*a**4)
C
C     solve cubic resolvent
      AA(3) =  8.d+0
      AA(2) = -4.d+0*p 
      AA(1) = -8.d+0*r
      AA(0) =  4.d+0*p*r - q**2
      call cubert(AA,z,ncube)
C      
      zsol = -1.d+99
      do 5 i=1,ncube
 5      zsol = max(zsol,z(i))
      z(1) = zsol
      xK2 = 2.d+0 * z(1) - p
      xK = sqrt(xK2)
Cglg
      if (xK.eq.0.) then
         Nsol = 0
         return
      end if
Cglg     
      xL = q/(2.d+0 * xK)
      sqp = xK2 - 4.d+0*(z(1) + xL)
      sqm = xK2 - 4.d+0*(z(1) - xL)
C
      do 10 i=1,4
 10     soli(i) = 0.d+0
      if       (sqp.ge.0.d+0 .and. sqm.ge.0.d+0) then
	sol(1) = 0.5d+0*( xK + sqrt(sqp))
	sol(2) = 0.5d+0*( xK - sqrt(sqp))
	sol(3) = 0.5d+0*(-xK + sqrt(sqm))
	sol(4) = 0.5d+0*(-xK - sqrt(sqm))
	Nsol = 4
      else if  (sqp.ge.0.d+0 .and. sqm.lt.0.d+0) then
	sol(1) =  0.5d+0*(xK + sqrt(sqp))
	sol(2) =  0.5d+0*(xK - sqrt(sqp))
	sol(3) = -0.5d+0*xK 
	sol(4) = -0.5d+0*xK 
	soli(3) =  sqrt(-0.25d+0 * sqm)
	soli(4) = -sqrt(-0.25d+0 * sqm)
	Nsol = 2
      else if  (sqp.lt.0.d+0 .and. sqm.ge.0.d+0) then
	sol(1) = 0.5d+0*(-xK + sqrt(sqm))
	sol(2) = 0.5d+0*(-xK - sqrt(sqm))
	sol(3) =  0.5d+0*xK 
	sol(4) =  0.5d+0*xK 
	soli(3) =  sqrt(-0.25d+0 * sqp)
	soli(4) = -sqrt(-0.25d+0 * sqp)
	Nsol = 2
      else if  (sqp.lt.0.d+0 .and. sqm.lt.0.d+0) then
	sol(1) = -0.5d+0*xK 
	sol(2) = -0.5d+0*xK 
	soli(1) =  sqrt(-0.25d+0 * sqm)
	soli(2) = -sqrt(-0.25d+0 * sqm)
	sol(3) =  0.5d+0*xK 
	sol(4) =  0.5d+0*xK 
	soli(3) =  sqrt(-0.25d+0 * sqp)
	soli(4) = -sqrt(-0.25d+0 * sqp)
	Nsol = 0
      endif
      do 20 i=1,4
 20     sol(i) = sol(i) - b/(4.d+0*a)
C
      return
      END

C     ***CUBIC************************************************08.11.1986
C     Solution of a cubic equation
C     Equations of lesser degree are solved by the appropriate formulas.
C     The solutions are arranged in ascending order.
C     NO WARRANTY, ALWAYS TEST THIS SUBROUTINE AFTER DOWNLOADING
C     ******************************************************************
C     A(0:3)      (i)  vector containing the polynomial coefficients
C     X(1:L)      (o)  results
C     L           (o)  number of valid solutions (beginning with X(1))
C     ==================================================================
      SUBROUTINE cubert(A,X,L)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(0:3),X(3),U(3)
      PARAMETER(PI=3.1415926535897932D+0,THIRD=1.D+0/3.D+0)
      INTRINSIC MIN,MAX,ACOS
C
C     define cubic root as statement function
      CBRT(Z)=SIGN(ABS(Z)**THIRD,Z)
C
C     ==== determine the degree of the polynomial ====
C
      IF (A(3).NE.0.D+0) THEN
C
C       cubic problem
	W=A(2)/A(3)*THIRD
	P=(A(1)/A(3)*THIRD-W**2)**3
	Q=-.5D+0*(2.D+0*W**3-(A(1)*W-A(0))/A(3))
	DIS=Q**2+P
	IF (DIS.LT.0.D+0) THEN
C         three real solutions!
C         Confine the argument of ACOS to the interval [-1;1]!
	  PHI=ACOS(MIN(1.D+0,MAX(-1.D+0,Q/SQRT(-P))))
	  P=2.D+0*(-P)**(5.D-1*THIRD)
	  DO 100 I=1,3
  100       U(I)=P*COS((PHI+DBLE(2*I)*PI)*THIRD)-W
	  X(1)=MIN(U(1),U(2),U(3))
	  X(2)=MAX(MIN(U(1),U(2)),MIN(U(1),U(3)),MIN(U(2),U(3)))
	  X(3)=MAX(U(1),U(2),U(3))
	  L=3
	ELSE
C         only one real solution!
	  DIS=SQRT(DIS)
	  X(1)=CBRT(Q+DIS)+CBRT(Q-DIS)-W
	  L=1
	END IF
C
      ELSE IF (A(2).NE.0.D+0) THEN
C
C       quadratic problem
	P=5.D-1*A(1)/A(2)
	DIS=P**2-A(0)/A(2)
	IF (DIS.GE.0.D+0) THEN
C         two real solutions!
	  X(1)=-P-SQRT(DIS)
	  X(2)=-P+SQRT(DIS)
	  L=2
	ELSE
C         no real solution!
	  L=0
	END IF
C
      ELSE IF (A(1).NE.0.D+0) THEN
C
C       linear equation
	X(1)=-A(0)/A(1)
	L=1
C
      ELSE
C       no equation
	L=0
      END IF
C
C     ==== perform one step of a newton iteration in order to minimize
C          round-off errors ====
      DO 110 I=1,L
	X(I)=X(I)-(A(0)+X(I)*(A(1)+X(I)*(A(2)+X(I)*A(3))))
     *  /(A(1)+X(I)*(2.D+0*A(2)+X(I)*3.D+0*A(3)))
  110 CONTINUE
      RETURN
      END
