c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine stepyf_04(nxf,nyf,nzf,nspecf,pstarf,psoldf,psnewf,
     &                   fkh,uf,vf,bcf,cncf,temprf,ch2of,psfcf,presf,
     &                   htf,dtf,deltht,ifine)
c
c-----ATDM v1.00
c
c     Performs horizontal advection in y directions for fine grid
c
c-----95/02/22:
c     -from UAM-V Revision 1.2  94/02/10  21:01:14  tcm
c     -moved call to WETDEPF from TERMSF to this routine
c     -removed DECAYF array
c     -removed d() array from common SHAS
c     -revised calculation of FDPFLO to include all layers, not just layer 1,
c      using RDECAYF straight out of WETDEPF routine
c     -modified SMOLAR concentration vector CX() to include reduction due to
c      wet deposition, using RDECAYF straight out of WETDEPF
c     -added DEPTHF, RAINF, & TEMPRF to argument list for WETDEPF call
c-----95/06/15:
c     -removed all code related to wet deposition (done in cloud sub-model)
c     -removed all code related to vertical advection (Lorenz cycle)
c     -advection performed for the quantity C* = P* x C
c-----95/09/18:
c     -restructured to scale C by P* value at beginning of time step, then
c      return to C after advection by dividing by P* value at end of time step
c     -alternating x/y advection for odd/even time step index
c     -added latitudinal cell scaling for y-advection to account for
c      cell width as a function of lat
c-----95/09/19:
c     -moved call to HGTNEW from ATDM to here to obtain new P* values
c
c     Input arguments:
c               nxf:            number of fine grid cells in x-direction
c               nyf:            number of fine grid cells in y-direction
c               nzf:            number of fine grid layers
c               nspecf:         number of modeled species
c               pstarf:         array of current P* values (mb)
c               psoldf:         old surface pressure from last met update (mb)
c               psnewf:         new surface pressure from next met update (mb)
c               fkh:            array of horizontal diffusivities (km2/min)
c               uf:             array of x-component windspeeds (km/min)
c               vf:             array of y-component windspeeds (km/min)
c               bcf:            array of fine grid boundary conditions (ppm)
c               cncf:           fine grid concentration array (ppm)
c               dtf:            fine grid time step (min)
c               deltht:         time since last met update (min)
c               ifine:          fine grid index
c     Output arguments:
c               cncf:           fine grid concentration array (ppm)
c     Files written:
c               6:              standard output file unit (*)     
c     Routines called:
c               smolar
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'balanc.cmd'
      include 'ngmap.cmd'
      include 'advec.cmd'
      include 'deffin.cmd'
      include 'latlon.cmd'
      include 'com3.cmd'
      include 'dtksubf.cmd'
c
      dimension fkh(nxf,nyf,nzf), uf(nxf,nyf,nzf), vf(nxf,nyf,nzf),
     &          pstarf(nxf,nyf),psoldf(nxf,nyf),psnewf(nxf,nyf),
     &          temprf(nxf,nyf,nzf),ch2of(nxf,nyf,nzf),psfcf(nxf,nyf),
     &          presf(nxf,nyf,nzf),htf(nxf,nyf,nzf)
      dimension bcf(mxfxy,4,mxspec,mxfz)
      dimension cncf(nxf,nyf,nspecf,nzf)
      common /shas/ w(mxxx),cx(mxxx),cb(mxxx),
     &              jadf(mxxx),nm1,b1,b2
      data maxpas /15/
c
      write(*,3001) dtf
 3001 format(' enter stepyf_04 ',f10.2)
c
c-------------------------------------------------------
c  Y ADVECTION
c
c  integrate one species at a time, one column at a time.
c
      do 240 k = 1,nzf
c
  101 j1 = 1
      j2 = nyf
      nm1 = nyf-1
      timlftk=dtf
      do 26 nstp=1,maxpas
c	write(*,*) ' STEPYF_04 ... nstp,timlftk :',nstp,timlftk
	if ( timlftk .le. 0.0001 ) goto 240
	if ( deltkf(k,ifine) .le. timlftk ) then
	  dtk=deltkf(k,ifine)
	else
	  dtk=timlftk
	endif
c
      do 260 i = 2,nxf-1
c
	timlfti=dtk
	do 95 nstpi=1,maxpas
	  if ( timlfti .le. 0.0001 ) goto 260
	  if ( dtfsj(i,k,ifine) .le. timlfti ) then
            dti=dtfsj(i,k,ifine)
	  else
            dti=timlfti
	  endif
c
        do 250 l = 1,nspecf
c
c     call advection scheme for each layer
c
            do 230 j = j1,j2
              jj = j - j1 + 1
              if (cncf(i,j,l,k) .ne. 0.0) then
                 if (cncf(i,j,l,k) .gt. 0.0) then
c                    write (*,*) i,j,k,l,cncf(i,j,l,k),delfx(j,ifine),delfy(ifine),dti,fkh(i,j,k),vf(i,j,k),ifngmp(i,j,ifine)
c                    write (*,*) 'stepyf'
c                    stop
                 end if
              end if
              if ( nstp .eq. 1 .and. nstpi .eq. 1 ) then
                cx(jj) = cncf(i,j,l,k)*delfx(j,ifine)
                cb(jj) = fkh(i,j,k)*dti/delfy(ifine)**2
                w(jj) = vf(i,j,k)
c
                if (ifngmp(i,j,ifine)/10 .eq. 0) then
                  jadf(jj) = 0
                else
                  jadf(jj) = 1
                endif
                if (ifgptr(ifine).eq.0) then
                  if (jj.le.nhf(ifine)) jadf(jj) = 1
                  if (jj.ge.nyf-nhf(ifine)) jadf(jj) = 1
                else
                  nnst = nhf(ifine)/nhf(ifgptr(ifine))
                  if (jj.le.nnst) jadf(jj) = 1
                  if (jj.ge.nyf-nnst) jadf(jj) = 1
                endif
              else
                cx(jj) = cncf(i,j,l,k)*delfx(j,ifine)
                cb(jj) = fkh(i,j,k)*dti/delfy(ifine)**2
              endif
  230       continue
            b1 = bcf(i,2,l,k)*delfx(j1,ifine)
            b2 = bcf(i,1,l,k)*delfx(j2,ifine)
c
c        write (*,*) 'before smolar', nstp, i, nstpi, l, j
            call smolar(dti,delfy(ifine))
c        write (*,*) 'after smolar', nstp, i, nstpi, l, j
c
            do 235 j = j1,j2
              jj = j - j1 + 1
              cncf(i,j,l,k) = amax1(bdnl(l),cx(jj)/delfx(j,ifine))
c              cncf(i,j,l,k) = cx(jj)/delfx(j,ifine)
  235       continue
c
c
  250   continue
c
         timlfti=timlfti-dti
c
   95    continue
  260 continue
c
       timlftk=timlftk-dtk
c
   26 continue
  240 continue
c
c      do 400 k = 1,nzf
c        do 401 l = 1,nspecf
c          do 402 j = 2,nyf-1
c            do 403 i = 2,nxf-1
c              cncf(i,j,l,k) = amax1(bdnl(l),cncf(i,j,l,k))
c 403        continue
c 402      continue
c 401    continue
c 400  continue
cc
      return
      end
