c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine stepz(deltat,idebug)
c
c-----ATDM v1.00
c
c     Uses simple upstream method ...
c
c     Computes vertical advection using Botts scheme for non-uniform grids
c     in sigma coordinates
c
c-----95/06/13:
c     C Emery SAI
c-----95/06/22:
c     -added idebug to argument list
c-----95/06/26:
c     -added sedimentation velocity component calculated in PARTSED
c      to sigdot
c-----95/07/14:
c     -removed calculation of sub-steps as this is an implicit scheme
c-----95/10/04:
c     -restructured code that adds vg to sigdot
c-----95/11/08:
c     -added code to skip cells covered by a fine grid
c-----95/12/12:
c     -added code to treat vg for nh4s as average of gso4 and aso4
c
c     Based on routines from SAQM/RADM
c
c     Input arguments:
c               deltat:         model time step (min)       
c               idebug:         diagnostic debug flag
c     Called by:
c               atdm 
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'deffin.cmd'
c
      real i_minus(mxz+1),i_plus(mxz+1),wf(mxz),vciflux(mxz+1)
      real sigmah(mxz),dsigh(mxz),dtodz(mxz),dsigi(mxz),vci(mxz)
      real a1co(3,mxz),a2co(3,mxz),vrtvel(mxz)
      logical lprnt
c
c-----Bit 8 (128's location) turns on debug writes in stepz
c
      idiag = and(idebug,128)
      if (idiag.ne.0) write(*,*)'Entering STEPZ, deltat = ',deltat
c
c-----Calculate depth parameters
c
      kmax = nlayer
      do 10 k = 1,kmax
        sigmah(k) = (sigma(k) + sigma(k+1))/2.
        dsigh(k) = sigma(k+1) - sigma(k)
   10 continue
      do 11 k = 1,kmax-1
        dsigi(k) = 1./(sigmah(k+1) - sigmah(k))
   11 continue
c
c-----Compute polynomial coefficients
c
c     coefficients for k=1 and k=nlayer
c
c      a1co(1,1) = dsigh(1)*dsigi(1)/8.
c      a1co(2,1) = -a1co(1,1)
c      a1co(2,kmax) = dsigh(kmax)*dsigi(kmax-1)/8.
c      a1co(3,kmax) = -a1co(2,kmax)
c
c     for k = 2 to nlayer-1
c
c      do 20 k = 2,kmax-1
c        dod = dsigi(k)/dsigi(k-1)
c        dpd = sigmah(k+1) - sigmah(k-1)
c        ds2o24 = dsigh(k)*dsigh(k)/24.
c        a1co(1,k) = dod/dpd*dsigh(k)/8.
c        a1co(2,k) = (1. - dod)*dsigi(k-1)*dsigh(k)/8.
c        a1co(3,k) = -1./dod/dpd*dsigh(k)/8.
c        a2co(1,k) = dsigi(k)/dpd*ds2o24
c        a2co(2,k) = -dsigi(k-1)*dsigi(k)*ds2o24
c        a2co(3,k) = dsigi(k-1)/dpd*ds2o24
c 20   continue
c
c-----Loop over rows and columns
c
      do 100 j = 1,noy
c
c-----Check for outside of modeling region
c
        if (istr(j).ge.nox) goto 100
        i1 = istr(j) + 1
        i2 = iend(j) - 1
        do 120 i = i1,i2
          if (j.le.jstr(i)) goto 120
          if (j.ge.jend(i)) goto 120
c
c-----Skip cells covered by fine grid
c
          do 130 ifine = 1,numfin
            if (i.ge.ixfb(ifine) .and. i.le.ixfe(ifine) .and.
     &          j.ge.jyfb(ifine) .and. j.le.jyfe(ifine)) goto 120
  130     continue
c
          lprnt = .false.
          if (i.eq.70 .and. j.eq.30) lprnt = .true.
c
c-----Loop over all species; load work array vci
c     and add particle sedimentation velocity to sigmadot
c
          do 150 l = 1,nospec
            do 200 k = 1,kmax
              vci(k) = conc(i,j,l,k)
              if (k.lt.kmax) then
	        if (l.eq.knh4s) then
		  vgt=conc(i,j,kgso4,k+1)+conc(i,j,kaso4,k+1)
		  if ( abs(vgt) .gt. 0.0 ) then
		    vg(i,j,k+1,knh4s)=
     &                    (vg(i,j,k+1,kgso4)*conc(i,j,kgso4,k+1)+
     &                     vg(i,j,k+1,kaso4)*conc(i,j,kaso4,k+1))/vgt
		  else
		    vg(i,j,k+1,knh4s)=0.0
	   	  endif
		endif
                if (vg(i,j,k+1,l).gt.0.) then
                  pstim = (h(i,j,k+1) - h(i,j,k))/vg(i,j,k+1,l)
                  vrtvel(k) = sigdot(i,j,k) + abs(dsigh(k+1))/pstim
                else
                  vrtvel(k) = sigdot(i,j,k)
                endif
                if (lprnt .and. idiag.ne.0) then
                  write(*,*)'k,l,h,h+,vg,sigdot,pstim,dsigh,vrtvel'
                  write(*,*) k,l,h(i,j,k),h(i,j,k+1),vg(i,j,k+1,l),
     &                       sigdot(i,j,k),pstim,dsigh(k+1),vrtvel(k)
                endif
              else
                vrtvel(k) = 0.
              endif
 200        continue
c
c-----Determine number of vertical advection sub-steps
c
c           dtmin = 60.*deltat
c           do 30 k = 1,kmax-1
c             dt = amin1(abs(dsigh(k)),abs(dsigh(k+1)))/
c    &                  (2.*abs(vrtvel(k)))
c             dtmin = amin1(dtmin,dt)
c  30       continue
c           if (dtmin.ge.60.*deltat) then
c             ndt = 1
c           else
c             ndt = int(60.*deltat/dtmin) + 1
c           endif
            ndt = 1
            dt = 60.*deltat/float(ndt)
            do 22 k = 1,kmax
              dtodz(k) = dt/abs(dsigh(k))
   22       continue
c
c-----Loop over sub-steps; initialize vertical arrays
c
            do 160 n = 1,ndt
c
c            do 201 k = 1,kmax+1
c              i_minus(k) = 0.
c              i_plus(k)  = 0.
c 201        continue
c
c-----First order for layer 1 and kmax 
c
c            a0 = vci(1)
c            a1 = a1co(1,1)*vci(2) + a1co(2,1)*vci(1)
c            if (vrtvel(1).lt.0.) then
c              cj_plus = -vrtvel(1)*dtodz(1)
c              x1 = 1. - 2.*cj_plus
c              x2 = x1*x1
c              temp = a0*cj_plus + a1*(1. - x2)
c              i_plus(2) = amax1(0.,temp)
c            endif
c            weight = amax1(i_plus(2),1.e-10,a0)
c            wf(1) = vci(1)*dsigh(1)/weight
cc
c            a0 = vci(kmax)
c            a1 = a1co(2,kmax)*vci(kmax) + a1co(3,kmax)*vci(kmax-1)
c            if (vrtvel(kmax-1).gt.0.) then
c              cj_minus = vrtvel(kmax-1)*dtodz(kmax)
c              x1 = 1. - 2.*cj_minus
c              x2 = x1*x1
c              temp = a0*cj_minus - a1*(1. - x2)
c              i_minus(kmax) = amax1(0.,temp)
c            endif
c            weight = amax1(i_minus(kmax),1.e-10,a0)
c            wf(kmax) = vci(kmax)*dsigh(kmax)/weight
cc
cc-----Second order scheme for k = 2 to kmax-1
cc
c            do 210 k = 2,kmax-1
cc
c              a0 = vci(k)
c              a1 = a1co(1,k)*vci(k+1) + a1co(2,k)*vci(k) +
c     & 	           a1co(3,k)*vci(k-1)
c              a2 = a2co(1,k)*vci(k+1) + a2co(2,k)*vci(k) +
c     &             a2co(3,k)*vci(k-1)
cc
c              if (vrtvel(k-1).gt.0.) then
c                cj_minus = vrtvel(k-1)*dtodz(k)
c                x1 = 1. - 2.*cj_minus
c                x2 = x1*x1
c                temp = a0*cj_minus - a1*(1. - x2) + a2*(1. - x1*x2)
c                i_minus(k) = amax1(0.,temp)
c              endif
cc
c              if (vrtvel(k).lt.0.) then
c                cj_plus = -vrtvel(k)*dtodz(k)
c                x1 = 1. - 2.*cj_plus
c                x2 = x1*x1
c                temp = a0*cj_plus + a1*(1. - x2) + a2*(1. - x1*x2)
c                i_plus(k+1) = amax1(0.,temp)
c	      endif
cc
c              weight = amax1(i_plus(k+1)+i_minus(k),1.e-10,a0+2.*a2)
c              wf(k) = vci(k)*dsigh(k)/weight
c  210       continue
c
c-----Apply layer fluxes to obtain new concentration profile
c
c
	    flux2=0.
            do 210 k = 1,kmax-1
	      flux1=flux2
	      kval=k
              if (vrtvel(k).gt.0.) kval=kval+1
 	      flux2=vci(kval)*vrtvel(k)
	      vci(k)=vci(k)+dtodz(k)*(flux2-flux1)
  210       continue
	    flux1=flux2
c	    flux2=vrtvel(kmax)*vci(kmax)
   	    flux2=0.0
c	    if(vrtvel(kmax) .gt. 0.) flux2=vrtvel(kmax)*ctop
	    vci(kmax)=vci(kmax)+dtodz(kmax)*(flux2-flux1)
c
c------End of sub-step loop; load final results back into conc
c
  160       continue
            do 170 k = 1,kmax
              if (lprnt .and. idiag.ne.0) 
     &          write(*,*)'l,k,conc,vci',l,k,conc(i,j,l,k),vci(k)
              conc(i,j,l,k) = amax1(vci(k),bdnl(l))
  170       continue
c
  150     continue
  120   continue
  100 continue
      return
      end
