c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine update(time,ndate,tdatnx,totime,idebug)
c
c-----ATDM v1.00
c
c     Updates meteorological parameters
c
c-----95/02/22:
c     -from UAM-V Revision 1.4  94/07/27  09:40:19  tcm
c     -removed inputs and code related to the albedo/haze/ozone file
c     -modified code for new PTSRCX and PTSRCY variables (see HEDPTS)
c     -removed call to CONVRT
c-----95/03/31 (v0.02):
c     -modifications made following UAM-V to correct vertical nesting code
c-----95/06/01:
c     -major modifications to code structure
c-----95/09/05:
c     -moved call to FGRTRP for surface temperature from ATDM to this routine
c      just before call to FGRTRP for 3-D temperature
c     -modified all calls to FGRTRP to include the Ts flag and Ts array 
c      (or dummy) in argument list
c-----95/09/19:
c     -modified arguments to FGRTRP to pass staggered wind flags when
c      appropriate
c-----96/08/12:
c     -modified if-then statement in loop 730 to not interp fine grid
c     -from other fine grids.  This is still OK since multiply nested
c     -grids must be defined outside to inside.  The original code may
c     -have accessed zero pointers.
c-----96/09/11:dfg
c     -added that if lmech is true, it skips to read ozone file.
c     -added that if lmech is true, it skips ozone fine grid interpolation.
c     -99/04/22:dfg and 00/01/07: mlg
c     -added block statement to read input o3,oh,no3,h2o2,ho2 from oh_o3 files.
c     -added call to rdfo3oh to read fine grid input o3,oh,no3,h2o2 and ho2
c
c     Input arguments:
c               time:           model time
c               ndate:          model date
c               tdatnx:         time at which to read new inputs
c               totime:         total simulated time
c               idebug:         diagnostic debug flag
c     Files read:
c               iwind:          wind speed components u,v (m/s)
c               ifwind:         fine grid wind speed components u,v (m/s)
c               iprs:           surface pressure (mb)
c               irain:          rainfall rate (inches/hr)
c               idepv:          deposition velocities (m/s)
c               itemp:          temperature (K)
c               ikv:            vertical diffusion coefficient (m**2/s)
c               io3:            ozone concentrations (ppm)
c               ih2o:           water vapor concentration (kg/kg)
c               iclh2o:         cloud water concentration (kg/kg)
c               irnh2o:         rain water concentration (kg/kg)
c     Files written:
c               6:              standard output file unit (*)
c               8:              diagnostic file
c               66:             output message file 
c     Routines called:
c               jtocal    caltoj    rdfwnd    pfage     fgrtrp    rdfprs
c               ffgtrp    gethp     getbc     rdfkv     rdfo3     beh084
c               diffus    diagn     clddiag   clddiagf  partsed
c     Called by:
c               setup     atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'com1f.cmd'
      include 'comuam.cmd'
      include 'oneday.cmd'
      include 'com3.cmd'
      include 'advec.cmd'
      include 'stack.cmd'
      include 'ctlflg.cmd'
      include 'shrfin.cmd'
c
      logical lfirst,ustgr,vstgr
      save lfirst
      dimension ntab(12)
      real zs(mxz), pp(mxz), tt(mxz), ww(mxz)
      character*10 mdum
      character*4 ifile(10)
c
      data ntab/31,28,31,30,31,30,31,31,30,31,30,31/
      data lfirst/.true./
      data eps /0.622/, rd /287./, grav /9.8/
c
c-----Bit 13 (4096's) of idebug turns on debugs in update
c
      idiag = and(idebug,4096)
      knteof=0
      if (ndate .lt. 99999) then
        njdate = ndate
        call jtocal(ndate,ncdate)
      else
        ncdate = ndate
        call caltoj(ndate,njdate)
      endif
      write(*,4000) time,ncdate,njdate
 4000 format(' enter update ',f10.1,i10,'(',i8,')')
c
c=======================================================================
c
c-----Read wind file
c
   10 read(iwind,end=411) tdata,idate
      write(*,5001) tdata,idate,time,ndate
 5001 format(' read iwind  ',f10.1,i10,f10.1,i10)
      do 35 k=1,nlayer
        read(iwind) ((u(i,j,k),i=1,nox),j=1,noy)
        read(iwind) ((v(i,j,k),i=1,nox),j=1,noy)
c
c     convert u & v from m/sec to km/min
c
        do 34 j=1,noy
          do 33 i=1,nox
            u(i,j,k) = u(i,j,k)*.06
            v(i,j,k) = v(i,j,k)*.06
   33     continue
   34   continue
   35 continue
c
c----Read wind files for fine grids also
c
      do 60 ifine=1,numfin
        if (ifwind(ifine) .gt. 0) then
          read(ifwind(ifine),end=411) tfdata,ifdate
          write(*,5010) ifine,tfdata,ifdate,time,ndate
 5010     format(' read ifwind, fine grid ',i3,2x,f10.1,i10,f10.1,i10)
c
c-----Stop if coarse and fine grid wind files are out of synch 
c
          if (ifdate .ne. idate .or. abs(tfdata-tdata) .gt. .1) then
            write(*,'(a,i3)') 'wind out of synch with fine grid ',ifine
            stop
          endif
          call rdfwnd(nfx(ifine),nfy(ifine),nfz(ifine),ifwind(ifine),
     &                uf(1,1,ifine),vf(1,1,ifine))
        endif
   60 continue
c
c-----Advance wind file to proper time and date
c
      if (idate .lt. 99999) then
        ncomp = njdate
      else
        ncomp = ncdate
      endif
      if (l1day) then
        if (abs(time-tdata).gt.1.0) go to 10
      else
        if (idate.lt.ncomp) go to 10
        if (idate.gt.ncomp) go to 110
        if (idate.eq.ncomp.and.(tdata+5.0).lt.time) go to 10
        if (idate.eq.ncomp.and.(time+5.0).lt.tdata) go to 110
      endif
      tdatnx=totime+dtdata
c
c=======================================================================
c
c-----Load new surface pressure into current array
c
      kntmix = 0
      write (*,*) 'lfirst=', lfirst
      if (lfirst) goto 99
  106 continue
      do 98 j = 1,noy
        do 97 i = 1,nox
          psfc(i,j) = psnew(i,j)
          psold(i,j) = psnew(i,j)
   97   continue
   98 continue
      elapsth = 0.0
c
c-----Load new fine grid surface pressure into current array
c
      do 109 ifine = 1,numfin
        call pfage(nfx(ifine),nfy(ifine),psnewf(1,ifine),
     &             psoldf(1,ifine),psfcf(1,ifine))
  109 continue
c
c-----Read surface pressure file
c
   99 read(iprs,end=412) ttim,iidat,((psnew(i,j),i=1,nox),j=1,noy)
      write (*,*) 'in surface pressure file',  ttim, iidat
      if (iidat .lt. 99999) then
        ncomp = njdate
      else
        ncomp = ncdate
      endif
c
c-----Read surface pressure files for fine grids also
c
      do 180 ifine=1,numfin
        if (ifprs(ifine) .gt. 0) then
          call rdfprs(tfdata,ifdate,nfx(ifine),nfy(ifine),
     &                ifprs(ifine),psnewf(1,ifine))
          write(*,5080) ifine,tfdata,ifdate,ttim,iidat
 5080     format(' read fprs, fine grid ',i3,2x,f10.1,i10,f10.1,i10)
c
c-----Stop if coarse and fine grid surface pressure files are out of synch
c
          if (ifdate .ne. iidat .or. abs(tfdata-ttim) .gt. .1) then
            write(*,'(a,i3)') 'prs out of synch with fine grid ',ifine
            stop
          endif
        endif
  180 continue
      write(*,5082) ttim,iidat,time,ndate
 5082 format(' read prs   ',f10.1,i10,f10.1,i10)
c
c-----If necessary, interpolate surface pressure to fine grids
c     
      do 182 ifine=1,numfin
        inttyp = 21
        if (ifprs(ifine) .le. 0) then
          if (ifgptr(ifine) .eq. 0) then
            call fgrtrp(mxx,mxy,nox,noy,1,psnew,
     &                nfx(ifine),nfy(ifine),1,
     &                psnewf(1,ifine),sigmaf(1,ifine),
     &                ifine,inttyp,0,dummy,.false.,.false.,idebug)
          else
            ifget = ifgptr(ifine)
            call ffgtrp(nfx(ifget),nfy(ifget),1,
     &                psnewf(1,ifget),ifget,
     &                nfx(ifine),nfy(ifine),1,
     &                psnewf(1,ifine),sigmaf(1,ifine),
     &                ifine,inttyp,idebug)
          endif
        endif
  182 continue
c
      kntmix=kntmix+1
      if (lfirst.and.kntmix.le.1) then
         write (*,*) 'kntmix=', kntmix
         write (*,*) 'lfirst=', lfirst
         write(*,5082) ttim,iidat,time,ndate
         go to 106
      end if
c
c-----If first time through advance the iprs file to proper date
c     note: psold is at current time and psnew is at next update time 
c           to allow for temporal changes in pressure.
c
      ttim=ttim-dtdata
      if (ttim.ge.0.0) go to 1077
      ttim=ttim+2400.0
      iidat=iidat-1
c
c-----Check if crossing month
c
      if (iidat.ne.((iidat/100)*100)) go to 1077
      if (iidat .lt. 100000) go to 1077
      nyr=iidat/10000
      nmth=iidat/100-nyr*100
      nmth1=nmth-1
      if (nmth1.gt.0) go to 1076
      nmth1=12
      nyr=nyr-1
 1076 continue
      nday=ntab(nmth1)
c
c-----Leap year?
c
      if (nmth1.eq.2.and.(mod(nyr,4).eq.0)) nday=nday+1
      iidat=nyr*10000+nmth1*100+nday
 1077 continue
      if (l1day) then
        if (abs(ttim).lt.1.0.and.abs(time-2400.0).lt.1.0) go to 402
        if (abs(time).lt.1.0.and.abs(ttim-2400.0).lt.1.0) go to 402
        if (abs(time-ttim).gt.1.0) go to 106
      else
        if (lfirst) then
          if (iidat.lt.ncomp) go to 106
          if (iidat.eq.ncomp.and.(ttim+1.0).lt.time) go to 106
        else
          if (iidat.ne.ncomp.or.abs(ttim-time).gt.1.0) go to 121
        endif
      endif
  402 continue
c
c=======================================================================
c
c-----Read temperature file
c
  113 continue
      read(itemp,end=414) ttim,iidat,((tsurf(i,j),i=1,nox),j=1,noy)
      do 1113 k=1,nlayer
        read(itemp,end=414) ttim,iidat,((temper(i,j,k),i=1,nox),j=1,noy)
 1113 continue
      if (iidat .lt. 99999) then
        ncomp = njdate
      else
        ncomp = ncdate
      endif
      write(*,5003) ttim,iidat,time,ndate
 5003 format(' read temp ',f10.1,i10,f10.1,i10)
c
c-----If first time through advance temp file to proper time
c
      if (l1day) then
        if (abs(ttim).lt.1.0.and.abs(time-2400.0).lt.1.0) go to 404
        if (abs(time).lt.1.0.and.abs(ttim-2400.0).lt.1.0) go to 404
        if (abs(ttim-time).gt.1.0) go to 113
      else
        if (lfirst) then
          if (iidat.lt.ncomp) go to 113
          if (iidat.eq.ncomp.and.(ttim+0.1).lt.time) go to 113
        endif
        if (iidat.ne.ncomp.or.abs(time-ttim).gt.1.0) go to 123
      endif
  404 continue
c
c-----Interpolate 2-D surface temperature and 3-D temperature to all fine grids
c
      do 4040 ifine=1,numfin
        inttyp = 21
        call fgrtrp(mxx,mxy,nox,noy,1,tsurf,
     &              nfx(ifine),nfy(ifine),1,
     &              tfsurf(1,ifine),sigmaf(1,ifine),
     &              ifine,inttyp,0,dummy,.false.,.false.,idebug)
        inttyp = 11
        call fgrtrp(mxx,mxy,nox,noy,nlayer,temper,
     &              nfx(ifine),nfy(ifine),nfz(ifine),
     &              temprf(1,1,ifine),sigmaf(1,ifine),
     &              ifine,inttyp,1,tfsurf(1,ifine),.false.,.false.,
     &              idebug)
 4040 continue
c
c=======================================================================
c
c-----Read water vapor concentration file
c
  117 continue
      do 1117 k=1,nlayer
        read(ih2o,end=416) ttim,iidat,((ch2o(i,j,k),i=1,nox),j=1,noy)
 1117 continue
      if (iidat .lt. 99999) then
        ncomp = njdate
      else
        ncomp = ncdate
      endif
      write(*,5005) ttim,iidat,time,ndate
 5005 format(' read water',f10.1,i10,f10.1,i10)
c
c-----If first time through advance temp file to proper time
c
      if (l1day) then
        if (abs(ttim).lt.1.0.and.abs(time-2400.0).lt.1.0) go to 406
        if (abs(time).lt.1.0.and.abs(ttim-2400.0).lt.1.0) go to 406
        if (abs(time-ttim).gt.1.0) go to 117
      else
        if (lfirst) then
          if (iidat.lt.ncomp) go to 117
          if (iidat.eq.ncomp.and.(ttim+0.1).lt.time) go to 117
        endif
        if (iidat.ne.ncomp.or.abs(ttim-time).gt.1.0) go to 129
      endif
  406 continue
c
c-----Interpolate water vapor to all fine grids
c
      write (*,*) 'before fgrtrp'
      inttyp = 11
      do 4060 ifine=1,numfin
        call fgrtrp(mxx,mxy,nox,noy,nlayer,ch2o,
     &              nfx(ifine),nfy(ifine),nfz(ifine),
     &              ch2of(1,1,ifine),sigmaf(1,ifine),
     &              ifine,inttyp,0,dummy,.false.,.false.,idebug)
 4060 continue
c
c=======================================================================
c
c-----Read cloud water concentration file
c
      write (*,*) 'in update and reading cloud water'
 118  continue
      do 1118 k=1,nlayer
c         write (*,*) 'k cloud=', k, iclh2o
        read(iclh2o,end=417) ttim,iidat,((clh2o(i,j,k),i=1,nox),j=1,noy)
 1118   continue
      if (iidat .lt. 99999) then
        ncomp = njdate
      else
        ncomp = ncdate
      endif
      write(*,5031) ttim,iidat,time,ndate
 5031 format(' read cloud water',f10.1,i10,f10.1,i10)
c
c-----If first time through advance temp file to proper time
c
      if (l1day) then
        if (abs(ttim).lt.1.0.and.abs(time-2400.0).lt.1.0) go to 407
        if (abs(time).lt.1.0.and.abs(ttim-2400.0).lt.1.0) go to 407
        if (abs(time-ttim).gt.1.0) go to 118
      else
        if (lfirst) then
          if (iidat.lt.ncomp) go to 118
          if (iidat.eq.ncomp.and.(ttim+0.1).lt.time) go to 118
        endif
        if (iidat.ne.ncomp.or.abs(ttim-time).gt.1.0) go to 330
      endif
 407  continue
c
c-----Interpolate cloud water to all fine grids
c
      inttyp = 11
      do 4061 ifine=1,numfin
        call fgrtrp(mxx,mxy,nox,noy,nlayer,clh2o,
     &              nfx(ifine),nfy(ifine),nfz(ifine),
     &              clh2of(1,1,ifine),sigmaf(1,ifine),
     &              ifine,inttyp,0,dummy,.false.,.false.,idebug)
 4061 continue
c
c=======================================================================
c
c-----Read rain water concentration file
c
 119  continue
      do 1119 k=1,nlayer
        read(irnh2o,end=418) ttim,iidat,((rnh2o(i,j,k),i=1,nox),j=1,noy)
 1119   continue
      if (iidat .lt. 99999) then
        ncomp = njdate
      else
        ncomp = ncdate
      endif
      write(*,5030) ttim,iidat,time,ndate
 5030 format(' read rain water',f10.1,i10,f10.1,i10)
c
c-----If first time through advance temp file to proper time
c
      if (l1day) then
        if (abs(ttim).lt.1.0.and.abs(time-2400.0).lt.1.0) go to 408
        if (abs(time).lt.1.0.and.abs(ttim-2400.0).lt.1.0) go to 408
        if (abs(time-ttim).gt.1.0) go to 119
      else
        if (lfirst) then
          if (iidat.lt.ncomp) go to 119
          if (iidat.eq.ncomp.and.(ttim+0.1).lt.time) go to 119
        endif
        if (iidat.ne.ncomp.or.abs(ttim-time).gt.1.0) go to 331
      endif
 408  continue
c
c-----Interpolate rain water to all fine grids
c
      inttyp = 11
      do 4062 ifine=1,numfin
        call fgrtrp(mxx,mxy,nox,noy,nlayer,rnh2o,
     &              nfx(ifine),nfy(ifine),nfz(ifine),
     &              rnh2of(1,1,ifine),sigmaf(1,ifine),
     &              ifine,inttyp,0,dummy,.false.,.false.,idebug)
 4062 continue
c
c=======================================================================
c
c-----Calculate pressure and layer interface height fields from surface
c     pressure, temperature, and moisture fields for all grids
c
      call gethp(mxx,mxy,mxz,nox,noy,nlayer,pmodtop,psfc,sigma,ch2o,
     &           temper,pstar,press,h)
      do 210 ifine=1,numfin
        call gethp(nfx(ifine),nfy(ifine),nfz(ifine),nfx(ifine),
     &             nfy(ifine),nfz(ifine),pmodtop,
     &             psfcf(1,ifine),sigmaf(1,ifine),ch2of(1,1,ifine),
     &             temprf(1,1,ifine),pstarf(1,ifine),presf(1,1,ifine),
     &             htf(1,1,ifine))
  210 continue
c
c=======================================================================
c
c-----Read rainfall rate file
c
      if (.not.lwet .or. irain.eq.0) go to 608
  108 continue
      read(irain,end=413) rtime,irdate,((rain(i,j),i=1,nox),j=1,noy)
      if (irdate .lt. 99999) then
        ncomp = njdate
      else
        ncomp = ncdate
      endif
c
      write(*,5007) rtime,irdate,time,ndate
 5007 format(' read rainfall ',f10.1,i10,f10.1,i10)
c
c-----If first time through advance rain file to proper time
c
      if (l1day) then
        if (abs(rtime).lt.1.0.and.abs(time-2400.0).lt.1.0) go to 403
        if (abs(time).lt.1.0.and.abs(rtime-2400.0).lt.1.0) go to 403
        if (abs(time-rtime).gt.1.0) go to 108
      else
        if (lfirst) then
          if (irdate.lt.ncomp) go to 108
          if (irdate.eq.ncomp.and.(rtime+0.1).lt.time) go to 108
        endif
        if (ncomp.ne.irdate.or.abs(time-rtime).gt.1.0) go to 128
      endif
  403 continue
c
c-----Interpolate rainfall to all fine grids
c
      inttyp = 21
      do 4030 ifine=1,numfin
        call fgrtrp(mxx,mxy,nox,noy,1,rain,
     &              nfx(ifine),nfy(ifine),1,
     &              rainf(1,ifine),sigmaf(1,ifine),
     &              ifine,inttyp,0,dummy,.false.,.false.,idebug)
 4030 continue
c
c=======================================================================
c
c-----Read deposition velocity file
c
  608 continue
      if (idepv.gt.0) then
        read(idepv,end=613) btime,ibdate
        write(*,5006) btime,ibdate,time,ndate
 5006   format(' read depvel ',f10.1,i10,f10.1,i10)
c
        do 6868 k=1,ndvsp
          ksp = kdvsp(k)
          if (ksp .eq. 0) then
            read(idepv,end=613) ((dum,i=1,nox),j=1,noy)
          else
            read(idepv,end=613) ((depvel(i,j,ksp),i=1,nox),j=1,noy)
          endif
          do 6085 ifine=1,numfin
c        
c-----Interpolate to get fine grid deposition velocities
c
            inttyp = 20
            call fgrtrp(mxx,mxy,nox,noy,nospec,depvel,
     &                    nfx(ifine),nfy(ifine),nospec,
     &                    depvelf(1,1,ifine),sigmaf(1,ifine),
     &                    ifine,inttyp,0,dummy,.false.,.false.,idebug)
            if (idiag .ne. 0) then
              write(*,'(a,i3)')
     &              ' finished fgrtrp depvelf for fg#',ifine
            endif
 6085     continue
 6868   continue
        if (ibdate .lt. 99999) then
          ncomp = njdate
        else
          ncomp = ncdate
        endif
c
c-----If first time through advance depvel file to proper time
c
        if (l1day) then
          if (abs(btime).lt.1.0.and.abs(time-2400.0).lt.1.0) go to 603
          if (abs(time).lt.1.0.and.abs(btime-2400.0).lt.1.0) go to 603
          if (abs(time-btime).gt.1.0) go to 608
        else
          if (lfirst) then
            if (ibdate.lt.ncomp) go to 608
            if (ibdate.eq.ncomp.and.(btime+0.1).lt.time) go to 608
          endif
          if (ncomp.ne.ibdate.or.abs(time-btime).gt.1.0) go to 628
        endif
c
      endif
  603 continue
c
c=======================================================================
c
c-----Read the boundry conditions
c
      call getbc(time,ndate,icbc,ibeof)
      if (ibeof .ne. 0) then
        write(*,*) ' end of file on boundary file.'
        stop
      endif
c
c=======================================================================
c
c-----Read vertical diffusion file
c
  228 continue
      do 1128 k=1,nlayer
        read(ikv,end=429) ttim,iidat,((kv(i,j,k),i=1,nox),j=1,noy)
 1128 continue
c
c-----Read fkv files for fine grids also
c
      do 70 ifine=1,numfin
        if (ifkv(ifine) .gt. 0) then
          call rdfkv(tfdata,ifdate,nfx(ifine),nfy(ifine),nfz(ifine),
     &                ifkv(ifine),fkv(1,1,ifine),htf(1,1,ifine))
          write(*,5020) ifine,tfdata,ifdate,ttim,iidat
 5020     format(' read fkv, fine grid ',i3,2x,f10.1,i10,f10.1,i10)
c
c-----Stop if coarse and fine grid fkv files are out of synch
c
          if (ifdate .ne. iidat .or. abs(tfdata-ttim) .gt. .1) then
            write(*,'(a,i3)') 'kv out of synch with fine grid ',ifine
            stop
          endif
        endif
   70 continue
      if (iidat .lt. 99999) then
        ncomp = njdate
      else
        ncomp = ncdate
      endif
c
c-----Limit kv's to 0.4*layer height (m)
c
      do 1112 k=1,nlayer
        do 1111 j=1,noy
          do 1110 i=1,nox
            zzmx = 0.4*h(i,j,k)
            if (kv(i,j,k).gt.zzmx) kv(i,j,k) = zzmx
 1110     continue
 1111   continue
 1112 continue
      write(*,5004) ttim,iidat,time,ndate
 5004 format(' read kv   ',f10.1,i10,f10.1,i10)
c
c-----If first time through advance vertdifkv file to proper time
c
      if (l1day) then
        if (abs(ttim).lt.1.0.and.abs(time-2400.0).lt.1.0) go to 428
        if (abs(time).lt.1.0.and.abs(ttim-2400.0).lt.1.0) go to 428
        if (abs(ttim-time).gt.1.0) go to 228
      else
        if (lfirst) then
          if (iidat.lt.ncomp) go to 228
          if (iidat.eq.ncomp.and.(ttim+0.1).lt.time) go to 228
        endif
        if (iidat.ne.ncomp.or.abs(time-ttim).gt.1.0) go to 328
      endif
  428 continue
c
c=======================================================================
c
c-----Read ozone concentration file
c
  229 continue
c
c
      if (lo3) then
 710         continue
        read(io3,end=720) ndu1, tmu1, iidat, tmu2
        t2=tmu1*100.0
        if (iidat .lt. 99999) then
          ncomp = njdate
        else
          ncomp = ncdate
        endif
        do k=1,nlayer
          read(io3) idum, (ifile(i),i=1,10),
     &         ((ohcon(i,j,k),i=1,nox),j=1,noy)
        end do
        do k=1,nlayer
          read(io3) idum, (ifile(i),i=1,10),
     &         ((ho2con(i,j,k),i=1,nox),j=1,noy)
        end do
        do k=1,nlayer
          read(io3) idum, (ifile(i),i=1,10),
     &         ((no3con(i,j,k),i=1,nox),j=1,noy)
        end do
        do k=1,nlayer
          read(io3) idum, (ifile(i),i=1,10),
     &         ((aso2con(i,j,k),i=1,nox),j=1,noy)
        end do
        do k=1,nlayer
          read(io3) idum, (ifile(i),i=1,10),
     &         ((hydro(i,j,k),i=1,nox),j=1,noy)
        end do
        do  k=1,nlayer
          read(io3) idum, (ifile(i),i=1,10),
     &         ((ozone(i,j,k),i=1,nox),j=1,noy)
        end do
        if (nspio3 .gt. 6) then
          do  k=1,nlayer
            read(io3) idum, (ifile(i),i=1,10),
     &         ((peccon(i,j,k),i=1,nox),j=1,noy)
          end do
        endif

        if (iidat.lt.ncomp) go to 710
        if (iidat.gt.ncomp) then
          write(*,*) 'Error on date in ozone & OH file'
          stop
        endif
c
        if (io3f.gt.0) then
          read(io3f) tf, ndf
          if (abs(tf-t2) .gt. 1.0 .or. ndf .ne. iidat) then
            write(*,*) 'Time or date error in O3 & OH file '
            write(*,*) 'fine: ',tf,ndf, ' coarse: ',t2,iidat
            stop
          endif
          do ifine=1,numfin
            call rdfo3oh(io3f,nfx(ifine),nfy(ifine),nfz(ifine),
     &                ohconf(1,1,1,ifine),no3conf(1,1,1,ifine),
     &                ho2conf(1,1,1,ifine),
     &                h2o2f(1,1,ifine),ozonef(1,1,ifine),
     &                aso2conf(1,1,1,ifine),pecconf(1,1,1,ifine))
          end do
        endif
        if (abs(time-t2).gt.1.0) go to 710
        go to 450
      endif

c
      if (io3.eq.0) goto 450
      do k=1,nlayer
        read(io3,end=430) ttim,iidat,((ohcon(i,j,k),i=1,nox),j=1,noy)
      end do
      do k=1,nlayer
        read(io3,end=430) ttim,iidat,((ho2con(i,j,k),i=1,nox),j=1,noy)
      end do
      do k=1,nlayer
        read(io3,end=430) ttim,iidat,((no3con(i,j,k),i=1,nox),j=1,noy)
      end do
      do k=1,nlayer
        read(io3,end=430) ttim,iidat,((aso2con(i,j,k),i=1,nox),j=1,noy)
      end do
      do k=1,nlayer
        read(io3,end=430) ttim,iidat,((hydro(i,j,k),i=1,nox),j=1,noy)
      end do
      do k=1,nlayer
        read(io3,end=430) ttim,iidat,((ozone(i,j,k),i=1,nox),j=1,noy)
      end do
      if (nspio3 .gt. 6) then
        do k=1,nlayer
          read(io3,end=430) ttim,iidat,((peccon(i,j,k),i=1,nox),j=1,noy)
        end do
      endif
c
c-----Read ozone files for fine grids also
c
      do 71 ifine=1,numfin
        if (ifo3(ifine) .gt. 0) then
          call rdfo3(tfdata,ifdate,nfx(ifine),nfy(ifine),nfz(ifine),
     &                ifo3(ifine),ohconf(1,1,1,ifine),h2o2f(1,1,ifine),ozonef(1,1,ifine),
     &                no3conf(1,1,1,ifine),
     &                ho2conf(1,1,1,ifine))
          write(*,5021) ifine,tfdata,ifdate,ttim,iidat
 5021     format(' read ozone, fine grid ',i3,2x,f10.1,i10,f10.1,i10)
c
c-----Stop if coarse and fine grid o3 files are out of synch
c
          if (ifdate .ne. iidat .or. abs(tfdata-ttim) .gt. .1) then
            write(*,'(a,i3)') 'O3 out of synch with fine grid ',ifine
            stop
          endif
        endif
   71 continue
      if (iidat .lt. 99999) then
        ncomp = njdate
      else
        ncomp = ncdate
      endif
      write(*,5017) ttim,iidat,time,ndate
 5017 format(' read ozone   ',f10.1,i10,f10.1,i10)
c
c-----If first time through advance ozone file to proper time
c
      if (l1day) then
        if (abs(ttim).lt.1.0.and.abs(time-2400.0).lt.1.0) go to 431
        if (abs(time).lt.1.0.and.abs(ttim-2400.0).lt.1.0) go to 431
        if (abs(ttim-time).gt.1.0) go to 229
      else
        if (lfirst) then
          if (iidat.lt.ncomp) go to 229
          if (iidat.eq.ncomp.and.(ttim+0.1).lt.time) go to 229
        endif
        if (iidat.ne.ncomp.or.abs(time-ttim).gt.1.0) go to 329
      endif
  431 continue
c
c=======================================================================
c
c-----Prepare inputs for, and call plume rise algorithm
c
       write(*,*) 'Just skip to read o3 data'
cdiag
  450 continue
      if (idiag .gt. 0) then
        write(*,*) ' calculate plumerise for all sources'
      endif
 
      do 152 l=1,nopts
        ix = int(ptsrcx(l)) + 1
        jy = int(ptsrcy(l)) + 1
        if (ix.gt.nox) ix = nox
        if (ix.lt.1) ix = 1
        if (jy.gt.noy) jy = noy
        if (jy.lt.1) jy = 1
        if (ix.le.istr(jy)) go to 152
        if (ix.ge.iend(jy)) go to 152
        if (jy.le.jstr(ix)) go to 152
        if (jy.ge.jend(ix)) go to 152
        do 151 kk=1,nlayer
          zs(kk) = h(ix,jy,kk)
          pp(kk) = press(ix,jy,kk)
          tt(kk) = temper(ix,jy,kk)
          if (lstagr) then
            uchk = (u(ix,jy,kk)+u(ix-1,jy,kk))/2.
            vchk = (v(ix,jy,kk)+v(ix,jy-1,kk))/2.
          else
            uchk = u(ix,jy,kk)
            vchk = v(ix,jy,kk)
          endif
          ww(kk) = 16.67*sqrt(uchk**2.+vchk**2.)
  151   continue

        if (idiag .gt. 0) then
          write(*,*) ' Calculate plumerise for source ', l 
          write(*,*) ' at ', ix, jy
          write(*,*) ' Stack params: ', stkht(l), stkd(l), 
     &                  vexit(l), temps(l)
        endif
        call beh084(stkht(l),stkd(l),vexit(l),temps(l),zs,
     &               ww,tt,pp,nlayer,heff(l))
        if (idiag .gt. 0) then
          write(*,*) ' Calculated plume height is ', heff(l)
        endif
        
  152 continue
c
c-----Calculate horizontal diffusivities
c
      if (idiag .gt. 0) then
        write(*,*) ' calculate diffusivities'
      endif
      call diffus
c
c-----If necessary, interpolate winds to fine grids
c     
      do 730 ifine=1,numfin
        inttyp = 10
        if (ifwind(ifine) .le. 0) then
c          if (ifgptr(ifine).eq.0 .or. ifwind(ifgptr(ifine)).eq.0) then
          if (ifgptr(ifine).eq.0 ) then
            ustgr = .false.
            if (lstagr) ustgr = .true.
            call fgrtrp(mxx,mxy,nox,noy,nlayer,u,
     &                  nfx(ifine),nfy(ifine),nfz(ifine),
     &                  uf(1,1,ifine),sigmaf(1,ifine),
     &                  ifine,inttyp,0,dummy,ustgr,.false.,idebug)
            vstgr = .false.
            if (lstagr) vstgr = .true.
            call fgrtrp(mxx,mxy,nox,noy,nlayer,v,
     &                  nfx(ifine),nfy(ifine),nfz(ifine),
     &                  vf(1,1,ifine),sigmaf(1,ifine),
     &                  ifine,inttyp,0,dummy,.false.,vstgr,idebug)
          else
            ifget = ifgptr(ifine)
            call ffgtrp(nfx(ifget),nfy(ifget),nfz(ifget),
     &                  uf(1,1,ifget),ifget,
     &                  nfx(ifine),nfy(ifine),nfz(ifine),
     &                  uf(1,1,ifine),sigmaf(1,ifine),
     &                  ifine,inttyp,idebug)
            call ffgtrp(nfx(ifget),nfy(ifget),nfz(ifget),
     &                  vf(1,1,ifget),ifget,
     &                  nfx(ifine),nfy(ifine),nfz(ifine),
     &                  vf(1,1,ifine),sigmaf(1,ifine),
     &                  ifine,inttyp,idebug)
          endif
        endif
  730 continue
c
c-----If necessary, interpolate kv's to fine grids
c     
      do 740 ifine=1,numfin
        inttyp = 0
        if (ifkv(ifine) .le. 0) then
          if (ifgptr(ifine) .eq. 0) then
            call fgrtrp(mxx,mxy,nox,noy,nlayer,kv,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                fkv(1,1,ifine),sigmaf(1,ifine), 
     &                ifine,inttyp,0,dummy,.false.,.false.,idebug)
          else
            ifget = ifgptr(ifine)
            call ffgtrp(nfx(ifget),nfy(ifget),nfz(ifget),
     &                fkv(1,1,ifget),ifget,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                fkv(1,1,ifine),sigmaf(1,ifine),
     &                ifine,inttyp,idebug)
          endif
        endif
  740 continue
c
c*** if lmech is true, skip the ozone fine grid intepolation.
c  
       if (lmech) go to 7411
       if (lo3) go to 7411
c
c-----If necessary, interpolate ozone to fine grids
c     
      do 742 ifine=1,numfin
        inttyp = 11
        if (ifo3(ifine) .le. 0) then
          if (ifgptr(ifine) .eq. 0) then
            call fgrtrp(mxx,mxy,nox,noy,nlayer,no3con,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                no3conf(1,1,1,ifine),sigmaf(1,ifine), 
     &                ifine,inttyp,0,dummy,.false.,.false.,idebug)
          else
            ifget = ifgptr(ifine)
            call ffgtrp(nfx(ifget),nfy(ifget),nfz(ifget),
     &                no3conf(1,1,1,ifget),ifget,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                no3conf(1,1,1,ifine),sigmaf(1,ifine),
     &                ifine,inttyp,idebug)
          endif
        endif
 742  continue
c
      do 744 ifine=1,numfin
        inttyp = 11
        if (ifo3(ifine) .le. 0) then
          if (ifgptr(ifine) .eq. 0) then
            call fgrtrp(mxx,mxy,nox,noy,nlayer,ho2con,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                ho2conf(1,1,1,ifine),sigmaf(1,ifine), 
     &                ifine,inttyp,0,dummy,.false.,.false.,idebug)
          else
            ifget = ifgptr(ifine)
            call ffgtrp(nfx(ifget),nfy(ifget),nfz(ifget),
     &                ho2conf(1,1,1,ifget),ifget,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                ho2conf(1,1,1,ifine),sigmaf(1,ifine),
     &                ifine,inttyp,idebug)
          endif
        endif
 744  continue
c     
      do 745 ifine=1,numfin
        inttyp = 11
        if (ifo3(ifine) .le. 0) then
          if (ifgptr(ifine) .eq. 0) then
            call fgrtrp(mxx,mxy,nox,noy,nlayer,ohcon,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                ohconf(1,1,1,ifine),sigmaf(1,ifine), 
     &                ifine,inttyp,0,dummy,.false.,.false.,idebug)
          else
            ifget = ifgptr(ifine)
            call ffgtrp(nfx(ifget),nfy(ifget),nfz(ifget),
     &                ohconf(1,1,1,ifget),ifget,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                ohconf(1,1,1,ifine),sigmaf(1,ifine),
     &                ifine,inttyp,idebug)
          endif
        endif
 745  continue
c     
      do 743 ifine=1,numfin
        inttyp = 11
        if (ifo3(ifine) .le. 0) then
          if (ifgptr(ifine) .eq. 0) then
            call fgrtrp(mxx,mxy,nox,noy,nlayer,h2o2,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                h2o2f(1,1,ifine),sigmaf(1,ifine), 
     &                ifine,inttyp,0,dummy,.false.,.false.,idebug)
          else
            ifget = ifgptr(ifine)
            call ffgtrp(nfx(ifget),nfy(ifget),nfz(ifget),
     &                h2o2f(1,1,ifget),ifget,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                h2o2f(1,1,ifine),sigmaf(1,ifine),
     &                ifine,inttyp,idebug)
          endif
        endif
 743  continue
c
      do 741 ifine=1,numfin
        inttyp = 11
        if (ifo3(ifine) .le. 0) then
          if (ifgptr(ifine) .eq. 0) then
            call fgrtrp(mxx,mxy,nox,noy,nlayer,ozone,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                ozonef(1,1,ifine),sigmaf(1,ifine), 
     &                ifine,inttyp,0,dummy,.false.,.false.,idebug)
          else
            ifget = ifgptr(ifine)
            call ffgtrp(nfx(ifget),nfy(ifget),nfz(ifget),
     &                ozonef(1,1,ifget),ifget,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                ozonef(1,1,ifine),sigmaf(1,ifine),
     &                ifine,inttyp,idebug)
          endif
        endif
  741 continue
c
      do ifine=1,numfin
        inttyp = 11
        if (ifo3(ifine) .le. 0) then
          if (ifgptr(ifine) .eq. 0) then
            call fgrtrp(mxx,mxy,nox,noy,nlayer,aso2con,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                aso2conf(1,1,1,ifine),sigmaf(1,ifine), 
     &                ifine,inttyp,0,dummy,.false.,.false.,idebug)
          else
            ifget = ifgptr(ifine)
            call ffgtrp(nfx(ifget),nfy(ifget),nfz(ifget),
     &                aso2conf(1,1,1,ifget),ifget,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                aso2conf(1,1,1,ifine),sigmaf(1,ifine),
     &                ifine,inttyp,idebug)
          endif
        endif
      enddo
c
      do ifine=1,numfin
        inttyp = 11
        if (ifo3(ifine) .le. 0) then
          if (ifgptr(ifine) .eq. 0) then
            call fgrtrp(mxx,mxy,nox,noy,nlayer,peccon,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                pecconf(1,1,1,ifine),sigmaf(1,ifine), 
     &                ifine,inttyp,0,dummy,.false.,.false.,idebug)
          else
            ifget = ifgptr(ifine)
            call ffgtrp(nfx(ifget),nfy(ifget),nfz(ifget),
     &                pecconf(1,1,1,ifget),ifget,
     &                nfx(ifine),nfy(ifine),nfz(ifine),
     &                pecconf(1,1,1,ifine),sigmaf(1,ifine),
     &                ifine,inttyp,idebug)
          endif
        endif
      enddo
c
 7411 continue
c
cdiag
c       write(*,*) 'just skip ozone fine grid interpolation'
c-----Diagnose cloud sub-model parameters for coarse and all fine
c     grids
c
      if (lpmchm .or. ltxchm) then
        call clddiag(time,ndate,idebug)
        if ( idiag .ne. 0 ) then
          write(*,*) ' Finished clddiag ...'
          call flush(6)
        endif
c
        do 750 ifine=1,numfin
        call clddiagf(nfx(ifine),nfy(ifine),nfz(ifine),lwet,
     &                delfx(1,ifine),delfy(ifine),
     &                uf(1,1,ifine),vf(1,1,ifine),
     &                htf(1,1,ifine),presf(1,1,ifine),temprf(1,1,ifine),
     &                rainf(1,ifine),ch2of(1,1,ifine),clh2of(1,1,ifine),
     &                rnh2of(1,1,ifine),
     &                sigmaf(1,ifine),
     &                pstarf(1,ifine),psfcf(1,ifine),tphioaf(1,ifine),
     &                cloudf(1,1,ifine),xmdf(1,1,ifine),kcldbf(1,ifine),
     &                kcldtf(1,ifine),kfrzf(1,ifine),fconvf(1,ifine),
     &                fstrtf(1,ifine),wratf(1,1,ifine),idebug,
     &                ffnonpr(1,1,ifine))
        if ( idiag .ne. 0 ) then
          write(*,*) ' Finished clddiagf ... ifine :',ifine
          call flush(6)
        endif
  750   continue
      endif
c
c-----Estimate the settling velocity for each cell on all grids
c
      if ((lfirst.and..not.lsed) .or. lsed) then
        call partsed(mxx,mxy,mxz,nox,noy,nlayer,nospec,temper,press,
     &               ch2o,vg)
        do 800 ifine = 1,numfin
           call partsed(nfx(ifine),nfy(ifine),nfz(ifine),nfx(ifine),
     &                  nfy(ifine),nfz(ifine),nospec,
     &                  temprf(1,1,ifine),presf(1,1,ifine),
     &                  ch2of(1,1,ifine),vgf(1,1,1,ifine))
  800   continue
      endif
      lfirst = .false.
c
c-----Perform diagnostics
c
      if (idiag .gt. 0) then
        write(*,*) ' call diagn'
      endif
      call diagn(time,ndate,2,0.,0.,0,0)
c
      return
c
c-----If time or date of data is in error, print message and stop
c
  110 write(66,120) time,ndate,tdata,idate
  120 format('1**ERROR:  Data for',f6.0,' on',i8,' is labeled',
     &        f6.0,'on',i8)
      stop21
  123 write(66,124) time,ndate,ttim,iidat
  124 format(' **ERROR:  temperature data for',f10.1,i10,
     &        ' is labeled ',f10.1,i10)
      stop22
  129 write(66,130) time,ndate,ttim,iidat
  130 format(' **ERROR:  water data for',f10.1,i10,' is labeled ',
     &        f10.1,i10)
      stop24
  121 write(66,122) time,ndate,ttim,iidat
  122 format(' **ERROR:  iprs data for',f10.1,i10,' is labeled ',
     &        f10.1,i10)
      stop25
  628 write(66,227) time,ndate,btime,ibdate
  227 format(' **ERROR:  depvel data for',f10.1,i10,' is labeled ',
     &        f10.1,i10)
      stop28
  128 write(66,127) time,ndate,rtime,irdate
  127 format(' **ERROR:  rain data for',f10.1,i10,' is labeled ',
     &        f10.1,i10)
      stop26
  328 write(66,138) time,ndate,ttim,iidat
  138 format(' **ERROR:  kv data for',f10.1,i10,' is labeled ',
     &        f10.1,i10)
      stop27
  329 write(66,139) time,ndate,ttim,iidat
  139 format(' **ERROR:  O3 data for',f10.1,i10,' is labeled ',
     &        f10.1,i10)
      stop29
 330  write(66,140) time,ndate,ttim,iidat
 140  format(' **ERROR:  Cloud water data for',f10.1,i10,' is labeled ',
     &        f10.1,i10)
      stop30
 331  write(66,141) time,ndate,ttim,iidat
 141  format(' **ERROR:  Rain water data for',f10.1,i10,' is labeled ',
     &        f10.1,i10)
      stop31
  411 continue
      if (.not.l1day) then
        write(6,5011) time,ndate,tdata,idate
 5011   format(' ERROR eof on wind ',2(f10.1,i10))
        stop4
      endif
      knteof=knteof+1
      if (knteof.gt.10) then
        write(6,6011) time,ndate,tdata,idate
 6011   format(' eof wind one day error ',2(f10.1,i10))
        stop3
      endif
      rewind iwind
      go to 10
  412 continue
      if (.not.l1day) then
        write(6,5012) time,ndate,ttim,iidat
 5012   format(' ERROR eof on iprs ',2(f10.1,i10))
        stop4
      endif
      knteof=knteof+1
      if (knteof.gt.10) then
        write(6,6012) time,ndate,ttim,iidat
 6012   format(' eof iprs one day error ',2(f10.1,i10))
        stop3
      endif
      rewind iprs
      go to 106
  413 continue
      if (.not.l1day) then
        write(6,5013) time,ndate,rtime,irdate
 5013   format(' ERROR eof on rain ',2(f10.1,i10))
        stop4
      endif
      knteof=knteof+1
      if (knteof.gt.10) then
        write(6,6013) time,ndate,rtime,irdate
 6013   format(' eof rain one day error ',2(f10.1,i10))
        stop3
      endif
      rewind irain
      go to 108
  613 continue
      if (.not.l1day) then
        write(6,5613) time,ndate,btime,ibdate
 5613   format(' ERROR eof on depvel ',2(f10.1,i10))
        stop4
      endif
      knteof=knteof+1
      if (knteof.gt.10) then
        write(6,6513) time,ndate,btime,ibdate
 6513   format(' eof depvel one day error ',2(f10.1,i10))
        stop3
      endif
      rewind idepv
      go to 608
  414 continue
      if (.not.l1day) then
        write(6,5014) time,ndate,ttim,iidat
 5014   format(' ERROR eof on temper ',2(f10.1,i10))
        stop4
      endif
      knteof=knteof+1
      if (knteof.gt.10) then
        write(6,6014) time,ndate,ttim,iidat
 6014   format(' eof temper one day error ',2(f10.1,i10))
        stop3
      endif
      rewind itemp
      go to 113
 416  continue
      if (.not.l1day) then
        write(6,5016) time,ndate,ttim,iidat
 5016   format(' ERROR eof on water ',2(f10.1,i10))
        stop4
      endif
      knteof=knteof+1
      if (knteof.gt.10) then
        write(6,6016) time,ndate,ttim,iidat
 6016   format(' eof water one day error ',2(f10.1,i10))
        stop3
      endif
      rewind ih2o
      go to 117
 417  continue
      if (.not.l1day) then
        write(6,5116) time,ndate,ttim,iidat
 5116   format(' ERROR eof on cloud water ',2(f10.1,i10))
        stop4
      endif
      knteof=knteof+1
      if (knteof.gt.10) then
        write(6,7116) time,ndate,ttim,iidat
 7116   format(' eof cloud water one day error ',2(f10.1,i10))
        stop3
      endif
      rewind iclh2o
      go to 118
 418  continue
      if (.not.l1day) then
        write(6,5117) time,ndate,ttim,iidat
 5117   format(' ERROR eof on rain water ',2(f10.1,i10))
        stop4
      endif
      knteof=knteof+1
      if (knteof.gt.10) then
        write(6,7117) time,ndate,ttim,iidat
 7117   format(' eof rain water one day error ',2(f10.1,i10))
        stop3
      endif
      rewind irnh2o
      go to 119
 429  continue
      if (.not.l1day) then
        write(6,5028) time,ndate,ttim,iidat
 5028   format(' ERROR eof on kv ',2(f10.1,i10))
        stop4
      endif
      knteof=knteof+1
      if (knteof.gt.10) then
        write(6,6028) time,ndate,ttim,iidat
 6028   format(' eof kv one day error ',2(f10.1,i10))
        stop3
      endif
      rewind ikv
      go to 228
  430 continue
      if (.not.l1day) then
        write(6,5029) time,ndate,ttim,iidat
 5029   format(' ERROR eof on ozone ',2(f10.1,i10))
        stop4
      endif
      knteof=knteof+1
      if (knteof.gt.10) then
        write(6,6029) time,ndate,ttim,iidat
 6029   format(' eof ozone one day error ',2(f10.1,i10))
        stop3
      endif
  720  write(*,*) 'Error: end of O3 & OH file'
       stop
       if (.not.lo3) rewind io3
      go to 229
      end



