c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine xmsflo(mdx,mdy,mdz,mdspc,nxg,nyg,nzg,delx,dely,cnca,
     &                  ua,va,delt,ifine)
c
c-----ATDM v1.00
c
c     Calculates mass flux across region boundaries based on coarse
c     grid variables
c
c-----95/06/09:
c     -from UAM-V Revision 1.2  94/02/10  21:03:06  tcm
c     -modified calculation of xbcflo variables for ppm rather than umol/m3
c-----95/06/22:
c     -removed etaa from argument list
c-----95/06/23:
c     -revised mfac and vfac for use with sigma rather than height
c     -removed hta from argument list
c     -added data statements for grav and amolwt
c
c     Input arguments:
c               mdx:            maximum number of grid cells in x-direction
c               mdy:            maximum number of grid cells in y-direction
c               mdz:            maximum number of layers
c               mdspc:          maximum number of species
c               nxg:            number of grid cells in x-direction
c               nyg:            number of grid cells in y-direction
c               nzg:            number of layers
c               delx:           cell size in x-direction (km)
c               dely:           cell size in y-direction (km)
c               cnca:           array of grid concentrations (ppm)
c               ua:             array of u-component wind speed (km/min)
c               va:             array of v-component wind speed (km/min)
c               delt:           time step (min)
c               ifine:          fine grid index
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'balanc.cmd'
      include 'advec.cmd'
c
      dimension cnca(mdx,mdy,mdspc,mdz),
     &          ua(mdx,mdy,mdz), va(mdx,mdy,mdz)
      dimension delx(mdy)
      real mfac
      data grav /9.8/, amolwt /28.8/
c
c  get start and end cells in coarse grid of this fine grid
c
      ix1 = ixfb(ifine)
      ix2 = ixfe(ifine)
      jy1 = jyfb(ifine)
      jy2 = jyfe(ifine)
c
c total fluxes across lateral boundaries for all layers
c
      do 200 k=1,nzg
        dsigma = sigma(k) - sigma(k+1)
        do 150 j=jy1,jy2
c
c       calculate boundary inflow and outflow for west bnd
c
          i = ix1
          bfac = 1.e6*delt*dely*dsigma
          if (lstagr) then
            wb = ua(i-1,j,k)
          else
            wb = .5*(ua(i-1,j,k)+ua(i,j,k))
          endif
          do 120 l=1,nospec
            if (wb .gt. 0.) then
              mfac = pstar(i-1,j)/(10.*grav*amolwt)
              cc = cnca(i-1,j,l,k)
              xbcflo(l,1,4,ifine) = xbcflo(l,1,4,ifine) + 
     &                              wb*cc*bfac*mfac
            else
              mfac = pstar(i,j)/(10.*grav*amolwt)
              cc = cnca(i,j,l,k)
              xbcflo(l,2,4,ifine) = xbcflo(l,2,4,ifine) - 
     &                              wb*cc*bfac*mfac
            endif
  120     continue
c
c             calculate boundary inflow and outflow for east bnd
c
          i = ix2
          bfac = 1.e6*delt*dely*dsigma
          if (lstagr) then
            wb = ua(i,j,k)
          else
            wb = .5*(ua(i,j,k)+ua(i+1,j,k))
          endif
          do 140 l=1,nospec
            if (wb .lt. 0.) then
              mfac = pstar(i+1,j)/(10.*grav*amolwt)
              cc = cnca(i+1,j,l,k)
              xbcflo(l,1,3,ifine) = xbcflo(l,1,3,ifine) - 
     &                              wb*cc*bfac*mfac
            else
              mfac = pstar(i,j)/(10.*grav*amolwt)
              cc = cnca(i,j,l,k)
              xbcflo(l,2,3,ifine) = xbcflo(l,2,3,ifine) + 
     &                              wb*cc*bfac*mfac
            endif
  140     continue
  150   continue
c
c             calculate boundary inflow and outflow for south bnd
c
        do 180 i=ix1,ix2
          j = jy1
          bfac = 1.e6*delt*delx(j)*dsigma
          if (lstagr) then
            wb = va(i,j-1,k)
          else
            wb = .5*(va(i,j,k)+va(i,j-1,k))
          endif
          do 160 l=1,nospec
            if (wb .gt. 0.) then
              mfac = pstar(i,j-1)/(10.*grav*amolwt)
              cc = cnca(i,j-1,l,k)
              xbcflo(l,1,2,ifine) = xbcflo(l,1,2,ifine) + 
     &                              wb*cc*bfac*mfac
            else
              mfac = pstar(i,j)/(10.*grav*amolwt)
              cc = cnca(i,j,l,k)
              xbcflo(l,2,2,ifine) = xbcflo(l,2,2,ifine) - 
     &                              wb*cc*bfac*mfac
            endif
  160     continue
c
c       calculate boundary inflow and outflow for north bnd
c
          j = jy2
          bfac = 1.e6*delt*delx(j)*dsigma
          if (lstagr) then
            wb = va(i,j,k)
          else
            wb = .5*(va(i,j,k)+va(i,j+1,k))
          endif
          do 170 l=1,nospec
            if (wb .lt. 0.) then
              mfac = pstar(i,j+1)/(10.*grav*amolwt)
              cc = cnca(i,j+1,l,k)
              xbcflo(l,1,1,ifine) = xbcflo(l,1,1,ifine) - 
     &                              wb*cc*bfac*mfac
            else
              mfac = pstar(i,j)/(10.*grav*amolwt)
              cc = cnca(i,j,l,k)
              xbcflo(l,2,1,ifine) = xbcflo(l,2,1,ifine) + 
     &                              wb*cc*bfac*mfac
            endif
  170     continue
  180   continue
  200 continue
c
c  calculate flow through top
c  NOTE: vertical wind at top = 0, xtpflo set to 0
c
c     vf1 = delt*dely*1000.
c     do 300 j=jy1,jy2
c       vf2 = vf1*delx(j)
c       do 290 i=ix1,ix2
          do 280 l=1,nospec
c           vfac = -etaa(i,j,nzg)*vf2
c           if (vfac .ge. 0.) then
c             xtpflo(l,2,ifine) = xtpflo(l,2,ifine) 
c    &                            + vfac*cnca(i,j,l,nzg)
              xtpflo(l,2,ifine) = 0.
c           else
c             xtpflo(l,1,ifine) = xtpflo(l,1,ifine) - vfac*caloft(l)
              xtpflo(l,1,ifine) = 0.
c           endif
  280     continue
c 290   continue
c 300 continue
c
      return
c
      end
